@@ -25,7 +25,6 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <net/if.h>
-#include <fnmatch.h>
 
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #include <sys/sysctl.h>
@@ -34,13 +33,49 @@
 
 extern ref<YPixmap> taskbackPixmap;
 
-static NetDevice* getNetDevice(cstring netdev)
+class NetDevice : public INetDevice {
+public:
+    NetDevice(mstring netdev) : fDevName(netdev) {}
+    const char* getPhoneNumber() override { return ""; }
+    mstring name() const { return fDevName; }
+protected:
+    mstring fDevName;
+};
+
+class NetLinuxDevice : public NetDevice {
+public:
+    NetLinuxDevice(mstring netdev) : NetDevice(netdev) {}
+    virtual bool isUp();
+    virtual void getCurrent(netbytes *in, netbytes *out, const void* sharedData);
+};
+
+class NetFreeDevice : public NetDevice {
+public:
+    NetFreeDevice(mstring netdev) : NetDevice(netdev) {}
+    virtual bool isUp();
+    virtual void getCurrent(netbytes *in, netbytes *out, const void* sharedData);
+};
+
+class NetOpenDevice : public NetDevice {
+public:
+    NetOpenDevice(mstring netdev) : NetDevice(netdev) {}
+    virtual bool isUp();
+    virtual void getCurrent(netbytes *in, netbytes *out, const void* sharedData);
+};
+
+class NetDummyDevice : public NetDevice {
+public:
+    NetDummyDevice(mstring netdev) : NetDevice(netdev) {}
+    virtual bool isUp() { return false; }
+    virtual void getCurrent(netbytes *in, netbytes *out, const void* sharedData)
+    { }
+};
+
+static NetDevice* getNetDevice(mstring netdev)
 {
     return
 #if defined(__linux__)
-        netdev.m_str().startsWith("ippp")
-            ? (NetDevice *) new NetIsdnDevice(netdev)
-            : (NetDevice *) new NetLinuxDevice(netdev)
+        new NetLinuxDevice(netdev)
 #elif defined(__FreeBSD__)
         new NetFreeDevice(netdev)
 #elif defined(__OpenBSD__) || defined(__NetBSD__)
@@ -52,7 +87,7 @@ static NetDevice* getNetDevice(cstring netdev)
 }
 
 NetStatus::NetStatus(
-    cstring netdev,
+    mstring netdev,
     NetStatusHandler* handler,
     YWindow *aParent):
     IApplet(this, aParent),
@@ -73,7 +108,6 @@ NetStatus::NetStatus(
     statusUpdateCount(0),
     unchanged(0),
     wasUp(false),
-    useIsdn(netdev.m_str().startsWith("ippp")),
     fDevName(netdev),
     fDevice(getNetDevice(netdev))
 {
@@ -86,12 +120,12 @@ NetStatus::NetStatus(
 
     setSize(taskBarNetSamples, taskBarGraphHeight);
 
-    getCurrent(0, 0, 0);
-    updateStatus(0);
+    getCurrent(nullptr, nullptr, nullptr);
+    updateStatus(nullptr);
     if (isUp()) {
         updateVisible(true);
     }
-    setTitle(cstring("NET-" + netdev));
+    setTitle("NET-" + netdev);
     updateToolTip();
 }
 
@@ -144,8 +178,8 @@ void NetStatus::updateToolTip() {
     char status[400];
 
     if (isUp()) {
-        char const * const sizeUnits[] = { "B", "KiB", "MiB", "GiB", "TiB", NULL };
-        char const * const rateUnits[] = { "B/s", "kB/s", "MB/s", NULL };
+        char const * const sizeUnits[] = { "B", "KiB", "MiB", "GiB", "TiB", nullptr };
+        char const * const rateUnits[] = { "B/s", "kB/s", "MB/s", nullptr };
 
         long const period(long(toDouble(monotime() - start_time)));
 
@@ -342,49 +376,6 @@ void NetStatus::draw(Graphics &g) {
     }
 }
 
-/**
- * Check isdnstatus, by parsing /dev/isdninfo.
- *
- * Need read-access on /dev/isdninfo.
- */
-#ifdef __linux__
-bool NetIsdnDevice::isUp() {
-    csmart str(load_text_file("/dev/isdninfo"));
-    char val[5][32];
-    int busage = 0;
-    int bflags = 0;
-
-    for (char* p = str; p && *p && p[1]; p = strchr(p, '\n')) {
-        p += strspn(p, " \n");
-        if (strncmp(p, "flags:", 6) == 0) {
-            sscanf(p, "%s %s %s %s %s", val[0], val[1], val[2], val[3], val[4]);
-            for (int i = 0; i < 4; i++) {
-                if (strcmp(val[i+1], "0") != 0)
-                    bflags |= (1 << i);
-            }
-        }
-        else if (strncmp(p, "usage:", 6) == 0) {
-            sscanf(p, "%s %s %s %s %s", val[0], val[1], val[2], val[3], val[4]);
-            for (int i = 0; i < 4; i++) {
-                if (strcmp(val[i+1], "0") != 0)
-                    busage |= (1 << i);
-            }
-        }
-        else if (strncmp(p, "phone:", 6) == 0) {
-            sscanf(p, "%s %s %s %s %s", val[0], val[1], val[2], val[3], val[4]);
-            for (int i = 0; i < 4; i++) {
-                if (strncmp(val[i+1], "?", 1) != 0)
-                    strlcpy(phoneNumber, val[i + 1], sizeof phoneNumber);
-            }
-        }
-    }
-
-    //msg("dbs: flags %d usage %d", bflags, busage);
-
-    return bflags && busage;
-}
-#endif // ifdef __linux__
-
 #if defined (__NetBSD__) || defined (__OpenBSD__)
 bool NetOpenDevice::isUp() {
     bool up = false;
@@ -582,35 +573,13 @@ void NetStatus::getCurrent(long *in, long *out, const void* sharedData) {
 
 NetStatusControl::~NetStatusControl() {
     for (int i = 0; i < fNetStatus.getCount(); ++i) {
-        NetStatus* status = 0;
+        NetStatus* status = nullptr;
         swap(status, fNetStatus[i].value);
         if (status)
             delete status;
     }
 }
 
-#ifdef __linux__
-void NetStatusControl::fetchSystemData() {
-    devStats.clear();
-    devicesText = load_text_file("/proc/net/dev");
-    if (devicesText == 0)
-        return;
-
-    for (char* p = devicesText; (p = strchr(p, '\n')) != 0; ) {
-        *p = 0;
-        while (*++p == ' ');
-        char* name = p;
-        p += strcspn(p, " :|\n");
-        if (*p == ':') {
-            *p = 0;
-            while (*++p && *p == ' ');
-            if (*p && *p != '\n')
-                devStats.append(netpair(name, p));
-        }
-    }
-}
-#endif
-
 NetStatusControl::NetStatusControl(IApp* app, YSMListener* smActionListener,
         IAppletContainer* taskBar, YWindow* aParent) :
     smActionListener(smActionListener),
@@ -622,7 +591,7 @@ NetStatusControl::NetStatusControl(IApp* app, YSMListener* smActionListener,
     while (devList.splitall(' ', &devName, &devList)) {
         if (devName.isEmpty())
             continue;
-        cstring devStr(devName);
+        mstring devStr(devName);
 
         if (strpbrk(devStr, "*?[]\\.")) {
             if (interfaces.isEmpty())
@@ -631,7 +600,7 @@ NetStatusControl::NetStatusControl(IApp* app, YSMListener* smActionListener,
             while (++iter) {
                 if (fNetStatus.has(iter))
                     continue;
-                if (fnmatch(devStr, iter, 0) == 0) {
+                if (upath(iter).fnMatch(devStr) == 0) {
                     createNetStatus(*iter);
                 }
             }
@@ -650,9 +619,9 @@ NetStatusControl::NetStatusControl(IApp* app, YSMListener* smActionListener,
     fUpdateTimer->setTimer(taskBarNetDelay, this, true);
 }
 
-NetStatus* NetStatusControl::createNetStatus(cstring netdev) {
+NetStatus* NetStatusControl::createNetStatus(mstring netdev) {
     NetStatus*& status = fNetStatus[netdev];
-    if (status == 0)
+    if (status == nullptr)
         status = new NetStatus(netdev, this, aParent);
     return status;
 }
@@ -663,7 +632,7 @@ void NetStatusControl::getInterfaces(YStringArray& names)
     unsigned const stop(99);
 
     struct if_nameindex* ifs = if_nameindex(), *i = ifs;
-    for (; i && i->if_index && i->if_name && i - ifs < stop; ++i)
+    for (; i && i->if_index && i->if_name && unsigned(i - ifs) < stop; ++i)
         names.append(i->if_name);
     if (ifs)
         if_freenameindex(ifs);
@@ -699,7 +668,7 @@ void NetStatusControl::relayout()
     taskBar->relayout();
 }
 
-void NetStatusControl::handleClick(const XButtonEvent &up, cstring netdev)
+void NetStatusControl::handleClick(const XButtonEvent &up, mstring netdev)
 {
     if (up.button == Button3) {
         interfaces.clear();
@@ -708,6 +677,7 @@ void NetStatusControl::handleClick(const XButtonEvent &up, cstring netdev)
         fMenu = new YMenu();
         fMenu->setActionListener(this);
         fMenu->addItem(_("NET"), -2, null, actionNull)->setEnabled(false);
+        fMenu->addSeparator();
         YStringArray::IterType iter = interfaces.iterator();
         while (++iter) {
             bool enable = true;
@@ -725,7 +695,7 @@ void NetStatusControl::handleClick(const XButtonEvent &up, cstring netdev)
             item->setChecked(visible);
             item->setEnabled(enable);
         }
-        fMenu->popup(0, 0, 0, up.x_root, up.y_root,
+        fMenu->popup(nullptr, nullptr, nullptr, up.x_root, up.y_root,
                      YPopupWindow::pfCanFlipVertical |
                      YPopupWindow::pfCanFlipHorizontal |
                      YPopupWindow::pfPopupMenu);
@@ -748,56 +718,65 @@ void NetStatusControl::actionPerformed(YAction action, unsigned int modifiers) {
             relayout();
         }
     }
-    fMenu = 0;
+    fMenu = nullptr;
     interfaces.clear();
 }
 
 #ifdef __linux__
 void NetStatusControl::linuxUpdate() {
-    fetchSystemData();
-
+    auto devicesText = filereader("/proc/net/dev").read_all();
+    if (devicesText == nullptr)
+        return;
     int const count(fNetStatus.getCount());
     bool covered[count];
-    for (int i = 0; i < count; ++i)
-        covered[i] = fNetStatus[i] == 0;
-
+    for (int i = 0; i < count; ++i) {
+        covered[i] = (0 == fNetStatus[i]);
+    }
+    using netpair = pair<const char *, const char *>;
     YArray<netpair> pending;
-
-    for (IterStats stat = devStats.iterator(); ++stat; ) {
+    auto checkIfInput = [&](const char* name, const char* data) {
         int index;
-        if (fNetStatus.find((*stat).name(), &index)) {
+        if (fNetStatus.find(name, &index)) {
             if (index < count && fNetStatus[index]) {
                 if (covered[index] == false) {
-                    fNetStatus[index]->timedUpdate((*stat).data());
+                    fNetStatus[index]->timedUpdate(data);
                     covered[index] = true;
                 }
             }
         }
         else {
             // oh, we got a new device? allowed?
-            pending.append(stat);
+            pending.append(netpair(name, data));
+        }
+    };
+
+    for (char* p = devicesText; (p = strchr(p, '\n')) != nullptr; ) {
+        *p = 0;
+        while (*++p == ' ');
+        char* name = p;
+        p += strcspn(p, " :|\n");
+        if (*p == ':') {
+            *p = 0;
+            while (*++p && *p == ' ');
+            if (*p && *p != '\n')
+                checkIfInput(name, p);
         }
     }
-
     // mark disappeared devices as down without additional ioctls
-    for (int i = 0; i < count; ++i)
+    for (int i = 0; i < count; ++i) {
         if (covered[i] == false && fNetStatus[i])
-            fNetStatus[i]->timedUpdate(0, true);
-
-    for (int i = 0; i < pending.getCount(); ++i) {
-        const netpair stat = pending[i];
-        YStringArray::IterType pat = patterns.iterator();
-        while (++pat && fnmatch(pat, stat.name(), 0));
+            fNetStatus[i]->timedUpdate(nullptr, true);
+    }
+    for (const auto& nameAndData : pending) {
+        auto pat = patterns.iterator();
+        while (++pat && upath(nameAndData.left).fnMatch(pat));
         if (pat) {
-            createNetStatus(stat.name())->timedUpdate(stat.data());
+            createNetStatus(nameAndData.left)->timedUpdate(nameAndData.right);
         } else {
             // ignore this device
-            fNetStatus[stat.name()] = 0;
+            fNetStatus[nameAndData.left] = 0;
         }
     }
-
-    devStats.clear();
-    devicesText = 0;
 }
 #endif
 
