@@ -22,57 +22,117 @@ static YColorName minimizedTaskBarAppFg(&clrMinimizedTaskBarAppText);
 static YColorName minimizedTaskBarAppBg(&clrMinimizedTaskBarApp);
 static YColorName invisibleTaskBarAppFg(&clrInvisibleTaskBarAppText);
 static YColorName invisibleTaskBarAppBg(&clrInvisibleTaskBarApp);
-ref<YFont> TaskBarApp::normalTaskBarFont;
-ref<YFont> TaskBarApp::activeTaskBarFont;
+static ref<YFont> normalTaskBarFont;
+static ref<YFont> activeTaskBarFont;
 
-lazy<YTimer> TaskBarApp::fRaiseTimer;
-
-TaskBarApp::TaskBarApp(ClientData *frame, TaskPane *taskPane, YWindow *aParent): YWindow(aParent) {
-    fTaskPane = taskPane;
-    fFrame = frame;
-    fPixmap = None;
-    fRepainted = false;
-    selected = 0;
-    fShown = true;
-    fFlashing = false;
-    fFlashOn = false;
-    fFlashStart = zerotime();
-    setParentRelative();
-    setToolTip(frame->getTitle());
-    //setDND(true);
+TaskBarApp::TaskBarApp(ClientData* frame, TaskButton* button) :
+    fFrame(frame),
+    fButton(button),
+    fShown(true)
+{
+    fButton->addApp(this);
 }
 
 TaskBarApp::~TaskBarApp() {
-    if (fTaskPane->dragging() == this)
-        fTaskPane->endDrag();
-
-    if (fRaiseTimer)
-        fRaiseTimer->disableTimerListener(this);
-
-    if (fPixmap)
-        XFreePixmap(xapp->display(), fPixmap);
+    fButton->remove(this);
 }
 
 void TaskBarApp::activate() const {
-    getFrame()->activateWindow(true, fFlashing);
+    bool raise = true;
+    bool flash = fButton->flashing() && fFrame->isUrgent();
+    fFrame->activateWindow(raise, flash);
 }
 
-bool TaskBarApp::isFocusTraversable() {
-    return true;
+void TaskBarApp::setShown(bool show) {
+    if (fShown != show) {
+        fShown = show;
+        fButton->setShown(this, show);
+    }
+}
+
+bool TaskBarApp::getShown() const {
+    return fShown || fFrame->isUrgent();
 }
 
 int TaskBarApp::getOrder() const {
     return fFrame->getTrayOrder();
 }
 
-void TaskBarApp::setShown(bool ashow) {
-    if (ashow != fShown) {
-        fShown = ashow;
+void TaskBarApp::setFlash(bool flashing) {
+    fButton->setFlash(flashing);
+}
+
+void TaskBarApp::setToolTip(mstring tip) {
+    fButton->setToolTip(tip);
+}
+
+void TaskBarApp::repaint() {
+    fButton->repaint();
+}
+
+TaskButton::TaskButton(TaskPane* taskPane):
+    YWindow(taskPane),
+    fTaskPane(taskPane),
+    fActive(nullptr),
+    fRepainted(false),
+    fShown(true),
+    fFlashing(false),
+    fFlashOn(false),
+    fFlashStart(zerotime()),
+    selected(0)
+{
+    setParentRelative();
+}
+
+TaskButton::~TaskButton() {
+    if (fTaskPane->dragging() == this)
+        fTaskPane->endDrag();
+}
+
+void TaskButton::activate() const {
+    if (fActive) {
+        fActive->activate();
+    }
+}
+
+bool TaskButton::getShown() const {
+    return fActive ? fActive->getShown() : false;
+}
+
+bool TaskButton::isFocusTraversable() {
+    return true;
+}
+
+int TaskButton::getOrder() const {
+    return fActive ? fActive->getOrder() : 0;
+}
+
+int TaskButton::getCount() const {
+    return fActive ? 1 : 0;
+}
+
+void TaskButton::addApp(TaskBarApp* tapp) {
+    if (fActive != tapp) {
+        fActive = tapp;
         fTaskPane->relayout();
     }
 }
 
-void TaskBarApp::setFlash(bool flashing) {
+void TaskButton::remove(TaskBarApp* tapp) {
+    if (fActive == tapp) {
+        fActive = nullptr;
+        fTaskPane->remove(this);
+    }
+}
+
+void TaskButton::setShown(TaskBarApp* tapp, bool ashow) {
+    if (tapp == fActive) {
+        if (ashow != visible())
+            fTaskPane->relayout();
+    }
+}
+
+void TaskButton::setFlash(bool flashing) {
     if (fFlashing != flashing) {
         fFlashing = flashing;
 
@@ -89,7 +149,7 @@ void TaskBarApp::setFlash(bool flashing) {
     }
 }
 
-void TaskBarApp::configure(const YRect2& r) {
+void TaskButton::configure(const YRect2& r) {
     if (r.resized()) {
         int dw = r.deltaWidth();
         if (r.deltaHeight() || dw < -2 || dw > 0 || fRepainted == false) {
@@ -101,20 +161,20 @@ void TaskBarApp::configure(const YRect2& r) {
     }
 }
 
-void TaskBarApp::repaint() {
+void TaskButton::repaint() {
     if (width() > 1 && height() > 1) {
         GraphicsBuffer(this).paint();
         fRepainted = true;
     }
 }
 
-void TaskBarApp::handleExpose(const XExposeEvent& exp) {
+void TaskButton::handleExpose(const XExposeEvent& exp) {
     if (fRepainted == false) {
         repaint();
     }
 }
 
-void TaskBarApp::paint(Graphics &g, const YRect& r) {
+void TaskButton::paint(Graphics& g, const YRect& r) {
     YColor bg, fg;
     ref<YPixmap> bgPix;
     ref<YPixmap> bgLeft;
@@ -129,7 +189,7 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
 
     if (selected == 3)
         style = 3;
-    else if (getFrame()->focused() || selected == 2)
+    else if ((fActive && getFrame()->focused()) || selected == 2)
         style = 2;
     else
         style = 1;
@@ -156,12 +216,12 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
             bgRightG = taskbuttonRightPixbuf;
             style = 1;
         }
-    } else if (!getFrame()->visibleNow()) {
+    } else if (fActive && !getFrame()->visibleNow()) {
         bg = invisibleTaskBarAppBg;
         fg = invisibleTaskBarAppFg;
         bgPix = taskbackPixmap;
         bgGrad = taskbackPixbuf;
-    } else if (getFrame()->isMinimized()) {
+    } else if (fActive && getFrame()->isMinimized()) {
         bg = minimizedTaskBarAppBg;
         fg = minimizedTaskBarAppFg;
         bgPix = taskbuttonminimizedPixmap;
@@ -170,7 +230,7 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
         bgGrad = taskbuttonminimizedPixbuf;
         bgLeftG = taskbuttonminimizedLeftPixbuf;
         bgRightG = taskbuttonminimizedRightPixbuf;
-    } else if (getFrame()->focused()) {
+    } else if (fActive && getFrame()->focused()) {
         bg = activeTaskBarAppBg;
         fg = activeTaskBarAppFg;
         bgPix = taskbuttonactivePixmap;
@@ -268,23 +328,32 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
         }
     }
 
-    ref<YIcon> icon(getFrame()->getIcon());
+    ref<YIcon> icon;
     bool iconDrawn = false;
-
-    if (taskBarShowWindowIcons && icon != null) {
+    if (taskBarShowWindowIcons) {
+        if (fActive) {
+            icon = getFrame()->getIcon();
+        }
+    }
+    if (icon != null) {
         int iconSize = YIcon::smallSize();
-
         int const y((height() - 3 - iconSize -
                      (wmLook == lookMetal)) / 2);
         iconDrawn = icon->draw(g, p + max(1, left), p + 1 + y, iconSize);
-        if (iconDrawn && p + max(1, left) + iconSize + 5 >= int(width()))
+        if (iconDrawn && p + max(1, left) + iconSize + 5 >= int(width())) {
+            if (bgGrad != null) {
+                g.maxOpacity();
+            }
             return;
+        }
     }
 
-    ustring str = getFrame()->getIconTitle();
-    if (str == null || str.length() == 0)
-        str = getFrame()->getTitle();
-
+    mstring str;
+    if (fActive) {
+        str = getFrame()->getIconTitle();
+        if (str.isEmpty())
+            str = getFrame()->getTitle();
+    }
     if (str != null) {
         ref<YFont> font = getFont();
         if (font != null) {
@@ -293,7 +362,7 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
 
             int iconSize = 0;
             int pad = max(1, left);
-            if (taskBarShowWindowIcons && iconDrawn) {
+            if (iconDrawn) {
                 iconSize = YIcon::smallSize();
                 pad += 2;
             }
@@ -308,9 +377,13 @@ void TaskBarApp::paint(Graphics &g, const YRect& r) {
                 g.drawStringEllipsis(p + tx, p + ty, str, wm);
         }
     }
+
+    if (bgGrad != null) {
+        g.maxOpacity();
+    }
 }
 
-unsigned TaskBarApp::maxHeight() {
+unsigned TaskButton::maxHeight() {
     unsigned activeHeight =
         (getActiveFont() != null) ? getActiveFont()->height() : 0;
     unsigned normalHeight =
@@ -318,28 +391,28 @@ unsigned TaskBarApp::maxHeight() {
     return 2 + max(activeHeight, normalHeight);
 }
 
-ref<YFont> TaskBarApp::getFont() {
+ref<YFont> TaskButton::getFont() {
     ref<YFont> font;
-    if (getFrame()->focused())
+    if (fActive && getFrame()->focused())
         font = getActiveFont();
     if (font == null)
         font = getNormalFont();
     return font;
 }
 
-ref<YFont> TaskBarApp::getNormalFont() {
+ref<YFont> TaskButton::getNormalFont() {
     if (normalTaskBarFont == null)
         normalTaskBarFont = YFont::getFont(XFA(normalTaskBarFontName));
     return normalTaskBarFont;
 }
 
-ref<YFont> TaskBarApp::getActiveFont() {
+ref<YFont> TaskButton::getActiveFont() {
     if (activeTaskBarFont == null)
         activeTaskBarFont = YFont::getFont(XFA(activeTaskBarFontName));
     return activeTaskBarFont;
 }
 
-void TaskBarApp::handleButton(const XButtonEvent &button) {
+void TaskButton::handleButton(const XButtonEvent& button) {
     YWindow::handleButton(button);
 
     if (fTaskPane->dragging())
@@ -350,7 +423,7 @@ void TaskBarApp::handleButton(const XButtonEvent &button) {
             selected = 2;
             repaint();
         } else if (button.type == ButtonRelease) {
-            if (selected == 2) {
+            if (selected == 2 && fActive) {
                 if (button.button == 1) {
                     if (getFrame()->focused() && getFrame()->visibleNow() &&
                         (!getFrame()->canRaise() || (button.state & ControlMask)))
@@ -386,7 +459,7 @@ void TaskBarApp::handleButton(const XButtonEvent &button) {
     }
 }
 
-void TaskBarApp::handleCrossing(const XCrossingEvent &crossing) {
+void TaskButton::handleCrossing(const XCrossingEvent& crossing) {
     if (selected > 0) {
         if (crossing.type == EnterNotify) {
             selected = 2;
@@ -399,49 +472,39 @@ void TaskBarApp::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
-void TaskBarApp::switchToPrev() {
-    TaskBarApp* act = fTaskPane->getActive();
-    TaskBarApp* app = Elvis(fTaskPane->predecessor(act), this);
-    if (app != act)
-        app->activate();
-}
-
-void TaskBarApp::switchToNext() {
-    TaskBarApp* act = fTaskPane->getActive();
-    TaskBarApp* app = Elvis(fTaskPane->successor(act), this);
-    if (app != act)
-        app->activate();
-}
-
-void TaskBarApp::handleClick(const XButtonEvent &up, int /*count*/) {
-    if (up.button == 3) {
+void TaskButton::handleClick(const XButtonEvent& up, int /*count*/) {
+    if (up.button == Button3 && fActive) {
         getFrame()->popupSystemMenu(this, up.x_root, up.y_root,
                                     YPopupWindow::pfCanFlipVertical |
                                     YPopupWindow::pfCanFlipHorizontal |
                                     YPopupWindow::pfPopupMenu);
     }
-    else if (up.button == Button4 && taskBarUseMouseWheel)
-        switchToPrev();
-    else if (up.button == Button5 && taskBarUseMouseWheel)
-        switchToNext();
+    else if (up.button == Button4 && taskBarUseMouseWheel) {
+        fTaskPane->switchToPrev();
+    }
+    else if (up.button == Button5 && taskBarUseMouseWheel) {
+        fTaskPane->switchToNext();
+    }
 }
 
-void TaskBarApp::handleDNDEnter() {
+void TaskButton::handleDNDEnter() {
     fRaiseTimer->setTimer(autoRaiseDelay, this, true);
     selected = 3;
     repaint();
 }
 
-void TaskBarApp::handleDNDLeave() {
-    if (fRaiseTimer)
-        fRaiseTimer->disableTimerListener(this);
+void TaskButton::handleDNDLeave() {
+    fRaiseTimer = null;
     selected = 0;
     repaint();
 }
 
-bool TaskBarApp::handleTimer(YTimer *t) {
+bool TaskButton::handleTimer(YTimer* t) {
     if (t == fRaiseTimer) {
-        getFrame()->wmRaise();
+        fRaiseTimer = null;
+        if (fActive) {
+            getFrame()->wmRaise();
+        }
     }
     if (t == fFlashTimer) {
         if (!fFlashing) {
@@ -460,7 +523,7 @@ bool TaskBarApp::handleTimer(YTimer *t) {
     return false;
 }
 
-void TaskBarApp::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
+void TaskButton::handleBeginDrag(const XButtonEvent& down, const XMotionEvent& motion) {
     if (down.button == Button1) {
         raise();
         fTaskPane->startDrag(this, 0, down.x + x(), down.y + y());
@@ -468,13 +531,15 @@ void TaskBarApp::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &m
     }
 }
 
-TaskPane::TaskPane(IAppletContainer *taskBar, YWindow *parent): YWindow(parent) {
-    fTaskBar = taskBar;
-    fNeedRelayout = true;
-    fForceImmediate = false;
-    fDragging = 0;
-    fDragX = fDragY = 0;
-
+TaskPane::TaskPane(IAppletContainer* taskBar, YWindow* parent):
+    YWindow(parent),
+    fTaskBar(taskBar),
+    fDragging(nullptr),
+    fDragX(0),
+    fDragY(0),
+    fNeedRelayout(true),
+    fForceImmediate(false)
+{
     addStyle(wsNoExpose);
     if (getGradient() == null && taskbackPixmap == null) {
         setBackground(taskBarBg);
@@ -483,21 +548,20 @@ TaskPane::TaskPane(IAppletContainer *taskBar, YWindow *parent): YWindow(parent)
 }
 
 TaskPane::~TaskPane() {
-    if (fDragging != 0)
+    if (fDragging)
         endDrag();
+
+    normalTaskBarFont = null;
+    activeTaskBarFont = null;
 }
 
-void TaskPane::insert(TaskBarApp *tapp) {
-    IterType it = fApps.reverseIterator();
+void TaskPane::insert(TaskBarApp* tapp) {
+    IterApps it = fApps.reverseIterator();
     while (++it && it->getOrder() > tapp->getOrder());
     (--it).insert(tapp);
 }
 
-void TaskPane::remove(TaskBarApp *tapp) {
-    findRemove(fApps, tapp);
-}
-
-TaskBarApp* TaskPane::predecessor(TaskBarApp *tapp) {
+TaskBarApp* TaskPane::predecessor(TaskBarApp* tapp) {
     const int count = fApps.getCount();
     const int found = find(fApps, tapp);
     if (found >= 0) {
@@ -508,10 +572,10 @@ TaskBarApp* TaskPane::predecessor(TaskBarApp *tapp) {
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
-TaskBarApp* TaskPane::successor(TaskBarApp *tapp) {
+TaskBarApp* TaskPane::successor(TaskBarApp* tapp) {
     const int count = fApps.getCount();
     const int found = find(fApps, tapp);
     if (found >= 0) {
@@ -522,32 +586,54 @@ TaskBarApp* TaskPane::successor(TaskBarApp *tapp) {
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
-TaskBarApp* TaskPane::findApp(YFrameWindow *frame) {
-    IterType task = fApps.iterator();
+TaskBarApp* TaskPane::findApp(ClientData* frame) {
+    IterApps task = fApps.iterator();
     while (++task && task->getFrame() != frame);
-    return task ? *task : 0;
+    return task ? *task : nullptr;
 }
 
-TaskBarApp* TaskPane::getActive() {
+TaskBarApp* TaskPane::getActiveApp() {
     return findApp(manager->getFocus());
 }
 
-TaskBarApp *TaskPane::addApp(YFrameWindow *frame) {
-    TaskBarApp *tapp = new TaskBarApp(frame, this, this);
-    if (tapp != 0) {
-        insert(tapp);
+TaskButton* TaskPane::getActiveButton() {
+    TaskBarApp* tapp = getActiveApp();
+    return tapp ? tapp->button() : nullptr;
+}
+
+TaskBarApp* TaskPane::addApp(ClientData* frame) {
+    TaskBarApp* tapp = nullptr;
+    TaskButton* task = new TaskButton(this);
+    if (task) {
+        tapp = new TaskBarApp(frame, task);
+        if (tapp) {
+            insert(tapp);
+            insert(task);
+        } else {
+            delete task;
+        }
     }
     return tapp;
 }
 
-void TaskPane::removeApp(YFrameWindow *frame) {
-    TaskBarApp* task = findApp(frame);
+void TaskPane::remove(TaskBarApp* task) {
     if (task) {
-        task->hide();
-        remove(task);
+        findRemove(fApps, task);
+    }
+}
+
+void TaskPane::insert(TaskButton* task) {
+    IterTask it = fTasks.reverseIterator();
+    while (++it && it->getOrder() > task->getOrder());
+    (--it).insert(task);
+}
+
+void TaskPane::remove(TaskButton* button) {
+    if (button) {
+        findRemove(fTasks, button);
         relayout();
     }
 }
@@ -561,7 +647,7 @@ void TaskPane::relayout(bool force) {
     }
 }
 
-bool TaskPane::handleTimer(YTimer *t) {
+bool TaskPane::handleTimer(YTimer* t) {
     if (t == fRelayoutTimer) {
         fRelayoutTimer = null;
         fNeedRelayout = true;
@@ -586,7 +672,7 @@ void TaskPane::relayoutNow(bool force) {
 
     int tc = 0;
 
-    for (IterType task = fApps.iterator(); ++task; ) {
+    for (IterTask task = fTasks.iterator(); ++task; ) {
         if (task->getShown())
             tc++;
         else
@@ -601,7 +687,7 @@ void TaskPane::relayoutNow(bool force) {
     int x = 0;
     int lc = 0;
 
-    for (IterType task = fApps.iterator(); ++task; ) {
+    for (IterTask task = fTasks.iterator(); ++task; ) {
         if (task->getShown()) {
             const int w1 = wid + (lc < rem);
             if (task != dragging()) {
@@ -616,22 +702,22 @@ void TaskPane::relayoutNow(bool force) {
         dragging()->show();
 }
 
-void TaskPane::handleClick(const XButtonEvent &up, int count) {
+void TaskPane::handleClick(const XButtonEvent& up, int count) {
     if (up.button == 3 && count == 1 && IS_BUTTON(up.state, Button3Mask)) {
         fTaskBar->contextMenu(up.x_root, up.y_root);
     }
     else if ((up.button == Button4 || up.button == Button5)
            && taskBarUseMouseWheel)
     {
-        TaskBarApp* app = getActive();
-        if (app)
-            app->handleClick(up, count);
+        TaskButton* button = getActiveButton();
+        if (button)
+            button->handleClick(up, count);
         else
             fTaskBar->windowTrayPane()->handleClick(up, count);
     }
 }
 
-void TaskPane::paint(Graphics &g, const YRect& r) {
+void TaskPane::paint(Graphics& g, const YRect& r) {
     if (taskbackPixmap == null && getGradient() == null) {
         g.setColor(taskBarBg);
         g.fillRect(r.x(), r.y(), r.width(), r.height());
@@ -639,16 +725,16 @@ void TaskPane::paint(Graphics &g, const YRect& r) {
 }
 
 unsigned TaskPane::maxHeight() {
-    return TaskBarApp::maxHeight();
+    return TaskButton::maxHeight();
 }
 
-void TaskPane::handleMotion(const XMotionEvent &motion) {
-    if (fDragging != 0) {
+void TaskPane::handleMotion(const XMotionEvent& motion) {
+    if (fDragging) {
         processDrag(motion.x, motion.y);
     }
 }
 
-void TaskPane::handleButton(const XButtonEvent &button) {
+void TaskPane::handleButton(const XButtonEvent& button) {
     if (button.type == ButtonRelease)
         endDrag();
     YWindow::handleButton(button);
@@ -660,23 +746,22 @@ void TaskPane::configure(const YRect2& r) {
     }
 }
 
-void TaskPane::startDrag(TaskBarApp *drag, int /*byMouse*/, int sx, int sy) {
-    if (fDragging == 0) {
-        if (!xapp->grabEvents(this, YXApplication::movePointer.handle(),
-                              ButtonPressMask |
-                              ButtonReleaseMask |
-                              PointerMotionMask, 1, 1, 0))
+void TaskPane::startDrag(TaskButton* drag, int /*byMouse*/, int sx, int sy) {
+    if (fDragging == nullptr) {
+        if (xapp->grabEvents(this, YXApplication::movePointer.handle(),
+                             ButtonPressMask |
+                             ButtonReleaseMask |
+                             PointerMotionMask))
         {
-            return ;
+            fDragging = drag;
+            fDragX = sx;
+            fDragY = sy;
         }
-        fDragging = drag;
-        fDragX = sx;
-        fDragY = sy;
     }
 }
 
 void TaskPane::processDrag(int mx, int my) {
-    if (fDragging == 0)
+    if (fDragging == nullptr)
         return;
 
     const int w2 = width() - fDragging->width() - 2;
@@ -693,7 +778,7 @@ void TaskPane::processDrag(int mx, int my) {
 
     int drag = -1;
     int drop = -1;
-    for (IterType task = fApps.iterator(); ++task; ) {
+    for (IterTask task = fTasks.iterator(); ++task; ) {
         if (task->getShown()) {
             if (task == fDragging)
                 drag = task.where();
@@ -702,7 +787,7 @@ void TaskPane::processDrag(int mx, int my) {
         }
     }
     if (drag >= 0 && drop >= 0) {
-        fApps.swap(drag, drop);
+        fTasks.swap(drag, drop);
     }
     fDragX += dx;
     fDragY += dy;
@@ -710,57 +795,51 @@ void TaskPane::processDrag(int mx, int my) {
 }
 
 void TaskPane::endDrag() {
-    if (fDragging != 0) {
+    if (fDragging) {
         xapp->releaseEvents();
-        fDragging = 0;
+        fDragging = nullptr;
         relayout(true);
     }
 }
 
 void TaskPane::movePrev() {
-    TaskBarApp* act = getActive();
-    if (!act)
-        return;
-    TaskBarApp* other = predecessor(act);
-    if (!other)
-        return;
-    const int other_index = find(fApps, other);
-    const int act_index = find(fApps, act);
-
-    if (other_index < 0 || other_index > act_index)
-        return;
-
-    fApps.swap(other_index, act_index);
-    relayout(true);
+    TaskButton* active = getActiveButton();
+    if (active) {
+        int index = find(fTasks, active);
+        if (index > 0) {
+            fTasks.swap(index, index - 1);
+            relayout(true);
+        }
+    }
 }
 
 void TaskPane::moveNext() {
-    TaskBarApp* act = getActive();
-    if (!act)
-        return;
-    TaskBarApp* other = successor(act);
-    if (!other)
-        return;
-    const int other_index = find(fApps, other);
-    const int act_index = find(fApps, act);
-
-    if (act_index < 0 || other_index < act_index)
-        return;
-
-    fApps.swap(other_index, act_index);
-    relayout(true);
+    TaskButton* active = getActiveButton();
+    if (active) {
+        int index = find(fTasks, active);
+        if (index + 1 < fTasks.getCount()) {
+            fTasks.swap(index, index + 1);
+            relayout(true);
+        }
+    }
 }
 
 void TaskPane::switchToPrev() {
-        TaskBarApp* app = getActive();
-        if (app)
-            app->switchToPrev();
+    TaskBarApp* act = getActiveApp();
+    if (act) {
+        TaskBarApp* pre = predecessor(act);
+        if (pre && pre != act)
+            pre->activate();
+    }
 }
 
 void TaskPane::switchToNext() {
-        TaskBarApp* app = getActive();
-        if (app)
-            app->switchToNext();
+    TaskBarApp* act = getActiveApp();
+    if (act) {
+        TaskBarApp* suc = successor(act);
+        if (suc && suc != act)
+            suc->activate();
+    }
 }
 
 // vim: set sw=4 ts=4 et:
