@@ -13,13 +13,13 @@
  */
 
 #include "config.h"
-
 #include "atray.h"
 #include "applet.h"
 #include "yprefs.h"
 #include "yxapp.h"
 #include "prefs.h"
 #include "wmframe.h"
+#include "wmmgr.h"
 #include "wmwinlist.h"
 #include "wpixmaps.h"
 #include "yrect.h"
@@ -43,12 +43,11 @@ TrayApp::TrayApp(ClientData *frame, TrayPane *trayPane, YWindow *aParent):
     fFrame = frame;
     fTrayPane = trayPane;
     selected = 0;
-    fShown = true;
+    fShown = (trayShowAllWindows || frame->visibleNow());
     fRepainted = false;
     setParentRelative();
     setToolTip(frame->getTitle());
-    setTitle(cstring(frame->getTitle()));
-    //setDND(true);
+    setTitle(frame->getTitle());
 }
 
 TrayApp::~TrayApp() {
@@ -67,8 +66,10 @@ int TrayApp::getOrder() const {
 }
 
 void TrayApp::setShown(bool ashow) {
-    if (ashow != fShown) {
+    if (fShown != ashow) {
         fShown = ashow;
+        setVisible(ashow);
+        fTrayPane->relayout();
     }
 }
 
@@ -80,12 +81,14 @@ void TrayApp::configure(const YRect2& r) {
 }
 
 void TrayApp::repaint() {
-    GraphicsBuffer(this).paint();
-    fRepainted = true;
+    if (1 < geometry().pixels()) {
+        GraphicsBuffer(this).paint();
+        fRepainted = true;
+    }
 }
 
-void TrayApp::handleExpose(const XExposeEvent& exp) {
-    if (fRepainted == false) {
+void TrayApp::handleExpose(const XExposeEvent& expose) {
+    if (fRepainted == false && expose.count == 0) {
         repaint();
     }
 }
@@ -154,7 +157,8 @@ void TrayApp::paint(Graphics &g, const YRect& r) {
             g.setColor(bg);
         int wd = int(width());
         int ht = int(height());
-        int sz = YIcon::smallSize();
+        int ti = min(trayIconMaxWidth, trayIconMaxHeight);
+        int sz = max(int(YIcon::smallSize()), min(ti, min(wd, ht)));
         icon->draw(g, (wd - sz) / 2, (ht - sz) / 2, sz);
     }
 }
@@ -274,7 +278,7 @@ TrayApp* TrayPane::predecessor(TrayApp *tapp) {
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 TrayApp* TrayPane::successor(TrayApp *tapp) {
@@ -288,7 +292,7 @@ TrayApp* TrayPane::successor(TrayApp *tapp) {
             }
         }
     }
-    return 0;
+    return nullptr;
 }
 
 TrayApp* TrayPane::findApp(YFrameWindow *frame) {
@@ -304,32 +308,22 @@ TrayApp* TrayPane::getActive() {
 TrayApp *TrayPane::addApp(YFrameWindow *frame) {
     TrayApp *tapp = new TrayApp(frame, this, this);
 
-    if (tapp != 0) {
+    if (tapp != nullptr) {
         IterType it = fApps.reverseIterator();
         while (++it && it->getOrder() > tapp->getOrder());
         (--it).insert(tapp);
 
-        tapp->show();
-
-        if (!(frame->visibleOn(manager->activeWorkspace()) ||
-              trayShowAllWindows))
-            tapp->setShown(false);
-
-        relayout();
+        if (tapp->getShown()) {
+            tapp->show();
+            relayout();
+        }
     }
     return tapp;
 }
 
-void TrayPane::removeApp(YFrameWindow *frame) {
-    for (IterType icon = fApps.iterator(); ++icon; ) {
-        if (icon->getFrame() == frame) {
-            icon->hide();
-            icon.remove();
-
-            relayout();
-            return;
-        }
-    }
+void TrayPane::remove(TrayApp* tapp) {
+    tapp->setShown(false);
+    findRemove(fApps, tapp);
 }
 
 int TrayPane::getRequiredWidth() {
