@@ -1,24 +1,13 @@
-#ifndef __BASE_H
-#define __BASE_H
+#ifndef BASE_H
+#define BASE_H
 
+#include "debug.h"
 #include <stddef.h>
 
 #ifndef __GNUC__
 #define __attribute__(a)
 #endif
 
-// use override helper keyword where available
-#if __cplusplus < 201103L
-#define OVERRIDE
-#else
-#define OVERRIDE override
-#endif
-#if __cplusplus == 199711L
-#define nullptr NULL
-#define override
-#define final
-#endif
-
 /*** Essential Arithmetic Functions *******************************************/
 
 /*
@@ -99,6 +88,10 @@ unsigned long strhash(const char* str);
 inline bool nonempty(const char* s) { return s && *s; }
 inline bool isEmpty(const char* s) { return !(s && *s); }
 
+#ifndef HAVE_MEMRCHR
+void* memrchr(const void*, char, size_t);
+#endif
+
 /*** Message Functions ********************************************************/
 
 void die(int exitcode, char const *msg, ...) __attribute__((format(printf, 2, 3) ));
@@ -107,7 +100,14 @@ void fail(char const *msg, ...) __attribute__((format(printf, 1, 2) ));
 void msg(char const *msg, ...) __attribute__((format(printf, 1, 2) ));
 void tlog(char const *msg, ...) __attribute__((format(printf, 1, 2) ));
 void precondition(const char *expr, const char *file, int line);
+
+/*** User Environment ********************************************************/
+
+char* userhome(const char* username);
+char* dollar_expansion(const char* name);
+char* tilde_expansion(const char* name);
 char* path_lookup(const char* name);
+
 char* progpath();
 void show_backtrace(const int limit = 0);
 
@@ -119,6 +119,7 @@ void show_backtrace(const int limit = 0);
 /*** Misc Stuff (clean up!!!) *************************************************/
 
 #define ACOUNT(x) (sizeof(x)/sizeof(x[0]))
+#define BUFNSIZE(x) x, sizeof(x)
 
 //!!! clean these up
 #define KEY_MODMASK(x) ((x) & (xapp->KeyMask))
@@ -130,17 +131,10 @@ void show_backtrace(const int limit = 0);
 #define HASMASK(w,e,n) ((((w) & ~(n)) & (e)) == (e))
 
 int strpcmp(char const *str, char const *pfx, char const *delim = "=:");
-#if 0
-unsigned strtoken(const char *str, const char *delim = " \t");
-#endif
 char const * strnxt(const char *str, const char *delim = " \t");
+const char* errno_string();
 const char *my_basename(const char *filename);
 
-#if 0
-bool strequal(const char *a, const char *b);
-int strnullcmp(const char *a, const char *b);
-#endif
-
 inline const char* boolstr(bool bval) {
     return bval ? "true" : "false";
 }
@@ -148,7 +142,7 @@ inline const char* boolstr(bool bval) {
 template <class T>
 inline char const * niceUnit(T & val, char const * const units[],
                              T const lim = 10240, T const div = 1024) {
-    char const * uname(0);
+    char const * uname(nullptr);
 
     if (units && *units) {
         uname = *units++;
@@ -167,6 +161,8 @@ inline char const * niceUnit(T & val, char const * const units[],
 
 bool testOnce(const char* file, const int line);
 
+bool little();
+
 /*** Bit Operations ***********************************************************/
 
 template <class M, class B>
@@ -237,59 +233,6 @@ void print_version_exit(const char *version);
 void check_help_version(const char *arg, const char *help, const char *version);
 void check_argv(int argc, char **argv, const char *help, const char *version);
 
-/*** file handling ************************************************************/
-
-/* read from file descriptor and zero terminate buffer. */
-int read_fd(int fd, char *buf, size_t buflen);
-
-/* read from filename and zero terminate the buffer. */
-int read_file(const char *filename, char *buf, size_t buflen);
-
-/* read all of filedescriptor and return a zero-terminated new[] string. */
-char* load_fd(int fd);
-
-/* read a file as a zero-terminated new[] string. */
-char* load_text_file(const char *filename);
-
-/******************************************************************************/
-
-#include "debug.h"
-
-extern bool loggingEvents;
-extern bool initLogEvents();
-
-void logAny(const union _XEvent& xev);
-void logButton(const union _XEvent& xev);
-void logClientMessage(const union _XEvent& xev);
-void logColormap(const union _XEvent& xev);
-void logConfigureNotify(const union _XEvent& xev);
-void logConfigureRequest(const union _XEvent& xev);
-void logCreate(const union _XEvent& xev);
-void logCrossing(const union _XEvent& xev);
-void logDestroy(const union _XEvent& xev);
-void logExpose(const union _XEvent& xev);
-void logFocus(const union _XEvent& xev);
-void logGravity(const union _XEvent& xev);
-void logKey(const union _XEvent& xev);
-void logMapRequest(const union _XEvent& xev);
-void logMapNotify(const union _XEvent& xev);
-void logUnmap(const union _XEvent& xev);
-void logMotion(const union _XEvent& xev);
-void logProperty(const union _XEvent& xev);
-void logReparent(const union _XEvent& xev);
-void logRandrScreen(const union _XEvent& xev);
-void logRandrNotify(const union _XEvent& xev);
-void logShape(const union _XEvent& xev);
-void logVisibility(const union _XEvent& xev);
-void logEvent(const union _XEvent& xev);
-
-typedef const char* (*AtomNameFunc)(unsigned long atom);
-void setAtomName(AtomNameFunc atomNameFunc);
-const char* getAtomName(unsigned long atom);
-void setLogEvent(int evtype, bool enable);
-bool toggleLogEvents();
-const char* eventName(int eventType);
-
 inline int intersection(int s1, int e1, int s2, int e2) {
     return max(0, min(e1, e2) - max(s1, s2));
 }
