@@ -5,11 +5,11 @@
 #include "ytime.h"
 #include "sysdep.h"
 #include "appnames.h"
+#include "ypointer.h"
 #include "ascii.h"
 using namespace ASCII;
-#ifdef HAVE_WORDEXP
-#include <wordexp.h>
-#endif
+#include "ywordexp.h"
+#include <X11/Xlib.h>
 
 #ifdef CONFIG_EXTERNAL_TRAY
 #define NOTRAY false
@@ -32,30 +32,40 @@ private:
     }
 
     bool expand(char *buf, size_t bufsiz) {
-#ifdef HAVE_WORDEXP
         wordexp_t w;
         if (wordexp(trim(buf), &w, 0) != 0 || w.we_wordc == 0)
             return false;
         size_t len = strlcpy(buf, trim(w.we_wordv[0]), bufsiz);
-        for (size_t k = 1; k < w.we_wordc && len < bufsiz; ++k) {
+        for (size_t k = 1; k < size_t(w.we_wordc) && len < bufsiz; ++k) {
             strlcat(buf, " ", bufsiz);
             len = strlcat(buf, trim(w.we_wordv[k]), bufsiz);
         }
         wordfree(&w);
         if (len >= bufsiz)
             return false;
-#else
-        char *str = trim(buf);
-        if (str > buf)
-            strlcpy(buf, str, bufsiz);
-#endif
         if (buf[0] == '#' || buf[0] == '=')
             buf[0] = 0;
         return buf[0] != 0;
     }
 
+    void setup() {
+        char buf[PATH_MAX];
+        char* pwd = getcwd(buf, sizeof buf);
+        if (pwd && !strcmp(pwd, "/") && access(pwd, W_OK)) {
+            char* home = getenv("HOME");
+            if (nonempty(home) && !access(home, W_OK|X_OK) && !chdir(home)) {
+                ; // success
+            } else {
+                csmart user(userhome(nullptr));
+                if (user && chdir(user) == 0) {
+                    setenv("HOME", user, true);
+                }
+            }
+        }
+    }
+
     const char *get_help_text() {
-        return _(
+        helptext = _(
         "  -c, --config=FILE   Let IceWM load preferences from FILE.\n"
         "  -t, --theme=FILE    Let IceWM load the theme from FILE.\n"
         "\n"
@@ -70,6 +80,17 @@ private:
         "  -n, --notray        Do not start icewmtray.\n"
         "  -s, --sound         Also start icesound.\n"
         );
+
+        helptext += _(
+        "\n"
+        "Debugging options:\n"
+        "  -v, --valgrind      Let \"/usr/bin/valgrind\" run icewm.\n"
+        "                      Thoroughly examines the execution of icewm.\n"
+        "  -g, --catchsegv     Let \"/usr/bin/catchsegv\" run icewm.\n"
+        "                      Gives a backtrace if icewm segfaults.\n"
+        );
+
+        return helptext.c_str();
     }
 
     const char *displayArg;
@@ -81,23 +102,28 @@ private:
     bool nobgArg;
     bool notrayArg;
     bool soundArg;
+    bool grindArg;
+    bool catchArg;
     char* argv0;
 
     void options(int *argc, char ***argv) {
+        const char* outputArg = nullptr;
         argv0 = **argv;
-        displayArg = 0;
-        configArg = 0;
-        themeArg = 0;
-        alphaArg = 0;
+        displayArg = nullptr;
+        configArg = nullptr;
+        themeArg = nullptr;
+        alphaArg = nullptr;
         icewmExe = ICEWMEXE;
         syncArg = false;
         nobgArg = false;
         notrayArg = NOTRAY;
         soundArg = false;
+        grindArg = false;
+        catchArg = false;
 
         for (char **arg = 1 + *argv; arg < *argv + *argc; ++arg) {
             if (**arg == '-') {
-                char *value(0);
+                char *value(nullptr);
                 if (GetArgument(value, "d", "display", arg, *argv+*argc)) {
                     if (value && *value)
                         displayArg = value;
@@ -112,18 +138,7 @@ private:
                     icewmExe = value;
                 }
                 else if (GetArgument(value, "o", "output", arg, *argv+*argc)) {
-                    upath path(upath(value).expand());
-                    int flags = O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_APPEND;
-                    int fd(path.open(flags, 0600));
-                    if (fd == -1) {
-                        perror(path.string());
-                    } else {
-                        dup2(fd, 1);
-                        dup2(fd, 2);
-                        if (fd > 2) {
-                            close(fd);
-                        }
-                    }
+                    outputArg = value;
                 }
                 else if (is_switch(*arg, "a", "alpha")) {
                     alphaArg = *arg;
@@ -140,6 +155,12 @@ private:
                 else if (is_switch(*arg, "s", "sound")) {
                     soundArg = true;
                 }
+                else if (is_switch(*arg, "v", "valgrind")) {
+                    grindArg = true;
+                }
+                else if (is_switch(*arg, "g", "catchsegv")) {
+                    catchArg = true;
+                }
                 else if (is_help_switch(*arg)) {
                     print_help_exit(get_help_text());
                 }
@@ -149,6 +170,20 @@ private:
                 else if (is_copying_switch(*arg)) {
                     print_copying_exit();
                 }
+#ifdef PRECON
+                else if (GetArgument(value, "e", "enquire", arg, *argv+*argc)) {
+                    int pref = atoi(value);
+                    bool term = false;
+                    bool res = enquireRestart(&term, pref);
+                    msg("enquire(term=%s, pref=%d) = %s",
+                        boolstr(term), pref, boolstr(res));
+                    ::exit(1);
+                }
+                else if (is_switch(*arg, "r", "rescue")) {
+                    rescueFocus();
+                    ::exit(1);
+                }
+#endif
                 else {
                     warn(_("Unknown option '%s'"), *arg);
                 }
@@ -156,16 +191,33 @@ private:
         }
         if (displayArg)
             setenv("DISPLAY", displayArg, 1);
+        if (outputArg) {
+            upath path(upath(outputArg).expand());
+            int flags = O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_APPEND;
+            int fd(path.open(flags, 0600));
+            if (fd == -1) {
+                perror(path.string());
+            } else {
+                dup2(fd, 1);
+                dup2(fd, 2);
+                if (fd > 2) {
+                    close(fd);
+                }
+            }
+        }
     }
 
 public:
     SessionManager(int *argc, char ***argv): YApplication(argc, argv) {
+        setup();
         options(argc, argv);
         startup_phase = 0;
+        bg_pid = -1;
         wm_pid = -1;
         tray_pid = -1;
+        term_pid = -1;
         sound_pid = -1;
-        bg_pid = -1;
+        rescue_pid = -1;
         crashtime = zerotime();
 
         catchSignal(SIGCHLD);
@@ -175,7 +227,7 @@ public:
     }
 
     void loadEnv(const char *scriptName) {
-        upath scriptFile = YApplication::findConfigFile(scriptName);
+        upath scriptFile = locateConfigFile(scriptName);
         if (scriptFile.nonempty()) {
             FILE *ef = scriptFile.fopen("r");
             if (!ef)
@@ -203,7 +255,7 @@ public:
 
     virtual int runProgram(const char *file, const char *const *args) {
         upath path;
-        if (strchr(file, '/') == NULL && strchr(argv0, '/') != NULL) {
+        if (strchr(file, '/') == nullptr && strchr(argv0, '/') != nullptr) {
             path = upath(argv0).parent() + file;
             if (path.isExecutable()) {
                 file = path.string();
@@ -215,11 +267,11 @@ public:
     }
 
     void runScript(const char *scriptName) {
-        upath scriptFile = YApplication::findConfigFile(scriptName);
+        upath scriptFile = locateConfigFile(scriptName);
         if (scriptFile.nonempty() && scriptFile.isExecutable()) {
             const char *cs = scriptFile.string();
             MSG(("Running session script: %s", cs));
-            runProgram(cs, 0);
+            runProgram(cs, nullptr);
         }
     }
 
@@ -244,12 +296,12 @@ public:
             args[k++] = "--sync";
         }
         if (k < narg) {
-            args[k] = 0;
+            args[k] = nullptr;
         }
     }
 
     void runIcewmbg(bool quit = false) {
-        const char *args[12] = { ICEWMBGEXE, 0, 0 };
+        const char *args[12] = { ICEWMBGEXE, nullptr, nullptr };
 
         if (nobgArg) {
             return;
@@ -264,7 +316,7 @@ public:
     }
 
     void runIcewmtray(bool quit = false) {
-        const char *args[12] = { ICEWMTRAYEXE, "--notify", 0 };
+        const char *args[12] = { ICEWMTRAYEXE, "--notify", nullptr };
         if (quit) {
             if (tray_pid != -1) {
                 kill(tray_pid, SIGTERM);
@@ -283,7 +335,7 @@ public:
     }
 
     void runIcesound(bool quit = false) {
-        const char *args[12] = { ICESOUNDEXE, "--verbose", 0 };
+        const char *args[12] = { ICESOUNDEXE, "--verbose", nullptr };
         if (soundArg == false) {
             return;
         }
@@ -302,6 +354,9 @@ public:
     }
 
     void runWM(bool quit = false) {
+        if (rescue_pid != -1) {
+            kill(rescue_pid, SIGTERM);
+        }
         if (quit) {
             if (wm_pid != -1) {
                 kill(wm_pid, SIGTERM);
@@ -311,11 +366,21 @@ public:
             wm_pid = -1;
         }
         else {
+            static const char valgrind[] = "/usr/bin/valgrind";
+            static const char catchsegv[] = "/usr/bin/catchsegv";
             const int size = 24;
-            const char* args[size] = {
-                icewmExe, "--notify", nullptr
-            };
-            appendOptions(args, 2, size);
+            const char* args[size];
+            int count = 0;
+            if (grindArg && access(valgrind, X_OK) == 0) {
+                args[count++] = valgrind;
+            }
+            else if (catchArg && access(catchsegv, X_OK) == 0) {
+                args[count++] = catchsegv;
+            }
+            args[count++] = icewmExe;
+            args[count++] = "--notify";
+            args[count] = nullptr;
+            appendOptions(args, count, size);
             char* copy = nullptr;
             if (wmoptions.length()) {
                 copy = strdup(wmoptions);
@@ -361,12 +426,20 @@ private:
                     tray_pid = -1;
                     checkTrayExitStatus(status);
                 }
+                else if (pid == term_pid) {
+                    term_pid = -1;
+                    if (wm_pid == -1)
+                        runWM();
+                }
                 else if (pid == sound_pid)
                     sound_pid = -1;
                 else if (pid == bg_pid) {
                     bg_pid = -1;
                     checkIcewmbgExitStatus(status);
                 }
+                else if (pid == rescue_pid) {
+                    rescue_pid = -1;
+                }
             }
         }
         else if (sig == SIGUSR1)
@@ -390,6 +463,7 @@ private:
         else if (WIFSIGNALED(status)) {
             tlog(_("%s was killed by signal %d."),
                     icewmExe, WTERMSIG(status));
+            bool terminal = false;
             if (isCoreSignal(WTERMSIG(status))) {
                 if (crashtime == zerotime()) {
                     crashtime = monotime();
@@ -399,13 +473,17 @@ private:
                     crashtime = monotime();
                     double delta = toDouble(crashtime - previous);
                     if (inrange(delta, 0.00001, 10.0)) {
-                        if (enquireRestart() == false) {
+                        if (enquireRestart(&terminal) == false) {
                             return;
                         }
                     }
                 }
             }
-            runWM();
+            if (terminal) {
+                term_pid = runProgram(TERM, nullptr);
+            } else {
+                runWM();
+            }
         }
     }
 
@@ -420,24 +498,42 @@ private:
         return false;
     }
 
-    bool enquireRestart() {
+    bool enquireRestart(bool* terminal, int prefer = 0) {
         const char* message = _(
             " IceWM crashed for the second time in 10 seconds. \n"
             " Do you wish to:\n\n"
             "\t1: Restart IceWM?\n"
             "\t2: Abort this session?\n"
+            "\t3: Run a terminal?\n"
             );
         const char* title = _("IceWM crash response");
+        csmart yadp(path_lookup("yad"));
         const char xmes[] = "/usr/bin/xmessage";
         const char kdia[] = "/usr/bin/kdialog";
         const char zeni[] = "/usr/bin/zenity";
         const size_t commandSize(1234);
         char command[commandSize] = "";
         const int timeout = 123;
-        if (upath(xmes).isExecutable()) {
+        if (--prefer < 0 && upath(yadp).isExecutable()) {
             snprintf(command, commandSize,
                      "%s "
-                     " -buttons Restart,Abort "
+                     " --image=info "
+                     " --center "
+                     " --width=420 "
+                     " --height=200 "
+                     " --timeout=%d "
+                     " --timeout-indicator=bottom "
+                     " --button=Restart:0 "
+                     " --button=Abort:1 "
+                     " --button=Terminal:2 "
+                     " --title='%s' "
+                     " --text='\n%s' ",
+                     (const char *) yadp, timeout, title, message);
+        }
+        else if (--prefer < 0 && upath(xmes).isExecutable()) {
+            snprintf(command, commandSize,
+                     "%s "
+                     " -buttons Restart,Abort,Terminal "
                      " -default Restart "
                      " -font 10x20 "
                      " -background black "
@@ -449,39 +545,45 @@ private:
                      " '\n%s\n' ",
                      xmes, timeout, title, message);
         }
-        else if (upath(kdia).isExecutable()) {
+        else if (--prefer < 0 && upath(kdia).isExecutable()) {
             snprintf(command, commandSize,
                      "%s "
                      " --yes-label Restart "
                      " --no-label Abort "
+                     " --cancel-label Terminal "
                      " --title '%s' "
-                     " --warningyesno "
+                     " --warningyesnocancel "
                      " '\n%s\n' ",
                      kdia, title, message);
         }
-        else if (upath(zeni).isExecutable()) {
+        else if (--prefer < 0 && upath(zeni).isExecutable()) {
             snprintf(command, commandSize,
                      "%s "
                      " --question "
                      " --ok-label Restart "
                      " --cancel-label Abort "
-                     " --timeout %d "
+                     " --timeout=%d "
                      " --title '%s' "
                      " --height 200 "
-                     " --text "
-                     " '%s' ",
+                     " --width 360 "
+                     " --text='%s' ",
                      zeni, timeout, title, message);
         }
         if (command[0] == '/') {
+            forkRescue();
             int status = system(command);
             if (WIFEXITED(status)) {
                 switch (WEXITSTATUS(status)) {
                     case 0:
                     case 5:
+                    case 70:
                     case 101: return true;
 
                     case 1:
                     case 102: this->exit(1); return false;
+
+                    case 2:
+                    case 103: *terminal = true; return true;
                 }
             }
             char *sp = strchr(command, ' ');
@@ -491,6 +593,19 @@ private:
         return true;
     }
 
+    void forkRescue() {
+        if (rescue_pid == -1) {
+            rescue_pid = fork();
+            if (rescue_pid == 0) {
+                resetSignals();
+                rescueFocus();
+                _exit(0);
+            }
+        }
+    }
+
+    void rescueFocus();
+
     void checkIcewmbgExitStatus(int status) {
         if (WIFEXITED(status)) {
             if (WEXITSTATUS(status) == ICESM_EXIT_RESTART) {
@@ -525,14 +640,121 @@ private:
 
 private:
     int startup_phase;
+    int bg_pid;
     int wm_pid;
     int tray_pid;
+    int term_pid;
     int sound_pid;
-    int bg_pid;
+    int rescue_pid;
     timeval crashtime;
-    cstring wmoptions;
+    mstring wmoptions;
+    mstring helptext;
 };
 
+void SessionManager::rescueFocus() {
+    Display* display = XOpenDisplay(nullptr);
+    if (display == nullptr) {
+        return;
+    }
+    XSynchronize(display, True);
+    Window root = DefaultRootWindow(display);
+    XSelectInput(display, root, PropertyChangeMask | SubstructureNotifyMask);
+    const int atomCount = 5;
+    Atom atoms[atomCount] = {
+        XInternAtom(display, "_NET_CLIENT_LIST", True),
+        XInternAtom(display, "_NET_CLIENT_LIST_STACKING", True),
+        XInternAtom(display, "_NET_ACTIVE_WINDOW", True),
+        XInternAtom(display, "_NET_SUPPORTING_WM_CHECK", True),
+        XInternAtom(display, "_NET_SUPPORTED", True),
+    };
+    Window dummy, *tree = None;
+    unsigned num = 0;
+    long inputMask = EnterWindowMask | LeaveWindowMask;
+    if (XQueryTree(display, root, &dummy, &dummy, &tree, &num)) {
+        for (unsigned i = 0; i < num; ++i) {
+            XWindowAttributes attr;
+            if (XGetWindowAttributes(display, tree[i], &attr) &&
+                attr.override_redirect == False &&
+                attr.map_state != IsUnmapped)
+            {
+                XSelectInput(display, tree[i], inputMask);
+            }
+        }
+        XFree(tree);
+    }
+    Window enterWindow = None;
+    double enterTime = 0.0;
+    double delay = 0.2;
+    bool loop = true;
+    while (loop) {
+        fsleep(delay);
+        while (loop && XEventsQueued(display, QueuedAfterReading) > 0) {
+            XEvent e;
+            XNextEvent(display, &e);
+            if (e.type == PropertyNotify) {
+                if (e.xproperty.state == PropertyNewValue) {
+                    for (int i = 0; i < atomCount; ++i) {
+                        if (e.xproperty.atom == atoms[i]) {
+                            loop = false;
+                        }
+                    }
+                }
+            }
+            else if (e.type == MapNotify) {
+                XWindowAttributes attr;
+                if (XGetWindowAttributes(display, e.xmap.window, &attr) &&
+                    attr.override_redirect == False &&
+                    attr.map_state != IsUnmapped)
+                {
+                    XSelectInput(display, e.xmap.window, inputMask);
+                }
+            }
+            else if (e.type == UnmapNotify) {
+                if (enterWindow == e.xunmap.window) {
+                    enterWindow = None;
+                }
+            }
+            else if (e.type == DestroyNotify) {
+                if (enterWindow == e.xdestroywindow.window) {
+                    enterWindow = None;
+                }
+            }
+            else if (e.type == EnterNotify) {
+                enterWindow = e.xcrossing.window;
+                enterTime = toDouble(monotime());
+            }
+            else if (e.type == LeaveNotify) {
+                if (e.xcrossing.window == enterWindow &&
+                    e.xcrossing.mode == NotifyNormal &&
+                    e.xcrossing.detail != NotifyInferior)
+                {
+                    enterWindow = None;
+                }
+            }
+        }
+        if (loop) {
+            int revert = RevertToNone;
+            Window win = None;
+            XGetInputFocus(display, &win, &revert);
+            if (win == None) {
+                XSetInputFocus(display, PointerRoot,
+                               RevertToPointerRoot,
+                               CurrentTime);
+            }
+            if (enterWindow) {
+                double now = toDouble(monotime());
+                if (now - enterTime >= delay * 0.9) {
+                    XSetInputFocus(display, enterWindow,
+                                   RevertToPointerRoot,
+                                   CurrentTime);
+                    enterWindow = None;
+                }
+            }
+        }
+    }
+    XCloseDisplay(display);
+}
+
 int main(int argc, char **argv) {
     SessionManager xapp(&argc, &argv);
 
