@@ -17,11 +17,7 @@
 #define CFGDEF
 #include "icewmbg_prefs.h"
 
-char const *ApplicationName = NULL;
-
-// used in YXFont but declared only in icewm sources, not in icewmbg special set.
-// Declaring it here for now.
-XIV(bool, fontPreferFreetype,                   true)
+char const *ApplicationName = nullptr;
 
 class PixFile {
 public:
@@ -42,7 +38,7 @@ public:
     }
     mstring file() const { return name; }
     Pixmap pixmap(Paths paths) {
-        time_t now = time(NULL);
+        time_t now = time(nullptr);
         if (pix == null) {
             if (last == 0 || last + 2 < now) {
                 pix = load(paths);
@@ -75,7 +71,7 @@ public:
             image = YPixmap::load(path);
         }
         if (image == null) {
-            tlog(_("Failed to load image '%s'."), cstring(name).c_str());
+            tlog(_("Failed to load image '%s'."), name.c_str());
         }
         return image;
     }
@@ -218,7 +214,7 @@ private:
     ref<YPixmap> currentBackgroundPixmap;
     ref<YPixmap> currentTransparencyPixmap;
     YColor currentBackgroundColor;
-    cstring pixmapName;
+    mstring pixmapName;
     YArray<int> sequence;
     lazy<YTimer> cycleTimer;
 
@@ -276,7 +272,7 @@ upath Background::getThemeDir() {
     if (themeDir == null && nonempty(themeName)) {
         upath path(themeName);
         if (path.isAbsolute() == false) {
-            if (strchr(themeName, '/') == 0) {
+            if (strchr(themeName, '/') == nullptr) {
                 path = upath("themes") + path + "default.theme";
             }
             else {
@@ -405,7 +401,7 @@ void Background::addImage(Strings& images, const char* name, bool append) {
 }
 
 void Background::add(const char* name, const char* value, bool append) {
-    if (value == 0 || *value == 0) {
+    if (value == nullptr || *value == 0) {
     }
     else if (0 == strcmp(name, "DesktopBackgroundImage")) {
         addImage(backgroundImages, value, append);
@@ -568,7 +564,7 @@ long* Background::getLongProperties(Atom property, int n) const {
     Atom r_type;
     int r_format;
     unsigned long nitems, lbytes;
-    unsigned char *prop(0);
+    unsigned char *prop(nullptr);
 
     if (XGetWindowProperty(display(), window(), property,
                            0, n, False, XA_CARDINAL,
@@ -581,7 +577,7 @@ long* Background::getLongProperties(Atom property, int n) const {
         }
         XFree(prop);
     }
-    return 0;
+    return nullptr;
 }
 
 int Background::getLongProperty(Atom property) const {
@@ -780,7 +776,7 @@ void Background::changeBackground(bool force) {
 void Background::restart() const {
     if (verbose) tlog("restart");
     if (mainArgv[0][0] == '/' ||
-        (strchr(mainArgv[0], '/') != 0 &&
+        (strchr(mainArgv[0], '/') != nullptr &&
          access(mainArgv[0], X_OK) == 0))
     {
         execv(mainArgv[0], mainArgv);
@@ -948,7 +944,7 @@ void addBgImage(const char* name, const char* value, bool append) {
 
 static void bgLoadConfig(const char *configFile, const char *overrideTheme)
 {
-    if (configFile == 0 || *configFile == 0)
+    if (configFile == nullptr || *configFile == 0)
         configFile = "preferences";
     if (overrideTheme && *overrideTheme)
         themeName = overrideTheme;
@@ -958,19 +954,9 @@ static void bgLoadConfig(const char *configFile, const char *overrideTheme)
             OSV("Theme", &themeName, "Theme name"),
             OK0()
         };
-
-        YConfig::findLoadConfigFile(globalBg, theme_prefs, configFile);
-        YConfig::findLoadConfigFile(globalBg, theme_prefs, "theme");
+        YConfig(theme_prefs).load(configFile).load("theme");
     }
-    YConfig::findLoadConfigFile(globalBg, icewmbg_prefs, configFile);
-    if (themeName != 0) {
-        MSG(("themeName=%s", themeName));
-
-        YConfig::findLoadThemeFile(globalBg, icewmbg_prefs,
-                *themeName == '/' ? themeName :
-                upath("themes").child(themeName));
-    }
-    YConfig::findLoadConfigFile(globalBg, icewmbg_prefs, "prefoverride");
+    YConfig(icewmbg_prefs).load(configFile).loadTheme().loadOverride();
 }
 
 static void bgParse(const char* name, const char* value) {
@@ -978,7 +964,7 @@ static void bgParse(const char* name, const char* value) {
     for (int i = 0; str.splitall(',', &opt, &str); ++i) {
         if (i + 1 < ICEBG_MAX_ARGS) {
             if ((opt = opt.trim()).nonempty()) {
-                globalBg->add(name, cstring(opt), 0 < i);
+                globalBg->add(name, opt, 0 < i);
             }
         }
     }
@@ -1001,21 +987,21 @@ int main(int argc, char **argv) {
     bool replace = false;
     bool shuffle = false;
     bool verbose = false;
-    const char* overrideTheme = 0;
-    const char* configFile = 0;
-    const char* image = 0;
-    const char* color = 0;
-    const char* trans = 0;
-    const char* semis = 0;
-    const char* center = 0;
-    const char* scaled = 0;
-    const char* multi = 0;
-    const char* cycle = 0;
-    const char* shuffleArg = 0;
+    const char* overrideTheme = nullptr;
+    const char* configFile = nullptr;
+    const char* image = nullptr;
+    const char* color = nullptr;
+    const char* trans = nullptr;
+    const char* semis = nullptr;
+    const char* center = nullptr;
+    const char* scaled = nullptr;
+    const char* multi = nullptr;
+    const char* cycle = nullptr;
+    const char* shuffleArg = nullptr;
 
     for (char **arg = argv + 1; arg < argv + argc; ++arg) {
         if (**arg == '-') {
-            char *value(0);
+            char *value(nullptr);
             if (is_switch(*arg, "r", "restart")) {
                 sendRestart = true;
             }
@@ -1132,7 +1118,7 @@ int main(int argc, char **argv) {
         cycleBackgroundsPeriod = atoi(cycle);
     }
 
-    globalBg = NULL;
+    globalBg = nullptr;
 
     return bg.mainLoop();
 }
