@@ -24,9 +24,10 @@ ObjectBar::ObjectBar(YWindow *parent): YWindow(parent) {
 }
 
 ObjectBar::~ObjectBar() {
+    ObjectButton::freeFont();
 }
 
-void ObjectBar::addButton(const ustring &name, ref<YIcon> icon, ObjectButton *button) {
+void ObjectBar::addButton(mstring name, ref<YIcon> icon, ObjectButton *button) {
     button->setToolTip(name);
     if (icon != null) {
         button->setIcon(icon, YIcon::smallSize());
@@ -56,6 +57,7 @@ void ObjectBar::paint(Graphics &g, const YRect& r) {
 
 void ObjectBar::addObject(DObject *object) {
     ObjectButton *button = new ObjectButton(this, object);
+    button->setTitle(object->getName().c_str());
     addButton(object->getName(), object->getIcon(), button);
 }
 
@@ -64,9 +66,10 @@ void ObjectBar::addSeparator() {
     objects.append(0);
 }
 
-void ObjectBar::addContainer(const ustring &name, ref<YIcon> icon, ObjectMenu *container) {
+void ObjectBar::addContainer(mstring name, ref<YIcon> icon, ObjectMenu *container) {
     if (container) {
         ObjectButton *button = new ObjectButton(this, container);
+        button->setTitle(name.c_str());
         addButton(name, icon, button);
     }
 }
@@ -160,4 +163,12 @@ void ObjectButton::handleButton(const XButtonEvent& up) {
     }
 }
 
+void ObjectBar::refresh() {
+    for (IterType iter(objects.iterator()); ++iter; ) {
+        if (*iter) {
+            iter->repaint();
+        }
+    }
+}
+
 // vim: set sw=4 ts=4 et:
