@@ -4,10 +4,7 @@
  * Copyright (C) 1997-2002 Marko Macek
  */
 #include "config.h"
-
-#ifndef NO_CONFIGURE_MENUS
 #include "themes.h"
-
 #include "yapp.h"
 #include "ymenu.h"
 #include "wmmgr.h"
@@ -26,13 +23,10 @@
 
 #include "intl.h"
 
-static void setDefaultTheme(const char *theme) {
-    char buf[600];
-    snprintf(buf, sizeof buf, "Theme=\"%s\"\n", theme);
-    WMConfig::setDefault("theme", buf);
-}
+const unsigned utf32ellipsis = 0x2026;
+extern ref<YFont> menuFont;
 
-DTheme::DTheme(IApp *app, YSMListener *smActionListener, const ustring &label, const ustring &theme):
+DTheme::DTheme(IApp *app, YSMListener *smActionListener, const mstring &label, const mstring &theme):
     DObject(app, label, null), fTheme(theme)
 {
     this->app = app;
@@ -46,10 +40,9 @@ void DTheme::open() {
     if (fTheme == null)
         return;
 
-    cstring cTheme(fTheme);
-    setDefaultTheme(cTheme.c_str());
+    WMConfig::setDefaultTheme(fTheme);
 
-    const char *bg[] = { ICEWMBGEXE, "-r", 0 };
+    const char *bg[] = { ICEWMBGEXE, "-r", nullptr };
     int pid = app->runProgram(bg[0], bg);
     app->waitProgram(pid);
 
@@ -69,23 +62,20 @@ void ThemesMenu::updatePopup() {
 void ThemesMenu::refresh() {
     removeAll();
 
-    pstring themes("/themes/");
+    mstring themes("/themes/");
     upath libThemes = YApplication::getLibDir() + themes;
     upath cnfThemes = YApplication::getConfigDir() + themes;
     upath prvThemes = YApplication::getPrivConfDir() + themes;
-    upath xdgThemes = YApplication::getXdgConfDir() + themes;
 
     if (nestedThemeMenuMinNumber)
         themeCount =
             countThemes(libThemes) +
             countThemes(cnfThemes) +
-            countThemes(prvThemes) +
-            countThemes(xdgThemes);
+            countThemes(prvThemes);
 
-    findThemes(libThemes, this);
-    findThemes(cnfThemes, this);
     findThemes(prvThemes, this);
-    findThemes(xdgThemes, this);
+    findThemes(cnfThemes, this);
+    findThemes(libThemes, this);
 
     addSeparator();
     add(newThemeItem(app, smActionListener, _("Default"), CONFIG_DEFAULT_THEME));
@@ -105,8 +95,8 @@ ThemesMenu::~ThemesMenu() {
 YMenuItem * ThemesMenu::newThemeItem(
         IApp *app,
         YSMListener *smActionListener,
-        const ustring& label,
-        const ustring& relThemeName) {
+        const mstring& label,
+        const mstring& relThemeName) {
     DTheme *dtheme = new DTheme(app, smActionListener, label, relThemeName);
 
     if (dtheme) {
@@ -117,49 +107,81 @@ YMenuItem * ThemesMenu::newThemeItem(
             return item;
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 void ThemesMenu::findThemes(const upath& path, YMenu *container) {
-    ustring defTheme("/default.theme");
+    mstring defTheme("/default.theme");
 
-    bool bNesting = nestedThemeMenuMinNumber && themeCount>nestedThemeMenuMinNumber;
+    bool bNesting = inrange(nestedThemeMenuMinNumber, 1, themeCount - 1);
+    char subName[5];
+    auto canUcodeEl = menuFont != null && menuFont->supports(utf32ellipsis);
+    memcpy(&subName, canUcodeEl ? "X\xe2\x80\xa6" : "X...", 5);
 
     for (udir dir(path); dir.next(); ) {
-        YMenuItem *im(NULL);
+        YMenuItem *im(nullptr);
+        YMenu* targetMenu = container;
         upath subdir = path + dir.entry();
         upath defThemePath = subdir + defTheme;
 
         if (defThemePath.isReadable()) {
-            ustring relThemeName = dir.entry() + defTheme;
+            mstring relThemeName = dir.entry() + defTheme;
             im = newThemeItem(app, smActionListener, dir.entry(), relThemeName);
         }
-        if (im && bNesting) {
-            char fLetter = ASCII::toUpper(dir.entry()[0]);
-            int targetItem = container->findFirstLetRef(fLetter, 0, 1);
-            if (targetItem < 0) // no submenu for us yet, create one
-            {
-                char smname[] = "....";
-                smname[0] = fLetter;
 
+        // maybe shift some stuff around to create a simple structure
+        if (im && bNesting) {
+                char fLetter = ASCII::toUpper(dir.entry()[0]);
+                subName[0] = fLetter;
+            int relatedItemPos = -2;
+
+            YMenuItem *subMenuItemTest = container->findName(subName);
+            if (subMenuItemTest && subMenuItemTest->getSubmenu())
+            {
+                targetMenu = subMenuItemTest->getSubmenu();
+                if (im->isChecked())
+                        subMenuItemTest->setChecked(true);
+            }
+            else if (subMenuItemTest)
+            {
+                // looks like a submenu but is an item
+                // of that kind... weird, ignore
+            }
+            else if (0 > (relatedItemPos = container->findFirstLetRef(fLetter, 0, 1)))
+            {
+                MSG(("adding: %s to main menu", subdir.string()));
+            }
+            else
+            {
+                // ok, have the position of the related entry
+                // which needs to be moved to submenu
+                YMenuItem *relatedItem = container->getItem(relatedItemPos);
+                MSG(("Moving %s to submenu to prepare for %s",
+                                relatedItem->getName().c_str(),
+                                subdir.string()));
                 YMenu *smenu = new YMenu();
-                YMenuItem *smItem = new YMenuItem(smname, 0, null, NULL, smenu);
-                if(smItem && smenu)
-                    container->addSorted(smItem, false);
-                targetItem = container->findFirstLetRef(fLetter, 0, 1);
-                if (targetItem < 0)
-                {
-                    warn("Failed to add submenu");
-                    return;
-                }
+                smenu->addSorted(relatedItem, false, true);
+                YMenuItem *newItem = new YMenuItem(subName, 0, null, actionNull, smenu);
+                newItem->setChecked(relatedItem->isChecked() || im->isChecked());
+                container->setItem(relatedItemPos, newItem);
+                targetMenu = smenu;
             }
-            container->getItem(targetItem)->getSubmenu()->addSorted(im, false);
         }
-        else if (im) { //the default method without Extra SubMenues
-            container->addSorted(im, false);
+
+        if (im) {
+            if (targetMenu->addSorted(im, false, true) == nullptr) {
+                delete im;
+                im = nullptr;
+            }
         }
         if (im) {
             findThemeAlternatives(app, smActionListener, subdir, dir.entry(), im);
+            if (im->isChecked()) {
+                YMenuItem *sub = container->findName(subName);
+                if (sub && sub->getSubmenu()) {
+                    sub->setChecked(true);
+                }
+            }
         }
     }
 }
@@ -169,31 +191,35 @@ void ThemesMenu::findThemeAlternatives(
     IApp *app,
     YSMListener *smActionListener,
     const upath& path,
-    const ustring& relName,
+    const mstring& relName,
     YMenuItem *item)
 {
-    ustring defTheme("default.theme");
-    ustring extension(".theme");
+    mstring defTheme("default.theme");
+    mstring extension(".theme");
     for (udir dir(path); dir.nextExt(extension); ) {
-        const ustring entry(dir.entry());
+        const mstring entry(dir.entry());
         if (entry != defTheme) {
             upath altThemePath(path + entry);
 
             if (altThemePath.isReadable()) {
                 YMenu *sub(item->getSubmenu());
 
-                if (sub == NULL)
+                if (sub == nullptr)
                     item->setSubmenu(sub = new YMenu());
 
                 if (sub) {
                     int prefixLength = entry.length() - extension.length();
-                    ustring tname(entry.substring(0, prefixLength));
-                    ustring relThemeName = upath(relName) + entry;
-                    sub->add(newThemeItem(app, smActionListener,
-                                tname, relThemeName));
+                    mstring tname(entry.substring(0, prefixLength));
+                    mstring relThemeName = upath(relName) + entry;
+                    YMenuItem *im = newThemeItem(app, smActionListener,
+                                tname, relThemeName);
+                    sub->add(im);
+                    if (im->isChecked())
+                        item->setChecked(true);
                 }
             }
         }
     }
 }
-#endif
+
+// vim: set sw=4 ts=4 et:
