@@ -1,11 +1,12 @@
-#ifndef __WMFRAME_H
-#define __WMFRAME_H
+#ifndef WMFRAME_H
+#define WMFRAME_H
 
 #include "ymsgbox.h"
 #include "wmoption.h"
-#include "wmmgr.h"
 #include "yicon.h"
 #include "ylist.h"
+#include "WinMgr.h"
+#include "workspaces.h"
 
 class YClientContainer;
 class MiniIcon;
@@ -16,9 +17,9 @@ class YFrameTitleBar;
 class YFrameWindow:
     public YWindow,
     public YActionListener,
-    public YTimerListener,
-    public YPopDownListener,
-    public YMsgBoxListener,
+    private YTimerListener,
+    private YPopDownListener,
+    private YMsgBoxListener,
     public ClientData,
     public YLayeredNode,
     public YCreatedNode,
@@ -33,11 +34,11 @@ public:
 
     void doManage(YFrameClient *client, bool &doActivate, bool &requestFocus);
     void afterManage();
-    void manage(YFrameClient *client);
+    void manage();
     void unmanage(bool reparent = true);
     void sendConfigure();
 
-    Window createPointerWindow(Cursor cursor, Window parent);
+    Window createPointerWindow(Cursor cursor, int gravity);
     void createPointerWindows();
     void grabKeys();
 
@@ -59,7 +60,7 @@ public:
 
     virtual bool handleTimer(YTimer *t);
 
-    virtual void actionPerformed(YAction action, unsigned int modifiers);
+    virtual void actionPerformed(YAction action, unsigned modifiers = 0);
     virtual void handleMsgBox(YMsgBox *msgbox, int operation);
     virtual YFrameWindow* frame() { return this; }
 
@@ -77,7 +78,7 @@ public:
     void doRaise();
     void wmClose();
     void wmConfirmKill();
-    static YMsgBox* wmConfirmKill(const ustring& title, YMsgBoxListener *recvr);
+    static YMsgBox* wmConfirmKill(const char* title, YMsgBoxListener *recvr);
     void wmKill();
     void wmNextWindow();
     void wmPrevWindow();
@@ -101,7 +102,7 @@ public:
     void hideTransients();
     void restoreHiddenTransients();
 
-    void DoMaximize(long flags);
+    void doMaximize(long flags);
 
     void loseWinFocus();
     void setWinFocus();
@@ -110,7 +111,7 @@ public:
 
     YFrameClient *client() const { return fClient; }
     YFrameTitleBar *titlebar();
-    YClientContainer *container() const { return fClientContainer; }
+    YClientContainer *container() const { return fContainer; }
 
     void startMoveSize(int x, int y, int direction);
 
@@ -142,24 +143,25 @@ public:
 
     void getDefaultOptions(bool &doActivate);
 
-    bool canSize(bool boriz = true, bool vert = true);
-    bool canMove() const;
-    bool canClose() const;
-    bool canMaximize() const;
-    bool canMinimize() const;
+    bool canSize(bool horiz = true, bool vert = true);
+    bool canMove() const { return hasbit(frameFunctions(), ffMove); }
+    bool canClose() const { return hasbit(frameFunctions(), ffClose); }
+    bool canMaximize() const { return hasbit(frameFunctions(), ffMaximize); }
+    bool canMinimize() const { return hasbit(frameFunctions(), ffMinimize); }
     bool canRestore() const;
-    bool canRollup() const;
-    bool canHide() const;
+    bool canRollup() const { return (frameFunctions() & ffRollup) && titleY(); }
+    bool canShow() const;
+    bool canHide() const { return hasbit(frameFunctions(), ffHide); }
     bool canLower() const;
-    bool canRaise();
-    bool canFullscreen() { return true; }
+    bool canRaise() const;
+    bool canFullscreen() const;
     bool overlaps(bool below);
     unsigned overlap(YFrameWindow *other);
 
     void insertFrame(bool top);
     void removeFrame();
-    void setAbove(YFrameWindow *aboveFrame); // 0 = at the bottom
-    void setBelow(YFrameWindow *belowFrame); // 0 = at the top
+    bool setAbove(YFrameWindow *aboveFrame);
+    bool setBelow(YFrameWindow *belowFrame);
 
     enum FindWindowFlags {
         fwfVisible    = 1 << 0, // visible windows only
@@ -187,7 +189,7 @@ public:
 
     void popupSystemMenu(YWindow *owner, int x, int y,
                          unsigned int flags,
-                         YWindow *forWindow = 0);
+                         YWindow *forWindow = nullptr);
     virtual void popupSystemMenu(YWindow *owner);
     virtual void handlePopDown(YPopupWindow *popup);
 
@@ -197,10 +199,9 @@ public:
                    int &cx, int &cy, int &cw, int &ch);
     void configureClient(const XConfigureRequestEvent &configureRequest);
     void configureClient(int cx, int cy, int cwidth, int cheight);
+    void netRestackWindow(Window window, int detail);
 
-#ifdef CONFIG_SHAPE
     void setShape();
-#endif
 
     enum YFrameFunctions {
         ffMove          = (1 << 0),
@@ -208,21 +209,21 @@ public:
         ffClose         = (1 << 2),
         ffMinimize      = (1 << 3),
         ffMaximize      = (1 << 4),
-        ffHide          = (1 << 5),
-        ffRollup        = (1 << 6)
+        ffRollup        = (1 << 5),
+        ffHide          = (1 << 6),
     };
 
     enum YFrameDecors {
         fdTitleBar      = (1 << 0),
-        fdSysMenu       = (1 << 1),
-        fdBorder        = (1 << 2),
-        fdResize        = (1 << 3),
-        fdClose         = (1 << 4),
-        fdMinimize      = (1 << 5),
-        fdMaximize      = (1 << 6),
-        fdHide          = (1 << 7),
-        fdRollup        = (1 << 8),
-        fdDepth         = (1 << 9)
+        fdResize        = (1 << 1),
+        fdClose         = (1 << 2),
+        fdMinimize      = (1 << 3),
+        fdMaximize      = (1 << 4),
+        fdRollup        = (1 << 5),
+        fdHide          = (1 << 6),
+        fdDepth         = (1 << 7),
+        fdBorder        = (1 << 8),
+        fdSysMenu       = (1 << 9),
     };
 
     enum YFrameOptions {
@@ -244,29 +245,38 @@ public:
         foIgnoreTaskBar            = (1 << 14),
         foIgnoreUrgent             = (1 << 15),
         foIgnoreWinList            = (1 << 16),
-        foNoFocusOnAppRaise        = (1 << 17),
-        foNoFocusOnMap             = (1 << 18),
-        foNoIgnoreTaskBar          = (1 << 19),
-        foNonICCCMConfigureRequest = (1 << 20),
-        foClose                    = (1 << 21),
+        foIgnoreActivationMessages = (1 << 17),
+        foNoFocusOnAppRaise        = (1 << 18),
+        foNoFocusOnMap             = (1 << 19),
+        foNoIgnoreTaskBar          = (1 << 20),
+        foNonICCCMConfigureRequest = (1 << 21),
+        foClose                    = (1 << 22),
     };
 
     unsigned frameFunctions() const { return fFrameFunctions; }
     unsigned frameDecors() const { return fFrameDecors; }
     unsigned frameOptions() const { return fFrameOptions; }
+    bool frameOption(YFrameOptions o) const { return hasbit(fFrameOptions, o); }
     void updateAllowed();
-    void updateNetWMState();
     void getFrameHints();
-    void getWindowOptions(WindowOption &opt, bool remove); /// !!! fix kludges
-    void getWindowOptions(WindowOptions *list, WindowOption &opt, bool remove);
+    bool haveHintOption() const { return fHintOption; }
+    WindowOption& getHintOption() { return *fHintOption; }
+    WindowOption getWindowOption();
+    void getWindowOptions(WindowOptions* list, WindowOption& opt, bool remove);
 
     YMenu *windowMenu();
 
     long getState() const { return fWinState; }
     void setState(long mask, long state);
+    bool hasState(long bit) const { return hasbit(fWinState, bit); }
+    bool notState(long bit) const { return !hasbit(fWinState, bit); }
 
-    bool isFullscreen() const { return (getState() & WinStateFullscreen); }
-
+    bool isFullscreen() const { return hasState(WinStateFullscreen); }
+    bool isResizable() const { return hasbit(frameFunctions(), ffResize); }
+    bool isUnmapped() const { return hasState(WinStateUnmapped); }
+    bool isMapped() const { return notState(WinStateUnmapped); }
+    void makeMapped() { return setState(WinStateUnmapped, None); }
+    bool hasBorders() const;
     int borderXN() const;
     int borderYN() const;
     int titleYN() const;
@@ -285,7 +295,7 @@ public:
     WindowListItem *winListItem() const { return fWinListItem; }
     void setWinListItem(WindowListItem *i) { fWinListItem = i; }
 
-    void addAsTransient();
+    bool addAsTransient();
     void removeAsTransient();
     void addTransients();
     void removeTransients();
@@ -301,7 +311,7 @@ public:
     ref<YIcon> getClientIcon() const { return fFrameIcon; }
     ref<YIcon> clientIcon() const;
 
-    void getNormalGeometryInner(int *x, int *y, int *w, int *h);
+    void getNormalGeometryInner(int *x, int *y, int *w, int *h) const;
     void setNormalGeometryOuter(int x, int y, int w, int h);
     void setNormalPositionOuter(int x, int y);
     void setNormalGeometryInner(int x, int y, int w, int h);
@@ -309,6 +319,7 @@ public:
 
     void setCurrentGeometryOuter(YRect newSize);
     void setCurrentPositionOuter(int x, int y);
+    void limitOuterPosition();
     void updateNormalSize();
 
     void updateTitle();
@@ -316,67 +327,45 @@ public:
     void updateIcon();
     void updateState();
     void updateLayer(bool restack = true);
-    //void updateWorkspace();
+    void updateIconPosition();
     void updateLayout();
-    void updateExtents();
     void performLayout();
 
-    void updateMwmHints();
+    void updateMwmHints(XSizeHints* sh);
     void updateProperties();
     void updateTaskBar();
+    void updateAppStatus();
 
-    enum WindowType {
-        wtCombo,
-        wtDesktop,
-        wtDialog,
-        wtDND,
-        wtDock,
-        wtDropdownMenu,
-        wtMenu,
-        wtNormal,
-        wtNotification,
-        wtPopupMenu,
-        wtSplash,
-        wtToolbar,
-        wtTooltip,
-        wtUtility
-    };
-
-    void setWindowType(enum WindowType winType) { fWindowType = winType; }
+    void setWindowType(WindowType winType) { fWindowType = winType; }
     bool isTypeDock() { return (fWindowType == wtDock); }
 
     int getWorkspace() const { return fWinWorkspace; }
     int getTrayOrder() const { return fTrayOrder; }
     void setWorkspace(int workspace);
-    void setWorkspaceHint(long workspace);
     long getActiveLayer() const { return fWinActiveLayer; }
     void setRequestedLayer(long layer);
     long getRequestedLayer() const { return fWinRequestedLayer; }
     long getTrayOption() const { return fWinTrayOption; }
     void setTrayOption(long option);
     void setDoNotCover(bool flag);
-    bool isMaximized() const { return (getState() &
-                                 (WinStateMaximizedHoriz |
-                                  WinStateMaximizedVert)) ? true : false; }
-    bool isMaximizedVert() const { return (getState() & WinStateMaximizedVert) ? true : false; }
-    bool isMaximizedHoriz() const { return (getState() & WinStateMaximizedHoriz) ? true : false; }
+    bool isMaximized() const { return hasState(WinStateMaximizedBoth); }
+    bool isMaximizedVert() const { return hasState(WinStateMaximizedVert); }
+    bool isMaximizedHoriz() const { return hasState(WinStateMaximizedHoriz); }
     bool isMaximizedFully() const { return isMaximizedVert() && isMaximizedHoriz(); }
-    bool isMinimized() const { return (getState() & WinStateMinimized) ? true : false; }
-    bool isHidden() const { return (getState() & WinStateHidden) ? true : false; }
-    bool isSkipPager() const { return (getState() & WinStateSkipPager) ? true : false; }
-    bool isSkipTaskBar() const { return (getState() & WinStateSkipTaskBar) ? true : false; }
-    bool isRollup() const { return (getState() & WinStateRollup) ? true : false; }
-    bool isSticky() const { return (getState() & WinStateSticky) ? true : false; }
-    bool isAllWorkspaces() const { return (getWorkspace() == -1) ? true : false; }
-    //bool isHidWorkspace() { return (getState() & WinStateHidWorkspace) ? true : false; }
-    //bool isHidTransient() { return (getState() & WinStateHidTransient) ? true : false; }
-
-    bool wasMinimized() const { return (getState() & WinStateWasMinimized) ? true : false; }
-    bool wasHidden() const { return (getState() & WinStateWasHidden) ? true : false; }
+    bool isMinimized() const { return hasState(WinStateMinimized); }
+    bool isHidden() const { return hasState(WinStateHidden); }
+    bool isSkipPager() const { return hasState(WinStateSkipPager); }
+    bool isSkipTaskBar() const { return hasState(WinStateSkipTaskBar); }
+    bool isRollup() const { return hasState(WinStateRollup); }
+    bool isSticky() const { return hasState(WinStateSticky); }
+    bool isAllWorkspaces() const { return (getWorkspace() == AllWorkspaces); }
+    bool wasMinimized() const { return hasState(WinStateWasMinimized); }
+    bool wasHidden() const { return hasState(WinStateWasHidden); }
 
     bool isIconic() const { return isMinimized() && fMiniIcon; }
-
+    bool hasMiniIcon() const { return fMiniIcon != nullptr; }
     MiniIcon *getMiniIcon();
+    ClassHint* classHint() const { return client()->classHint(); }
 
     bool isManaged() const { return fManaged; }
     void setManaged(bool isManaged) { fManaged = isManaged; }
@@ -386,7 +375,7 @@ public:
     bool visibleOn(int workspace) const {
         return (isAllWorkspaces() || getWorkspace() == workspace);
     }
-    bool visibleNow() const { return visibleOn(manager->activeWorkspace()); }
+    bool visibleNow() const;
 
     bool isModal();
     bool hasModal();
@@ -398,14 +387,12 @@ public:
     bool inWorkArea() const;
     bool affectsWorkArea() const;
 
-    bool doNotCover() const {
-        return (frameOptions() & foDoNotCover) ? true : false;
-    }
+    bool doNotCover() const { return frameOption(foDoNotCover); }
 
     virtual ref<YIcon> getIcon() const { return clientIcon(); }
 
-    virtual ustring getTitle() const { return client()->windowTitle(); }
-    virtual ustring getIconTitle() const { return client()->iconTitle(); }
+    virtual mstring getTitle() const { return client()->windowTitle(); }
+    virtual mstring getIconTitle() const { return client()->iconTitle(); }
 
     void updateNetWMStrut();
     void updateNetWMStrutPartial();
@@ -413,7 +400,7 @@ public:
     void updateNetWMUserTime();
     void updateNetWMUserTimeWindow();
     void updateNetWMWindowOpacity();
-    void updateNetWMFullscreenMonitors(int, int, int, int);
+    void updateNetWMFullscreenMonitors(int top, int bottom, int left, int right);
 
     int strutLeft() { return fStrutLeft; }
     int strutRight() { return fStrutRight; }
@@ -421,21 +408,20 @@ public:
     int strutBottom() { return fStrutBottom; }
     bool haveStruts() const { return fHaveStruts; }
 
-    void updateUrgency();
     void setWmUrgency(bool wmUrgency);
-    bool isUrgent() { return fWmUrgency || fClientUrgency; }
+    bool isUrgent() const;
 
-    int getScreen();
+    int getScreen() const;
+    void refresh();
 
-    long getOldLayer() { return fOldLayer; }
-    void saveOldLayer() { fOldLayer = fWinActiveLayer; }
+    long windowTypeLayer() const;
 
     bool hasIndicators() const { return indicatorsCreated; }
     Window topSideIndicator() const { return topSide; }
     Window topLeftIndicator() const { return topLeft; }
     Window topRightIndicator() const { return topRight; }
     Time since() const { return fStartManaged; }
-    bool startMinimized() const { return hasbit(fFrameOptions, foMinimized); }
+    bool startMinimized() const { return frameOption(foMinimized); }
 
     void addToWindowList();
     void removeFromWindowList();
@@ -457,12 +443,9 @@ private:
 
     int normalX, normalY, normalW, normalH;
     int posX, posY, posW, posH;
-    int extentLeft, extentRight, extentTop, extentBottom;
-
-    int iconX, iconY;
 
     YFrameClient *fClient;
-    YClientContainer *fClientContainer;
+    YClientContainer *fContainer;
     YFrameTitleBar *fTitleBar;
 
     YPopupWindow *fPopupActive;
@@ -474,14 +457,13 @@ private:
 
     Window topSide, leftSide, rightSide, bottomSide;
     Window topLeft, topRight, bottomLeft, bottomRight;
-    bool indicatorsCreated;
-    bool indicatorsVisible;
 
     TaskBarApp *fTaskBarApp;
     TrayApp *fTrayApp;
     MiniIcon *fMiniIcon;
     WindowListItem *fWinListItem;
     ref<YIcon> fFrameIcon;
+    lazy<WindowOption> fHintOption;
 
     YFrameWindow *fOwner;
     YFrameWindow *fTransient;
@@ -497,7 +479,6 @@ private:
     long fWinTrayOption;
     long fWinState;
     long fWinOptionMask;
-    long fOldLayer;
     int fTrayOrder;
 
     int fFullscreenMonitorsTop;
@@ -523,10 +504,11 @@ private:
     int fShapeTitleY;
     int fShapeBorderX;
     int fShapeBorderY;
+    unsigned fShapeDecors;
+    mstring fShapeTitle;
 
     bool fHaveStruts;
-    bool fWmUrgency;
-    bool fClientUrgency;
+    bool indicatorsCreated;
 
     enum WindowType fWindowType;
 
@@ -549,13 +531,12 @@ private:
     //static int fMouseFocusX, fMouseFocusY;
 
     void repaint();
-    void setGeometry(const YRect &);
-    void setPosition(int, int);
-    void setSize(int, int);
+    void setGeometry(const YRect &) = delete;
+    void setPosition(int, int) = delete;
+    void setSize(int, int) = delete;
     void setWindowGeometry(const YRect &r) {
         YWindow::setGeometry(r);
     }
-    friend class MiniIcon;
 };
 
 #endif
