@@ -21,8 +21,12 @@
 #include <regex.h>
 #include "intl.h"
 
+#define utf8ellipsis "\xe2\x80\xa6"
+const unsigned utf32ellipsis = 0x2026;
+extern ref<YFont> menuFont;
+
 DObjectMenuItem::DObjectMenuItem(DObject *object):
-    YMenuItem(object->getName(), -3, null, YAction(), 0)
+    YMenuItem(object->getName(), -3, null, YAction(), nullptr)
 {
     fObject = object;
     if (object->getIcon() != null)
@@ -38,7 +42,7 @@ void DObjectMenuItem::actionPerformed(YActionListener * /*listener*/, YAction /*
     fObject->open();
 }
 
-DFile::DFile(IApp *app, const ustring &name, ref<YIcon> icon, upath path): DObject(app, name, icon) {
+DFile::DFile(IApp *app, const mstring &name, ref<YIcon> icon, upath path): DObject(app, name, icon) {
     this->app = app;
     fPath = path;
 }
@@ -47,17 +51,16 @@ DFile::~DFile() {
 }
 
 void DFile::open() {
-    const char *args[] = { openCommand, fPath.string(), 0 };
+    const char *args[] = { openCommand, fPath.string(), nullptr };
     app->runProgram(openCommand, args);
 }
 
-ObjectMenu::ObjectMenu(YActionListener *actionListener, YWindow *parent): YMenu(parent) {
+ObjectMenu::ObjectMenu(YActionListener *actionListener, YWindow *parent):
+        YMenu(parent),
+        wmActionListener(nullptr) {
     setActionListener(actionListener);
 }
 
-ObjectMenu::~ObjectMenu() {
-}
-
 void ObjectMenu::addObject(DObject *fObject) {
     add(new DObjectMenuItem(fObject));
 }
@@ -70,7 +73,7 @@ void ObjectMenu::addSeparator() {
     YMenu::addSeparator();
 }
 
-void ObjectMenu::addContainer(const ustring &name, ref<YIcon> icon, ObjectMenu *container) {
+void ObjectMenu::addContainer(mstring name, ref<YIcon> icon, ObjectMenu *container) {
     if (container) {
         YMenuItem *item =
             addSubmenu(name, -3, container);
@@ -80,7 +83,7 @@ void ObjectMenu::addContainer(const ustring &name, ref<YIcon> icon, ObjectMenu *
     }
 }
 
-DObject::DObject(IApp *app, const ustring &name, ref<YIcon> icon):
+DObject::DObject(IApp *app, const mstring &name, ref<YIcon> icon):
     fName(name), fIcon(icon)
 {
     this->app = app;
@@ -96,7 +99,7 @@ void DObject::open() {
 DProgram::DProgram(
     IApp *app,
     YSMListener *smActionListener,
-    const ustring &name,
+    const mstring &name,
     ref<YIcon> icon,
     const bool restart,
     const char *wmclass,
@@ -112,7 +115,7 @@ DProgram::DProgram(
     this->app = app;
     this->smActionListener = smActionListener;
     if (fArgs.isEmpty() || fArgs.getString(fArgs.getCount() - 1))
-        fArgs.append(0);
+        fArgs.append(nullptr);
 }
 
 DProgram::~DProgram() {
@@ -138,26 +141,25 @@ DProgram *DProgram::newProgram(
     upath exe,
     YStringArray &args)
 {
+    DProgram* program = nullptr;
     if (exe != null) {
-        MSG(("LOOKING FOR: %s\n", exe.string().c_str()));
-        upath fullname = findPath(getenv("PATH"), X_OK, exe);
-        if (fullname == null) {
-            MSG(("Program %s (%s) not found.", name, exe.string().c_str()));
-            return 0;
+        const char* exestr = exe.string();
+        MSG(("LOOKING FOR: %s\n", exestr));
+        csmart path(path_lookup(exestr));
+        if (path) {
+            program = new DProgram(app, smActionListener, name, icon,
+                                   restart, wmclass, upath(path), args);
+        } else {
+            MSG(("Program %s (%s) not found.", name, exestr));
         }
-
-        DProgram *program =
-            new DProgram(app, smActionListener, name, icon, restart, wmclass, fullname, args);
-
-        return program;
     }
-    return NULL;
+    return program;
 }
 
 KProgramArrayType keyProgs;
 
 KProgram::KProgram(const char *key, DProgram *prog, bool bIsDynSwitchMenuProg)
-    : fKey(NoSymbol), fMod(0), bIsDynSwitchMenu(bIsDynSwitchMenuProg), fProg(prog), pSwitchWindow(0)
+    : fKey(NoSymbol), fMod(0), bIsDynSwitchMenu(bIsDynSwitchMenuProg), fProg(prog), pSwitchWindow(nullptr)
 {
     YConfig::parseKey(key, &fKey, &fMod);
     keyProgs.append(this);
@@ -173,7 +175,7 @@ class MenuProgSwitchItems: public ISwitchItems {
 public:
     MenuProgSwitchItems(DProgram* prog, KeySym key, unsigned keymod) : ISwitchItems()
         , zTarget(0), key(key), mod(keymod) {
-        menu = new MenuProgMenu(wmapp, wmapp, NULL /* no wmaction handling*/,
+        menu = new MenuProgMenu(wmapp, wmapp, nullptr /* no wmaction handling*/,
                 "switch popup internal menu", prog->fCmd, prog->fArgs);
     }
     virtual void updateList() override {
@@ -215,22 +217,22 @@ public:
         moveTarget(zdown);
     }
     virtual void reset() override {
-        zTarget=0;
+        zTarget = 0;
     }
     virtual void cancel() override {
     }
     virtual void accept(IClosablePopup *parent) override {
-        YMenuItem* item=menu->getItem(zTarget);
+        YMenuItem* item = menu->getItem(zTarget);
         if (!item) return;
         // even through all the obscure "abstraction" it should just run DObjectMenuItem::actionPerformed
-        item->actionPerformed(0, actionRun, 0);
+        item->actionPerformed(nullptr, actionRun, 0);
         parent->close();
     }
 
     virtual int getActiveItem() override {
         return zTarget;
     }
-    virtual ustring getTitle(int idx) override {
+    virtual mstring getTitle(int idx) override {
         if (idx<0 || idx>=this->getCount())
             return null;
         return menu->getItem(idx)->getName();
@@ -252,7 +254,7 @@ void KProgram::open(unsigned mods) {
 
     if (bIsDynSwitchMenu) {
         if (!pSwitchWindow) {
-            pSwitchWindow = new SwitchWindow(manager, new MenuProgSwitchItems(fProg, fKey, fMod), quickSwitchVertical);
+            pSwitchWindow = new SwitchWindow(desktop, new MenuProgSwitchItems(fProg, fKey, fMod), quickSwitchVertical);
         }
         pSwitchWindow->begin(true, mods);
     }
@@ -264,7 +266,7 @@ MenuFileMenu::MenuFileMenu(
     IApp *app,
     YSMListener *smActionListener,
     YActionListener *wmActionListener,
-    ustring name,
+    mstring name,
     YWindow *parent)
     :
     ObjectMenu(wmActionListener, parent),
@@ -321,7 +323,7 @@ MenuProgMenu::MenuProgMenu(
     IApp *app,
     YSMListener *smActionListener,
     YActionListener *wmActionListener,
-    ustring name,
+    mstring name,
     upath command,
     YStringArray &args,
     long timeout,
@@ -341,7 +343,7 @@ MenuProgMenu::~MenuProgMenu() {
 }
 
 void MenuProgMenu::updatePopup() {
-    time_t now = time(NULL);
+    time_t now = time(nullptr);
     if (fModTime == 0 || (0 < fTimeout && now >= fModTime + fTimeout)) {
         refresh();
         fModTime = now;
@@ -390,7 +392,7 @@ void StartMenu::updatePopup() {
 
 FocusMenu::FocusMenu() {
     struct FocusModelNameAction {
-        FocusModels mode;
+        FocusModel mode;
         const char *name;
         YAction action;
     } foci[] = {
@@ -424,13 +426,20 @@ public:
     {
         YMenu *fo, *qs, *tb, *sh, *al, *mz, *ke, *ks, *kw, *sc, *st;
         YMenuItem *item;
+        bool useElps = menuFont != null && menuFont->supports(utf32ellipsis);
 
         addSubmenu("_Focus", -2, fo = new YMenu, "focus");
         addSubmenu("_QuickSwitch", -2, qs = new YMenu, "pref");
         addSubmenu("_TaskBar", -2, tb = new YMenu, "pref");
         addSubmenu("_Show", -2, sh = new YMenu, "pref");
-        addSubmenu("_A... - L...", -2, al = new YMenu, "pref");
-        addSubmenu("_M... - Z...", -2, mz = new YMenu, "pref");
+        addSubmenu(
+                useElps ?
+                        "_A" utf8ellipsis " - L" utf8ellipsis :
+                        "_A... - L...", -2, al = new YMenu, "pref");
+        addSubmenu(
+                useElps ?
+                        "_M" utf8ellipsis " - Z" utf8ellipsis :
+                        "_M... - Z...", -2, mz = new YMenu, "pref");
         addSubmenu("_KeyWin", -2, ke = new YMenu, "key");
         addSubmenu("K_eySys", -2, ks = new YMenu, "key");
         addSubmenu("KeySys_Workspace", -2, kw = new YMenu, "key");
@@ -450,7 +459,7 @@ public:
                            0 == strncmp(o->name, "Quic", 4) ? qs :
                            0 == strncmp(o->name, "Show", 4) ||
                            0 == strncmp(o->name, "Page", 4) ? sh :
-                           0 != strstr(o->name, "Focus") ? fo :
+                           nullptr != strstr(o->name, "Focus") ? fo :
                            o->name[0] <= 'L' ? al : mz;
                 item = m->addItem(o->name, -2, null, EAction(index[k] + 1));
                 if (o->boolval()) {
@@ -472,14 +481,15 @@ public:
                 sc->addItem(o->name, -2, mstring(val), actionNull);
             }
             else if (o->type == cfoption::CF_STR) {
-                const char* str = o->str();
+                auto* str = o->str();
                 if (str) {
                     char val[40];
                     size_t len = strlcpy(val, str, sizeof val);
+                    auto elp = useElps ? utf8ellipsis : "...";
                     if (len >= sizeof val)
-                        strlcpy(val + sizeof val - 4, "...", 4);
-                    if ((str = strstr(val, "://")) != 0 && strlen(str) > 6)
-                        strlcpy(val + (str - val) + 3, "...", 4);
+                        strlcpy(val + sizeof val - 4, elp, 4);
+                    if (!!(str = strstr(val, "://")) && strlen(str) > 6)
+                        strlcpy(val + (str - val) + 3, elp, 4);
                     st->addItem(o->name, -2, val, actionNull);
                 }
             }
@@ -540,7 +550,7 @@ public:
     static upath preferencesPath() {
         const int perm = 0600;
 
-        ustring conf(wmapp->getConfigFile());
+        mstring conf(wmapp->getConfigFile());
         if (conf.nonempty() && conf != "preferences") {
             upath path(wmapp->findConfigFile(conf));
             if (path.isWritable())
@@ -568,9 +578,9 @@ public:
         if (path == null)
             return fail(_("Unable to write to %s"), "preferences");
 
-        csmart text(path.loadText());
-        if (text == 0)
-            (text = new char[1])[0] = 0;
+        auto text(path.loadText());
+        if (!text)
+            (text = fcsmart::create(1)).data()[0] = 0;
         size_t tlen = strlen(text);
         for (int k = 0; k < n; ++k) {
             const int i = mods[k];
@@ -598,7 +608,7 @@ public:
 
     static bool updateOption(cfoption* o, char* text) {
         char buf[99];
-        snprintf(buf, sizeof buf, "^[ \t]*%s[ \t]*=[ \t]*[!-~]", o->name);
+        snprintf(buf, sizeof buf, "^[ \t]*%s[ \t]*=", o->name);
         regex_t pat = {};
         int c = regcomp(&pat, buf, REG_EXTENDED | REG_NEWLINE);
         if (c) {
@@ -611,8 +621,10 @@ public:
         regmatch_t m = {};
         while (0 == regexec(&pat, s, 1, &m, 0)) {
             s += m.rm_eo;
-            s[-1] = '0' + o->boolval();
-            s = replaceComment(s, "# 0/1");
+            if (*s) {
+                *s++ = '0' + o->boolval();
+                s = replaceComment(s, "# 0/1");
+            }
         }
         regfree(&pat);
         return s > text;
@@ -633,7 +645,7 @@ public:
 
     static bool insertOption(cfoption* o, char* text) {
         char buf[99];
-        snprintf(buf, sizeof buf, "^#+[ \t]*%s[ \t]*=[ \t]*[!-~]", o->name);
+        snprintf(buf, sizeof buf, "^#+[ \t]*%s[ \t]*=", o->name);
         regex_t pat = {};
         int c = regcomp(&pat, buf, REG_EXTENDED | REG_NEWLINE);
         if (c) {
@@ -658,12 +670,12 @@ public:
 
     static char* retrieveComment(cfoption* o) {
         static const char path[] = LIBDIR "/preferences";
-        char* res = 0;
-        csmart text(load_text_file(path));
+        char* res = nullptr;
+        auto text(filereader(path).read_all());
         if (text) {
             char buf[99];
             snprintf(buf, sizeof buf,
-                     "(\n(#  .*\n)*)#+[ \t]*%s[ \t]*=[ \t]*[!-~]",
+                     "(\n(#  .*\n)*)#+[ \t]*%s[ \t]*=",
                      o->name);
             regex_t pat = {};
             int c = regcomp(&pat, buf, REG_EXTENDED | REG_NEWLINE);
@@ -684,21 +696,21 @@ public:
         return res ? res : newstr("\n");
     }
 
-    static void writePrefs(const upath dest, const char* text, size_t tlen) {
-        const upath temp(dest.path() + ".tmp");
+    static void writePrefs(upath dest, const char* text, size_t tlen) {
+        upath temp(dest.path() + ".tmp");
         int fd = temp.open(O_CREAT | O_WRONLY | O_TRUNC, 0600);
         if (fd == -1) {
-            fail(_("Unable to write to %s"), temp.string().c_str());
+            fail(_("Unable to write to %s"), temp.string());
         } else {
             ssize_t w = write(fd, text, tlen);
             if (size_t(w) != tlen)
-                fail(_("Unable to write to %s"), temp.string().c_str());
+                fail(_("Unable to write to %s"), temp.string());
             close(fd);
             if (size_t(w) == tlen) {
                 if (temp.renameAs(dest))
                     fail(_("Unable to rename %s to %s"),
-                         temp.string().c_str(),
-                         dest.string().c_str());
+                         temp.string(),
+                         dest.string());
                 else
                     mods.clear();
             }
@@ -733,7 +745,7 @@ HelpMenu::HelpMenu(
             upath path = upath(ICEHELPIDX).parent() + help[k].name;
             args.append(path.string());
         }
-        args.append(0);
+        args.append(nullptr);
 
         DProgram *prog = DProgram::newProgram(
             app,
@@ -753,11 +765,10 @@ HelpMenu::HelpMenu(
 void StartMenu::refresh() {
     MenuFileMenu::refresh();
 
-    if (itemCount())
-        addSeparator();
+    addSeparator();
 
 /// TODO #warning "make this into a menuprog (ala gnome.cc), and use mime"
-    if (openCommand && openCommand[0]) {
+    if (nonempty(openCommand)) {
         upath path[] = { upath::root(), YApplication::getHomeDir() };
         YMenu *sub;
         ref<YIcon> folder = YIcon::getIcon("folder");
@@ -772,75 +783,58 @@ void StartMenu::refresh() {
                 if (folder != null)
                     item->setIcon(folder);
             }
-            else if (sub) {
+            else {
                 delete sub;
             }
         }
         addSeparator();
     }
 
-    int const oldItemCount = itemCount();
-
     if (showPrograms) {
-        ObjectMenu *programs = new MenuFileMenu(app, smActionListener, wmActionListener, "programs", 0);
+        ObjectMenu *programs = new MenuFileMenu(app, smActionListener,
+                                    wmActionListener, "programs", nullptr);
         ///    if (programs->itemCount() > 0)
         addSubmenu(_("Programs"), 0, programs, "programs");
     }
 
-    if (showRun) {
-        if (runDlgCommand && runDlgCommand[0])
-            addItem(_("_Run..."), -2, null, actionRun, "run");
+    if (showRun && nonempty(runDlgCommand)) {
+        addItem(_("_Run..."), -2, null, actionRun, "run");
     }
 
-    if (itemCount() != oldItemCount) addSeparator();
-    if (showWindowList)
+    if (showWindowList) {
         addItem(_("_Windows"), -2, actionWindowList, windowListMenu, "windows");
+    }
 
-    if ( (!showTaskBar && showAbout) ||
-        showHelp ||
-        showFocusModeMenu ||
-        showThemesMenu ||
-        showSettingsMenu
-    )
-    if (showWindowList)
-        addSeparator();
+    YMenu* settings = new YMenu();
 
-    if (!showTaskBar) {
-        if (showAbout)
-            addItem(_("_About"), -2, actionAbout, 0, "about");
+    if (!showTaskBar && showAbout) {
+        settings->addItem(_("_About"), -2, actionAbout, nullptr, "about");
     }
 
     if (showHelp) {
-        HelpMenu *helpMenu =
-            new HelpMenu(app, smActionListener, wmActionListener);
-            addSubmenu(_("_Help"), -2, helpMenu, "help");
+        HelpMenu* help = new HelpMenu(app, smActionListener, wmActionListener);
+        settings->addSubmenu(_("_Help"), -2, help, "help");
     }
 
-    if (showSettingsMenu || showFocusModeMenu || showThemesMenu) {
-        // When we have only 2 entries (focus + themes) then
-        // it doesn't make sense to create a whole new YMenu.
-        // Therefore we will reuse 'this' as value for settings.
-        YMenu *settings = this; // new YMenu();
+    if (showFocusModeMenu) {
+        FocusMenu *focus = new FocusMenu();
+        settings->addSubmenu(_("_Focus"), -2, focus, "focus");
+    }
 
-        if (showFocusModeMenu) {
-            FocusMenu *focus = new FocusMenu();
-            settings->addSubmenu(_("_Focus"), -2, focus, "focus");
-        }
+    if (showSettingsMenu) {
+        PrefsMenu *prefs = new PrefsMenu();
+        settings->addSubmenu(_("_Preferences"), -2, prefs, "pref");
+    }
 
-        if (showSettingsMenu) {
-            PrefsMenu *prefs = new PrefsMenu();
-            settings->addSubmenu(_("_Preferences"), -2, prefs, "pref");
-        }
+    if (showThemesMenu) {
+        YMenu *themes = new ThemesMenu(app, smActionListener, wmActionListener);
+        settings->addSubmenu(_("_Themes"), -2, themes, "themes");
+    }
 
-        if (showThemesMenu) {
-            YMenu *themes = new ThemesMenu(app, smActionListener, wmActionListener);
-            if (themes)
-                settings->addSubmenu(_("_Themes"), -2, themes, "themes");
-        }
-
-        // Only add a menu if we created one:
-        if (this != settings)
-            addSubmenu(_("Se_ttings"), -2, settings, "settings");
+    if (settings->itemCount()) {
+        addSubmenu(_("Se_ttings"), -2, settings, "settings");
+    } else {
+        delete settings;
     }
 
     if (showLogoutMenu) {
