@@ -43,6 +43,8 @@ YWindowManagerStatus::YWindowManagerStatus()
 }
 
 YWindowManagerStatus::~YWindowManagerStatus() {
+    if (statusMoveSize == nullptr || statusWorkspace == nullptr)
+        statusFont = null;
 }
 
 void YWindowManagerStatus::configure(const YRect2& r) {
@@ -89,7 +91,7 @@ void YWindowManagerStatus::paint(Graphics &g, const YRect &/*r*/) {
     g.setColor(statusFg);
     g.setFont(statusFont);
 
-    ustring status(getStatus());
+    mstring status(getStatus());
     g.drawChars(status,
                 width() / 2 - statusFont->textWidth(status) / 2,
                 height() - statusFont->descent() - 2);
@@ -98,7 +100,7 @@ void YWindowManagerStatus::paint(Graphics &g, const YRect &/*r*/) {
 void YWindowManagerStatus::begin() {
     setPosition(x(),
                  taskBarAtTop ? 4 :
-                 (manager->height() - height()) - 4);
+                 (desktop->height() - height()) - 4);
     raise();
     show();
 }
@@ -119,11 +121,11 @@ void MoveSizeStatus::end() {
     statusMoveSize = null;
 }
 
-ustring MoveSizeStatus::longestStatus() {
+mstring MoveSizeStatus::longestStatus() {
    return "9999x9999+9999+9999";
 }
 
-ustring MoveSizeStatus::getStatus() {
+mstring MoveSizeStatus::getStatus() {
     char status[50];
     snprintf(status, 50, "%dx%d%+d%+d", fW, fH, fX, fY);
     return status;
@@ -145,8 +147,8 @@ void MoveSizeStatus::setStatus(YFrameWindow *frame, const YRect &r) {
     fX = r.x();
     fY = r.y();
     if (sh && (sh->flags & PResizeInc)) {
-        fW = (width - sh->base_width) / non_zero(sh->width_inc);
-        fH = (height - sh->base_height) / non_zero(sh->height_inc);
+        fW = (width - sh->base_width) / max(1, sh->width_inc);
+        fH = (height - sh->base_height) / max(1, sh->height_inc);
     } else {
         fW = width;
         fH = height;
@@ -160,8 +162,8 @@ void MoveSizeStatus::setStatus(YFrameWindow *frame) {
     fX = frame->x ();//// + frame->borderX ();
     fY = frame->y ();//// + frame->borderY () + frame->titleY ();
     if (sh && (sh->flags & PResizeInc)) {
-        fW = (frame->client()->width() - sh->base_width) / non_zero(sh->width_inc);
-        fH = (frame->client()->height() - sh->base_height) / non_zero(sh->height_inc);
+        fW = (frame->client()->width() - sh->base_width) / max(1, sh->width_inc);
+        fH = (frame->client()->height() - sh->base_height) / max(1, sh->height_inc);
     } else {
         fW = frame->client()->width();
         fH = frame->client()->height();
@@ -194,12 +196,12 @@ void WorkspaceStatus::end() {
     statusWorkspace = null;
 }
 
-ustring WorkspaceStatus::getStatus() {
+mstring WorkspaceStatus::getStatus() {
     return getStatus(workspaceNames[workspace]);
 }
 
-ustring WorkspaceStatus::getStatus(const char* name) {
-    return ustring(_("Workspace: ")).append(name);
+mstring WorkspaceStatus::getStatus(const char* name) {
+    return mstring(_("Workspace: ")).append(name);
 }
 
 void WorkspaceStatus::begin(long workspace) {
@@ -215,8 +217,8 @@ void WorkspaceStatus::setStatus(long workspace) {
     timer.startTimer();
 }
 
-ustring WorkspaceStatus::longestStatus() {
-    const char* longestWorkspaceName = NULL;
+mstring WorkspaceStatus::longestStatus() {
+    const char* longestWorkspaceName = nullptr;
     int maxWorkspaceNameLength = 0;
 
     for (int w = 0; w < workspaceCount; ++w) {
