@@ -9,6 +9,7 @@
 #include "wmswitch.h"
 #include "wpixmaps.h"
 #include "wmframe.h"
+#include "wmmgr.h"
 #include "yxapp.h"
 #include "prefs.h"
 #include "yprefs.h"
@@ -43,10 +44,10 @@ class WindowItemsCtrlr : public ISwitchItems
         } else
             GetZListWorkspace(false, -1);
 
-        if (fActiveWindow != 0 && find(zList, fActiveWindow) == -1)
-            fActiveWindow = 0;
-        if (fLastWindow != 0 && find(zList, fLastWindow) == -1)
-            fLastWindow = 0;
+        if (fActiveWindow != nullptr && find(zList, fActiveWindow) == -1)
+            fActiveWindow = nullptr;
+        if (fLastWindow != nullptr && find(zList, fLastWindow) == -1)
+            fLastWindow = nullptr;
     }
 
     void freeList() {
@@ -74,7 +75,7 @@ class WindowItemsCtrlr : public ISwitchItems
                 }
 
                 if (!w->isUrgent()) {
-                    if (workspaceOnly && w->isSticky() &&
+                    if (workspaceOnly && w->isAllWorkspaces() &&
                         workspace != fRoot->activeWorkspace()) {
                         continue;
                     }
@@ -97,7 +98,7 @@ class WindowItemsCtrlr : public ISwitchItems
                     } else {
                         if (pass == 2) append(w);
                     }
-                } else if (w->frameOptions() & YFrameWindow::foIgnoreQSwitch) {
+                } else if (w->frameOption(YFrameWindow::foIgnoreQSwitch)) {
                 } else if (w->avoidFocus()) {
                     if (pass == 5) append(w);
                 } else if (w->isHidden()) {
@@ -141,18 +142,18 @@ public:
     }
     inline virtual int setTarget(int zPosition) override
     {
-        zTarget=zPosition;
+        zTarget = zPosition;
         if (inrange(zTarget, 0, getCount() - 1))
             fActiveWindow = zList[zTarget];
         else
-            fActiveWindow = 0;
+            fActiveWindow = nullptr;
         return zPosition;
     }
 
 
     WindowItemsCtrlr() :
-        zTarget(0), fRoot(manager), fActiveWindow(0), fLastWindow(0),
-        fWMClass(0)
+        zTarget(0), fRoot(manager), fActiveWindow(nullptr), fLastWindow(nullptr),
+        fWMClass(nullptr)
     {
     }
 
@@ -167,7 +168,7 @@ public:
         return zTarget;
     }
 
-    virtual ustring getTitle(int idx) override
+    virtual mstring getTitle(int idx) override
     {
         if (inrange(idx, 0, getCount() - 1))
             return zList[idx]->client()->windowTitle();
@@ -209,18 +210,31 @@ public:
             fActiveWindow->activateWindow(true, false);
         }
         freeList();
-        fLastWindow = fActiveWindow = 0;
+        fLastWindow = fActiveWindow = nullptr;
     }
 
     virtual void accept(IClosablePopup *parent) override {
-        if (fActiveWindow == 0)
+        if (fActiveWindow == nullptr)
             cancel();
         else {
             fActiveWindow->activateWindow(true, false);
             parent->close();
         }
         freeList();
-        fLastWindow = fActiveWindow = 0;
+        fLastWindow = fActiveWindow = nullptr;
+    }
+
+    virtual void destroyTarget() override {
+        auto id = getActiveItem();
+        if (id < 0) return;
+        auto item = zList[id];
+        if (!item) return;
+        auto client = item->client();
+        if (!client) return;
+        auto frame = client->getFrame();
+        if (!frame) return;
+        frame->activateWindow(true, false);
+        manager->getFocus()->wmClose();
     }
 
     virtual void destroyedItem(void *item) override
@@ -230,18 +244,16 @@ public:
 
         YFrameWindow* frame = (YFrameWindow*) item;
         if (frame == fLastWindow)
-            fLastWindow = 0;
+            fLastWindow = nullptr;
+        auto pos = zTarget;
         updateList();
-        if (frame == fActiveWindow || fActiveWindow == 0) {
-            zTarget = -1;
-            moveTarget(true);
-        }
+        setTarget( (pos >= 0 && pos < getCount()) ? pos : 0);
         displayFocusChange(fActiveWindow);
     }
 
     virtual bool isKey(KeySym k, unsigned int vm) override {
         return gKeySysSwitchNext.eq(k, vm) ||
-              (gKeySysSwitchClass.eq(k, vm) && fWMClass != 0);
+              (gKeySysSwitchClass.eq(k, vm) && fWMClass != nullptr);
     }
 };
 
@@ -309,23 +321,23 @@ void SwitchWindow::resize(int xiscreen) {
     int dx, dy;
     unsigned dw, dh;
 
-    manager->getScreenGeometry(&dx, &dy, &dw, &dh, xiscreen);
+    desktop->getScreenGeometry(&dx, &dy, &dw, &dh, xiscreen);
 
     MSG(("got geometry for %d: %d %d %d %d", xiscreen, dx, dy, dw, dh));
 
-    ustring cTitle = zItems->getTitle(zItems->getActiveItem());
+    mstring cTitle = zItems->getTitle(zItems->getActiveItem());
 
     int aWidth =
         quickSwitchSmallWindow ?
         (int) dw * 1/3
         : (quickSwitchVertical ? (int) dw * 2/5 : (int) dw * 3/5);
 
-    int tWidth=0;
+    int tWidth = 0;
     if (quickSwitchMaxWidth) {
         int space = (int) switchFont->textWidth(" ");   /* make entries one space character wider */
         int zCount = zItems->getCount();
         for (int i = 0; i < zCount; i++) {
-            ustring title = zItems->getTitle(i);
+            mstring title = zItems->getTitle(i);
             int oWidth = title != null ? (int) switchFont->textWidth(title) + space : 0;
             if (oWidth > tWidth)
                 tWidth = oWidth;
@@ -414,9 +426,7 @@ void SwitchWindow::paint(Graphics &g, const YRect &/*r*/) {
     else
         g.fillRect(1, 1, width() - 3, height() - 3);
 
-    return m_verticalStyle
-        ? paintVertical(g)
-        : paintHorizontal(g);
+    m_verticalStyle ? paintVertical(g) : paintHorizontal(g);
 }
 
 void SwitchWindow::paintHorizontal(Graphics &g) {
@@ -461,7 +471,7 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
         g.setColor(switchFg);
         g.setFont(switchFont);
 
-        ustring cTitle = zItems->getTitle(zItems->getActiveItem());
+        mstring cTitle = zItems->getTitle(zItems->getActiveItem());
         if (cTitle != null) {
             const int x = max((width() - tOfs -
                                switchFont->textWidth(cTitle)) >> 1, 0U) + tOfs;
@@ -535,8 +545,9 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
                                         iconSize + 2 * quickSwitchIBorder,
                                         iconSize + 2 * quickSwitchIBorder);
 
-                            if (icon != null)
+                            if (icon != null) {
                                 icon->draw(g, x, y - ds / 2, iconSize);
+                            }
                         } else {
                             icon->draw(g, x, y, YIcon::largeSize());
                         }
@@ -551,7 +562,7 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
 
 int SwitchWindow::calcHintedItem(int x, int y)
 {
-    if(quickSwitchVertical)
+    if (quickSwitchVertical)
         return (y - m_hintAreaStart) / m_hintAreaStep;
     else if(quickSwitchAllIcons && !quickSwitchHugeIcon)
         return (x - m_hintAreaStart) / m_hintAreaStep;
@@ -562,7 +573,7 @@ int SwitchWindow::calcHintedItem(int x, int y)
 void SwitchWindow::handleMotion(const XMotionEvent& motion) {
     int hintId = calcHintedItem(motion.x, motion.y);
     //printf("hint id: %d\n", hintId);
-    if(hintId == m_hlItemFromMotion || hintId == -2)
+    if (hintId == m_hlItemFromMotion || hintId == -2)
         return;
     m_hlItemFromMotion = hintId;
     repaint();
@@ -590,8 +601,8 @@ void SwitchWindow::paintVertical(Graphics &g) {
 
         g.setFont(switchFont);
         g.setColor(switchFg);
-        for (int i = 0, zCount=zItems->getCount(); i < zCount; i++) {
-            if(contentY + frameHght > (int) height())
+        for (int i = 0, zCount = zItems->getCount(); i < zCount; i++) {
+            if (contentY + frameHght > (int) height())
                 break;
             if (i == zItems->getActiveItem()) {
                 g.setColor(switchMbg);
@@ -601,11 +612,11 @@ void SwitchWindow::paintVertical(Graphics &g) {
             else
                 g.setColor(switchFg);
 
-            ustring cTitle = zItems->getTitle(i);
+            mstring cTitle = zItems->getTitle(i);
 
             if (cTitle != null) {
                 const int titleY = contentY + (iconSize + g.font()->ascent())/2;
-                g.drawStringEllipsis(titleX, titleY, cstring(cTitle).c_str(), strWid);
+                g.drawStringEllipsis(titleX, titleY, cTitle.c_str(), strWid);
             }
             ref<YIcon> icon = zItems->getIcon(i);
             if (icon != null) {
@@ -615,7 +626,7 @@ void SwitchWindow::paintVertical(Graphics &g) {
                 icon->draw(g, iconX, contentY, iconSize);
             }
 
-            if(i == m_hlItemFromMotion && i != zItems->getActiveItem())
+            if (i == m_hlItemFromMotion && i != zItems->getActiveItem())
             {
                 g.setColor(switchMbg);
                 g.drawRect(frameX, contentY-quickSwitchIBorder, frameWid, frameHght);
@@ -651,7 +662,7 @@ void SwitchWindow::begin(bool zdown, int mods, char* wmclass) {
     int item = zItems->getActiveItem();
     if (item >= 0) {
         displayFocus(item);
-        isUp = popup(0, 0, 0, xiscreen, YPopupWindow::pfNoPointerChange);
+        isUp = popup(nullptr, nullptr, nullptr, xiscreen, YPopupWindow::pfNoPointerChange);
     }
 
     if (zItems->getCount() < 1) {
@@ -713,6 +724,12 @@ bool SwitchWindow::handleKey(const XKeyEvent &key) {
             cancel();
             return true;
         }
+        else if ((IS_WMKEY(k, vm, gKeyWinClose)))
+        {
+            zItems->destroyTarget();
+            return true;
+        }
+
         if (zItems->isKey(k, vm) && !modDown(m)) {
             accept();
             return true;
@@ -753,12 +770,17 @@ bool SwitchWindow::modDown(int mod) {
 
 void SwitchWindow::handleButton(const XButtonEvent &button) {
     //printf("got click, hot item: %d\n", m_hintedItem);
-    int hintId = calcHintedItem(button.x, button.y);
-    if (button.button == Button1 && button.type == ButtonPress) {
+    if (button.type == ButtonPress) {
+        int hintId = calcHintedItem(button.x, button.y);
         if (hintId >= 0 && hintId < zItems->getCount()) {
-            zItems->reset();
-            zItems->setTarget(hintId);
-            accept();
+            if (button.button == Button1) {
+                zItems->setTarget(hintId);
+                accept();
+            }
+            if (button.button == Button2) {
+                zItems->setTarget(hintId);
+                zItems->destroyTarget();
+            }
         }
     }
     YPopupWindow::handleButton(button);
