@@ -27,6 +27,8 @@ public:
 
     // move the focused target up or down and return the new focused element
     virtual int moveTarget(bool zdown)=0;
+    // run closing/destruction process for the pointed object
+    virtual void destroyTarget() {}
     // set the target explicitly rather than switching around
     virtual int setTarget(int zPosition)=0;
     /// Show changed focus preview to user
@@ -41,7 +43,7 @@ public:
 
     // XXX: convert to iterator
     virtual int getActiveItem()=0;
-    virtual ustring getTitle(int idx) =0;
+    virtual mstring getTitle(int idx) =0;
     virtual ref<YIcon> getIcon(int idx) =0;
 
     // Manager notification about windows disappearing under the fingers
@@ -55,22 +57,22 @@ public:
 
 class SwitchWindow: public YPopupWindow, IClosablePopup {
 public:
-    SwitchWindow(YWindow *parent = 0,
-                 ISwitchItems *items = 0, bool verticalStyle=true);
+    SwitchWindow(YWindow *parent = nullptr,
+                 ISwitchItems *items = nullptr, bool verticalStyle = true);
     virtual ~SwitchWindow();
 
-    virtual void paint(Graphics &g, const YRect &r) OVERRIDE;
-    virtual void repaint() OVERRIDE;
+    virtual void paint(Graphics &g, const YRect &r) override;
+    virtual void repaint() override;
 
-    void begin(bool zdown, int mods, char* wmclass = 0);
+    void begin(bool zdown, int mods, char* wmclass = nullptr);
 
-    virtual void activatePopup(int flags) OVERRIDE;
-    virtual void deactivatePopup() OVERRIDE;
+    virtual void activatePopup(int flags) override;
+    virtual void deactivatePopup() override;
 
-    virtual void handleExpose(const XExposeEvent &expose) OVERRIDE {}
-    virtual bool handleKey(const XKeyEvent &key) OVERRIDE;
-    virtual void handleButton(const XButtonEvent &button) OVERRIDE;
-    void handleMotion(const XMotionEvent &motion) OVERRIDE;
+    virtual void handleExpose(const XExposeEvent &expose) override {}
+    virtual bool handleKey(const XKeyEvent &key) override;
+    virtual void handleButton(const XButtonEvent &button) override;
+    void handleMotion(const XMotionEvent &motion) override;
     void destroyedFrame(YFrameWindow *frame);
 
 private:
@@ -93,7 +95,6 @@ private:
     ref<YFont> switchFont;
 
     int modsDown;
-
     bool isUp;
 
     bool modDown(int m);
@@ -101,7 +102,7 @@ private:
     void resize(int xiscreen);
 
     void cancel();
-    virtual bool close() OVERRIDE;
+    virtual bool close() override;
     void accept();
     void displayFocus(int itemIdx);
     //YFrameWindow *nextWindow(YFrameWindow *from, bool zdown, bool next);
