@@ -7,30 +7,25 @@
  */
 
 #include "config.h"
-
-#include "yfull.h"
-#include "ypaint.h"
 #include "wmtaskbar.h"
-#include "yprefs.h"
-
-#include "wmmgr.h"
 #include "wmframe.h"
-#include "wmclient.h"
-#include "wmaction.h"
+#include "wmmgr.h"
+#include "wmconfig.h"
 #include "wmprog.h"
-#include "workspaces.h"
 #include "wmwinmenu.h"
 #include "wmapp.h"
-#include "sysdep.h"
 #include "wmwinlist.h"
+#include <unistd.h>
 
 #include "aaddressbar.h"
 #include "aclock.h"
+#include "akeyboard.h"
 #include "acpustatus.h"
 #include "amemstatus.h"
 #include "apppstatus.h"
 #include "amailbox.h"
 #include "objbar.h"
+#include "objbutton.h"
 #include "atasks.h"
 #include "atray.h"
 #include "aworkspaces.h"
@@ -41,41 +36,47 @@
 
 #include "intl.h"
 
-TaskBar *taskBar = 0;
+TaskBar *taskBar;
 
 YColorName taskBarBg(&clrDefaultTaskBar);
 
 EdgeTrigger::EdgeTrigger(TaskBar *owner):
     fTaskBar(owner),
-    fDoShow(false)
+    fHideOrShow(Hide)
 {
     setStyle(wsOverrideRedirect | wsInputOnly);
     setPointer(YXApplication::leftPointer);
-    setDND(true);
+    setDND(enabled());
     setTitle("IceEdge");
-    fAutoHideTimer->setTimer(autoShowDelay, this, false);
 }
 
 EdgeTrigger::~EdgeTrigger() {
 }
 
-void EdgeTrigger::startHide() {
-    fDoShow = false;
-    fAutoHideTimer->startTimer(autoHideDelay);
+void EdgeTrigger::startTimer(HideOrShow show) {
+    fHideOrShow = show;
+    long delay = max(10, show ? autoShowDelay : autoHideDelay);
+    fAutoHideTimer->setTimer(delay, this, true);
 }
 
-void EdgeTrigger::stopHide() {
-    fDoShow = false;
-    fAutoHideTimer->stopTimer();
+void EdgeTrigger::stopTimer() {
+    fHideOrShow = Hide;
+    fAutoHideTimer = null;
 }
 
-bool EdgeTrigger::enabled() const {
-    return (taskBarAutoHide | taskBarFullscreenAutoShow) & !taskBarKeepBelow;
+bool EdgeTrigger::enabled() {
+    return (taskBarAutoHide | (taskBarFullscreenAutoShow & !taskBarKeepBelow));
 }
 
-void EdgeTrigger::show() {
-    if (enabled()) {
+void EdgeTrigger::show(bool enable) {
+    bool enabled(this->enabled());
+    if (enable && enabled) {
         YWindow::show();
+    } else {
+        YWindow::hide();
+        if (enabled) {
+            startTimer();
+        }
     }
 }
 
@@ -84,93 +85,78 @@ void EdgeTrigger::handleCrossing(const XCrossingEvent &crossing) {
         unsigned long last = YWindow::getLastEnterNotifySerial();
         if (crossing.serial != last && crossing.serial != last + 1) {
             MSG(("enter notify %d %d", crossing.mode, crossing.detail));
-            fDoShow = true;
-            fAutoHideTimer->startTimer(autoShowDelay);
+            startTimer(Show);
         }
     } else if (crossing.type == LeaveNotify /* && crossing.mode != NotifyNormal */) {
-        fDoShow = false;
         MSG(("leave notify"));
-        fAutoHideTimer->stopTimer();
+        stopTimer();
     }
 }
 
 void EdgeTrigger::handleDNDEnter() {
-    fDoShow = true;
-    fAutoHideTimer->startTimer(autoShowDelay);
+    startTimer(Show);
 }
 
 void EdgeTrigger::handleDNDLeave() {
-    fDoShow = false;
-    fAutoHideTimer->startTimer(autoHideDelay);
+    startTimer();
 }
 
-
 bool EdgeTrigger::handleTimer(YTimer *t) {
     MSG(("taskbar handle timer"));
-    return fTaskBar->autoTimer(fDoShow);
+    return fTaskBar->autoTimer(fHideOrShow);
 }
 
 TaskBar::TaskBar(IApp *app, YWindow *aParent, YActionListener *wmActionListener, YSMListener *smActionListener):
-    YFrameClient(aParent, 0),
-    fGraphics(this),
+    YFrameClient(aParent, nullptr),
     fSurface(taskBarBg, taskbackPixmap, taskbackPixbuf),
-    fTasks(0),
-    fCollapseButton(0),
-    fWindowTray(0),
-    fMailBoxStatus(0),
-    fMEMStatus(0),
-    fCPUStatus(0),
-    fApm(0),
-    fNetStatus(0),
-    fObjectBar(0),
-    fApplications(0),
-    fWinList(0),
-    fShowDesktop(0),
-    fAddressBar(0),
-    fWorkspaces(0),
-    fDesktopTray(0),
+    fTasks(nullptr),
+    fCollapseButton(nullptr),
+    fWindowTray(nullptr),
+    fKeyboardStatus(nullptr),
+    fMailBoxControl(nullptr),
+    fMEMStatus(nullptr),
+    fCPUStatus(nullptr),
+    fApm(nullptr),
+    fNetStatus(nullptr),
+    fObjectBar(nullptr),
+    fApplications(nullptr),
+    fWinList(nullptr),
+    fShowDesktop(nullptr),
+    fAddressBar(nullptr),
+    fWorkspaces(nullptr),
+    fDesktopTray(nullptr),
+    fEdgeTrigger(nullptr),
     wmActionListener(wmActionListener),
     smActionListener(smActionListener),
     app(app),
     fIsHidden(taskBarAutoHide),
     fFullscreen(false),
     fIsCollapsed(false),
-    fIsMapped(false),
     fMenuShown(false),
     fNeedRelayout(false),
     fButtonUpdate(false),
-    fEdgeTrigger(0)
+    fWorkspacesUpdate(false)
 {
     taskBar = this;
 
-    ///setToplevel(true);
-
     addStyle(wsNoExpose);
-
     setWinHintsHint(WinHintsSkipFocus |
                     WinHintsSkipWindowMenu |
                     WinHintsSkipTaskBar);
 
-    setWinWorkspaceHint(-1);
-    setWinLayerHint((taskBarAutoHide || fFullscreen) ? WinLayerAboveAll :
-                    fIsCollapsed ? WinLayerAboveDock :
-                    taskBarKeepBelow ? WinLayerBelow : WinLayerDock);
+    setWorkspaceHint(AllWorkspaces);
+    updateWinLayer();
     Atom protocols[2] = {
       _XA_WM_DELETE_WINDOW,
       _XA_WM_TAKE_FOCUS
-      //_NET_WM_PING,
-      //_NET_WM_SYNC_REQUEST,
     };
     XSetWMProtocols(xapp->display(), handle(), protocols, 2);
-
-    {
-        XWMHints wmhints = { InputHint, False, };
-        ClassHint clhint("icewm", "TaskBar");
-        YTextProperty text("TaskBar");
-        XSetWMProperties(xapp->display(), handle(), &text, &text,
-                         nullptr, 0, nullptr, &wmhints, &clhint);
-        setProperty(_XA_NET_WM_PID, XA_CARDINAL, getpid());
-    }
+    XWMHints wmhints = { InputHint, False, };
+    ClassHint clhint("icewm", "TaskBar");
+    YTextProperty text("TaskBar");
+    XSetWMProperties(xapp->display(), handle(), &text, &text,
+                     nullptr, 0, nullptr, &wmhints, &clhint);
+    setNetPid();
 
     setMwmHints(MwmHints(
        MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS,
@@ -183,38 +169,34 @@ TaskBar::TaskBar(IApp *app, YWindow *aParent, YActionListener *wmActionListener,
 
     initApplets();
 
-    getPropertiesList();
-    getWMHints();
-    getClassHint();
-    fIsMapped = true;
-
     MSG(("taskbar"));
 }
 
 TaskBar::~TaskBar() {
     detachDesktopTray();
-    delete fEdgeTrigger; fEdgeTrigger = 0;
-    delete fClock; fClock = 0;
-    delete fMailBoxStatus; fMailBoxStatus = 0;
+    delete fEdgeTrigger; fEdgeTrigger = nullptr;
+    delete fClock; fClock = nullptr;
+    delete fKeyboardStatus; fKeyboardStatus = nullptr;
+    delete fMailBoxControl; fMailBoxControl = nullptr;
 #ifdef MEM_STATES
-    delete fMEMStatus; fMEMStatus = 0;
+    delete fMEMStatus; fMEMStatus = nullptr;
 #endif
-    delete fWinList; fWinList = 0;
-    delete fApplications; fApplications = 0;
-    delete fObjectBar; fObjectBar = 0;
-    delete fWorkspaces; fWorkspaces = 0;
+    delete fWinList; fWinList = nullptr;
+    delete fApplications; fApplications = nullptr;
+    delete fObjectBar; fObjectBar = nullptr;
+    delete fWorkspaces; fWorkspaces = nullptr;
 #ifdef MAX_ACPI_BATTERY_NUM
-    delete fApm; fApm = 0;
+    delete fApm; fApm = nullptr;
 #endif
 #ifdef IWM_STATES
-    delete fCPUStatus; fCPUStatus = 0;
+    delete fCPUStatus; fCPUStatus = nullptr;
 #endif
-    delete fNetStatus; fNetStatus = 0;
-    delete fAddressBar; fAddressBar = 0;
-    delete fTasks; fTasks = 0;
-    delete fWindowTray; fWindowTray = 0;
-    delete fCollapseButton; fCollapseButton = 0;
-    delete fShowDesktop; fShowDesktop = 0;
+    delete fNetStatus; fNetStatus = nullptr;
+    delete fAddressBar; fAddressBar = nullptr;
+    delete fTasks; fTasks = nullptr;
+    delete fWindowTray; fWindowTray = nullptr;
+    delete fCollapseButton; fCollapseButton = nullptr;
+    delete fShowDesktop; fShowDesktop = nullptr;
     xapp->dropClipboard();
     taskBar = nullptr;
     if (getFrame())
@@ -245,7 +227,7 @@ public:
         addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
         if (minimizeToDesktop)
         // TRANSLATORS: This appears in a group with others items, so please make the hotkeys unique in the set: # T_ile Horizontally, Ca_scade, _Arrange, _Minimize All, _Hide All, _Undo, Arrange _Icons, _Windows, _Refresh, _About, _Logout
-            addItem(_("Arrange _Icons"), -2, KEY_NAME(gKeySysArrangeIcons), actionArrangeIcons)->setEnabled(false);
+            addItem(_("Arrange _Icons"), -2, KEY_NAME(gKeySysArrangeIcons), actionArrangeIcons);
         addSeparator();
         // TRANSLATORS: This appears in a group with others items, so please make the hotkeys unique in the set: # T_ile Horizontally, Ca_scade, _Arrange, _Minimize All, _Hide All, _Undo, Arrange _Icons, _Windows, _Refresh, _About, _Logout
         addItem(_("_Windows"), -2, actionWindowList, windowListMenu);
@@ -253,7 +235,7 @@ public:
         // TRANSLATORS: This appears in a group with others items, so please make the hotkeys unique in the set: # T_ile Horizontally, Ca_scade, _Arrange, _Minimize All, _Hide All, _Undo, Arrange _Icons, _Windows, _Refresh, _About, _Logout
         addItem(_("_Refresh"), -2, null, actionRefresh);
         // TRANSLATORS: This appears in a group with others items, so please make the hotkeys unique in the set: # T_ile Horizontally, Ca_scade, _Arrange, _Minimize All, _Hide All, _Undo, Arrange _Icons, _Windows, _Refresh, _About, _Logout
-        addItem(_("_About"), -2, actionAbout, 0);
+        addItem(_("_About"), -2, actionAbout, nullptr);
         if (showLogoutMenu) {
             addSeparator();
             if (showLogoutSubMenu)
@@ -274,14 +256,14 @@ void TaskBar::initApplets() {
     if (taskBarShowMEMStatus)
         fMEMStatus = new MEMStatus(this, this);
     else
-        fMEMStatus = 0;
+        fMEMStatus = nullptr;
 #endif
 
 #ifdef IWM_STATES
     if (taskBarShowCPUStatus)
         fCPUStatus = new CPUStatusControl(smActionListener, this, this);
     else
-        fCPUStatus = 0;
+        fCPUStatus = nullptr;
 #endif
 
     if (taskBarShowNetStatus)
@@ -292,7 +274,7 @@ void TaskBar::initApplets() {
     if (taskBarShowClock)
         fClock = new YClock(smActionListener, this, this);
     else
-        fClock = 0;
+        fClock = nullptr;
 
 #ifdef MAX_ACPI_BATTERY_NUM
     if (taskBarShowApm && (access(APMDEV, 0) == 0 ||
@@ -309,12 +291,12 @@ void TaskBar::initApplets() {
         fApm = new YApm(this, true);
         if ( ! fApm->hasBatteries()) {
                 delete fApm;
-                fApm = 0;
+                fApm = nullptr;
         }
         else fApm->setTitle("IceAPM");
     }
     else
-        fApm = 0;
+        fApm = nullptr;
 #endif
 
     if (taskBarShowCollapseButton) {
@@ -328,12 +310,17 @@ void TaskBar::initApplets() {
             fCollapseButton->setTitle("Collapse");
         }
     } else
-        fCollapseButton = 0;
+        fCollapseButton = nullptr;
 
     if (taskBarShowMailboxStatus) {
-        fMailBoxStatus = new MailBoxControl(app, smActionListener, this, this);
+        fMailBoxControl = new MailBoxControl(app, smActionListener, this, this);
     } else
-        fMailBoxStatus = 0;
+        fMailBoxControl = nullptr;
+
+    if (configKeyboards.nonempty()) {
+        fKeyboardStatus = new KeyboardStatus(this, this);
+    } else
+        fKeyboardStatus = nullptr;
 
     if (taskBarShowStartMenu) {
         class LazyRootMenu : public LazyMenu {
@@ -345,7 +332,7 @@ void TaskBar::initApplets() {
         fApplications->setToolTip(_("Favorite Applications"));
         fApplications->setTitle("TaskBarMenu");
     } else
-        fApplications = 0;
+        fApplications = nullptr;
 
     fObjectBar = new ObjectBar(this);
     if (fObjectBar) {
@@ -354,7 +341,11 @@ void TaskBar::initApplets() {
             MenuLoader(app, smActionListener, wmActionListener)
             .loadMenus(t, fObjectBar);
         }
-        fObjectBar->setTitle("IceToolbar");
+        if (fObjectBar->nonempty()) {
+            fObjectBar->setTitle("IceToolbar");
+        } else {
+            delete fObjectBar; fObjectBar = nullptr;
+        }
     }
     if (taskBarShowWindowListMenu) {
         class LazyWindowListMenu : public LazyMenu {
@@ -366,7 +357,7 @@ void TaskBar::initApplets() {
         fWinList->setToolTip(_("Window List Menu"));
         fWinList->setTitle("ShowWindowList");
     } else
-        fWinList = 0;
+        fWinList = nullptr;
     if (taskBarShowShowDesktopButton) {
         fShowDesktop = new ObjectButton(this, actionShowDesktop);
         fShowDesktop->setText("__");
@@ -389,12 +380,12 @@ void TaskBar::initApplets() {
         fTasks = new TaskPane(this, this);
         fTasks->setTitle("TaskPane");
     } else
-        fTasks = 0;
+        fTasks = nullptr;
     if (taskBarShowTray) {
         fWindowTray = new TrayPane(this, this);
         fWindowTray->setTitle("TrayPane");
     } else
-        fWindowTray = 0;
+        fWindowTray = nullptr;
 
     if (taskBarEnableSystemTray) {
         const char atomstr[] =
@@ -412,7 +403,7 @@ void TaskBar::initApplets() {
         fDesktopTray->setTitle("SystemTray");
         fDesktopTray->relayout();
     } else
-        fDesktopTray = 0;
+        fDesktopTray = nullptr;
 
     if (fCollapseButton) {
         fCollapseButton->raise();
@@ -427,86 +418,97 @@ void TaskBar::trayChanged() {
 struct LayoutInfo {
     YWindow *w;
     bool left;
-    int row; // 0 = bottom, 1 = top
+    bool row; // 0 = bottom, 1 = top
     bool show;
-    int pre, post;
     bool expand;
+    short pre, post;
 
     LayoutInfo() :
-        w(0), left(false), row(0), show(false), pre(0), post(0), expand(false) {}
-    LayoutInfo(YWindow *_w, bool l, int r, bool s, int p, int o, bool e) :
-        w(_w), left(l), row(r), show(s), pre(p), post(o), expand(e) {}
+        w(nullptr), left(false), row(false), show(false),
+        expand(false), pre(0), post(0) {}
+    LayoutInfo(YWindow *w, bool l, bool r, bool s, bool e, short p, short o) :
+        w(w), left(l), row(r), show(s), expand(e), pre(p), post(o) {}
 };
 
-bool operator==(const LayoutInfo &l1, const LayoutInfo &l2)
-{
-    return memcmp(&l1, &l2, sizeof(LayoutInfo)) == 0;
-}
-
 void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
+    enum { Over, Here };
+    enum { Bot, Top };
+    enum { Same, Show };
+    enum { Keep, Grow };
     LayoutInfo nw;
-    YArray<LayoutInfo> wlist;
-    wlist.setCapacity(13);
+    YArray<LayoutInfo> wlist(16);
 
     bool issue314 = taskBarAtTop;
-    nw = LayoutInfo( fApplications, true, issue314, true, 0, 0, true );
+    nw = LayoutInfo( fApplications, Here, issue314, Show, Grow, 0, 0 );
     wlist.append(nw);
-    nw = LayoutInfo( fShowDesktop, true, !issue314, true, 0, 0, true );
+    if (taskBarShowShowDesktopButton == 1) {
+        nw = LayoutInfo( fShowDesktop, Here, !issue314, Show, Grow, 0, 0 );
+        wlist.append(nw);
+    }
+    nw = LayoutInfo( fWinList, Here, !issue314, Show, Grow, 0, 0 );
     wlist.append(nw);
-    nw = LayoutInfo( fWinList, true, !issue314, true, 0, 0, true );
+    nw = LayoutInfo( fObjectBar, Here, Top, Show, Grow, 4, 0 );
     wlist.append(nw);
-    nw = LayoutInfo( fObjectBar, true, 1, true, 4, 0, true );
-    wlist.append(nw);
-    nw = LayoutInfo( fCollapseButton, false, 0, true, 0, 2, true );
+    nw = LayoutInfo( fCollapseButton, Over, Bot, Show, Grow, 0, 2 );
     wlist.append(nw);
     nw = LayoutInfo( fWorkspaces, taskBarWorkspacesLeft,
                      taskBarDoubleHeight && taskBarWorkspacesTop,
                      taskBarShowWorkspaces && workspaceCount > 0,
-                     4, 4, true );
+                     Grow, 4, 4 );
     wlist.append(nw);
 
-    nw = LayoutInfo( fClock, false, 1, false, 2, 2, false );
+    if (taskBarShowShowDesktopButton == 2) {
+        nw = LayoutInfo( fShowDesktop, Over, Top, Show, Keep, 1, 1);
+        wlist.append(nw);
+    }
+
+    nw = LayoutInfo( fClock, Over, Top, Same, Keep, 2, 2 );
     wlist.append(nw);
+
     if (taskBarShowMailboxStatus) {
-        for (MailBoxControl::IterType m = fMailBoxStatus->iterator(); ++m; ) {
-            nw = LayoutInfo( *m, false, 1, true, 1, 1, false );
+        for (auto m = fMailBoxControl->iterator(); ++m; ) {
+            nw = LayoutInfo( *m, Over, Top, Show, Keep, 1, 1 );
             wlist.append(nw);
         }
     }
+    if (fKeyboardStatus) {
+        nw = LayoutInfo( fKeyboardStatus, Over, Top, Show, Keep, 1, 1 );
+        wlist.append(nw);
+    }
 
 #ifdef IWM_STATES
     if (taskBarShowCPUStatus) {
-        CPUStatusControl::IterType it = fCPUStatus->getIterator();
+        auto it = fCPUStatus->getIterator();
         while (++it)
         {
-            nw = LayoutInfo(*it, false, 1, true, 2, 2, false );
+            nw = LayoutInfo(*it, Over, Top, Show, Keep, 2, 2 );
             wlist.append(nw);
         }
     }
 #endif
 
 #ifdef MEM_STATES
-    nw = LayoutInfo( fMEMStatus, false, 1, false, 2, 2, false );
+    nw = LayoutInfo( fMEMStatus, Over, Top, Same, Keep, 2, 2 );
     wlist.append(nw);
 #endif
 
     if (taskBarShowNetStatus) {
-        NetStatusControl::IterType it = fNetStatus->getIterator();
+        auto it = fNetStatus->getIterator();
         while (++it)
         {
             if (*it != 0) {
-                nw = LayoutInfo(*it, false, 1, false, 2, 2, false);
+                nw = LayoutInfo(*it, Over, Top, Same, Keep, 2, 2 );
                 wlist.append(nw);
             }
         }
     }
 #ifdef MAX_ACPI_BATTERY_NUM
-    nw = LayoutInfo( fApm, false, 1, true, 0, 2, false );
+    nw = LayoutInfo( fApm, Over, Top, Show, Keep, 0, 2 );
     wlist.append(nw);
 #endif
-    nw = LayoutInfo( fDesktopTray, false, 1, true, 1, 1, false );
+    nw = LayoutInfo( fDesktopTray, Over, Top, Show, Keep, 1, 1 );
     wlist.append(nw);
-    nw = LayoutInfo( fWindowTray, false, 0, true, 1, 1, true );
+    nw = LayoutInfo( fWindowTray, Over, Bot, Show, Grow, 1, 1 );
     wlist.append(nw);
     const int wcount = wlist.getCount();
 
@@ -517,12 +519,12 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
 
     if (!taskBarDoubleHeight)
         for (int i = 0; i < wcount; i++)
-            wlist[i].row = 0;
+            wlist[i].row = false;
     for (int i = 0; i < wcount; i++) {
-        if (!wlist[i].w)
-             continue;
-        if (wlist[i].w->height() > h[wlist[i].row])
-            h[wlist[i].row] = wlist[i].w->height();
+        if (wlist[i].w) {
+            if (h[wlist[i].row] < wlist[i].w->height())
+                h[wlist[i].row] = wlist[i].w->height();
+        }
     }
 
     unsigned w = (desktop->getScreenGeometry().width()
@@ -544,7 +546,7 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
 
     right[0] = w;
     right[1] = w;
-    if (taskBarShowWindows && fTasks != 0) {
+    if (taskBarShowWindows && fTasks != nullptr) {
         h[0] = max(h[0], max(YIcon::smallSize() + 8, fTasks->maxHeight()));
     }
 
@@ -585,6 +587,7 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
 
     if (taskBarShowWindows) {
         if (fTasks) {
+            fTasks->hide();
             fTasks->setGeometry(YRect(left[0],
                                       y[0],
                                       max(0, right[0] - left[0]),
@@ -612,7 +615,12 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
 }
 
 void TaskBar::relayoutNow() {
-    manager->lockWorkArea();
+    if (fUpdates.nonempty()) {
+        for (YFrameWindow* frame : fUpdates) {
+            frame->updateAppStatus();
+        }
+        fUpdates.clear();
+    }
     if (windowTrayPane())
         windowTrayPane()->relayoutNow();
     if (fNeedRelayout) {
@@ -624,57 +632,59 @@ void TaskBar::relayoutNow() {
         fButtonUpdate = false;
         buttonUpdate();
     }
-    manager->unlockWorkArea();
+    if (fWorkspacesUpdate) {
+        fWorkspacesUpdate = false;
+        fWorkspaces->repaint();
+    }
 }
 
 void TaskBar::updateFullscreen(bool fullscreen) {
-    fFullscreen = fullscreen;
-    if (fFullscreen || fIsHidden)
-        fEdgeTrigger->show();
-    else
-        fEdgeTrigger->hide();
+    if (fFullscreen != fullscreen && getFrame()) {
+        fFullscreen = fullscreen;
+        fEdgeTrigger->show((fFullscreen | fIsHidden) && !fIsCollapsed);
+    }
 }
 
 void TaskBar::updateLocation() {
     fNeedRelayout = false;
 
-    if (fIsHidden) {
-        if (fIsMapped && getFrame())
+    if (fIsHidden && !fIsCollapsed) {
+        if (getFrame() && visible())
             getFrame()->wmHide();
-        else
-            hide();
         xapp->sync();
     }
 
     int dx, dy;
     unsigned dw, dh;
-    manager->getScreenGeometry(&dx, &dy, &dw, &dh, -1);
+    desktop->getScreenGeometry(&dx, &dy, &dw, &dh, -1);
 
     int x = dx;
     unsigned int w = 0;
     unsigned int h = 0;
 
-    w = (dw/100.0) * taskBarWidthPercentage;
-    if (strcmp(taskBarJustify, "right") == 0) x = dw - w;
-    if (strcmp(taskBarJustify, "center") == 0) x = (dw - w)/2;
+    if (taskBarWidthPercentage < 100) {
+        w = (dw * taskBarWidthPercentage + 50) / 100;
+        if (strcmp(taskBarJustify, "right") == 0)
+            x = dx + (dw - w);
+        if (strcmp(taskBarJustify, "center") == 0)
+            x = dx + (dw - w)/2;
+    }
 
     updateLayout(w, h);
 
     if (fIsCollapsed) {
         if (fCollapseButton) {
             w = fCollapseButton->width();
-            h = fCollapseButton->height();
-        } else {
+            h = fCollapseButton->height() + 1;
+            fCollapseButton->setPosition(0, 1);
+            fCollapseButton->raise();
+            fCollapseButton->show();
+        }
+        else {
             w = h = 0;
         }
 
-        x = dw - w;
-
-        if (fCollapseButton) {
-            fCollapseButton->show();
-            fCollapseButton->raise();
-            fCollapseButton->setPosition(0, 0);
-        }
+        x = dx + (dw - w);
     }
 
     int by = taskBarAtTop ? dy : dy + dh - 1;
@@ -683,17 +693,24 @@ void TaskBar::updateLocation() {
 
     int y = taskBarAtTop ? dy : dy + dh - h;
 
-    if ( !fIsHidden) {
-        if (fIsMapped && getFrame()) {
-            getFrame()->configureClient(x, y, w, h);
+    if ( !fIsHidden || fIsCollapsed) {
+        if (getFrame()) {
+            if (geometry() != YRect(x, y, w, h)) {
+                XConfigureRequestEvent conf;
+                conf.type = ConfigureRequest;
+                conf.window = handle();
+                conf.x = x;
+                conf.y = y;
+                conf.width = int(w);
+                conf.height = int(h);
+                conf.value_mask = CWX | CWY | CWWidth | CWHeight;
+                getFrame()->configureClient(conf);
+            }
             getFrame()->wmShow();
         } else
             setGeometry(YRect(x, y, w, h));
     }
-    if (fIsHidden || fFullscreen)
-        fEdgeTrigger->show();
-    else
-        fEdgeTrigger->hide();
+    fEdgeTrigger->show((fFullscreen | fIsHidden) && !fIsCollapsed);
 
     ///!!! fix
     updateWMHints();
@@ -702,11 +719,13 @@ void TaskBar::updateLocation() {
 void TaskBar::updateWMHints() {
     YStrut strut;
     if (!taskBarAutoHide && !fIsCollapsed) {
-        Atom h = Atom(height());
-        if (taskBarAtTop)
-            strut.top = h;
-        else
-            strut.bottom = h;
+        YRect geo = desktop->getScreenGeometry();
+        if (y() + height() == geo.y() + geo.height()) {
+            strut.bottom = Atom(height());
+        }
+        else if (y() == geo.y()) {
+            strut.top = Atom(height());
+        }
     }
     if (fStrut != strut) {
         fStrut = strut;
@@ -715,6 +734,26 @@ void TaskBar::updateWMHints() {
     }
 }
 
+void TaskBar::updateWinLayer() {
+    long layer = (taskBarAutoHide || fFullscreen) ? WinLayerAboveAll
+               : fIsCollapsed ? WinLayerAboveDock
+               : taskBarKeepBelow ? WinLayerBelow : WinLayerDock;
+    if (getFrame()) {
+        getFrame()->wmSetLayer(layer);
+    } else {
+        setLayerHint(layer);
+    }
+}
+
+void TaskBar::handleFocus(const XFocusChangeEvent& focus) {
+    if (focus.type == FocusOut) {
+        if (focus.mode == NotifyUngrab) {
+            if (focus.detail == NotifyPointer) {
+                updateWMHints();
+            }
+        }
+    }
+}
 
 void TaskBar::handleCrossing(const XCrossingEvent &crossing) {
     unsigned long last = YWindow::getLastEnterNotifySerial();
@@ -722,7 +761,7 @@ void TaskBar::handleCrossing(const XCrossingEvent &crossing) {
         (crossing.serial != last + 1 || crossing.detail != NotifyVirtual));
 
     if (crossing.type == EnterNotify) {
-        fEdgeTrigger->stopHide();
+        fEdgeTrigger->stopTimer();
     }
     else if (crossing.type == LeaveNotify) {
         if (crossing.detail == NotifyInferior ||
@@ -730,11 +769,11 @@ void TaskBar::handleCrossing(const XCrossingEvent &crossing) {
            (crossing.detail == NotifyAncestor && crossing.mode != NotifyNormal))
         {
             if (ahwm_hack) {
-                fEdgeTrigger->stopHide();
+                fEdgeTrigger->stopTimer();
             }
         } else {
             if (ahwm_hack) {
-                fEdgeTrigger->startHide();
+                fEdgeTrigger->startTimer();
             }
         }
     }
@@ -754,22 +793,23 @@ void TaskBar::paint(Graphics &g, const YRect& r) {
     //g.draw3DRect(0, 0, width() - 1, height() - 1, true);
 
     // When TaskBarDoubleHeight=1 this draws the upper half.
-    if (fGradient != null)
+    if (fGradient != null) {
         g.drawImage(fGradient, r.x(), r.y(), r.width(), r.height(),
                     r.x(), r.y());
-    else
-    if (taskbackPixmap != null)
+    }
+    else if (taskbackPixmap != null) {
         g.fillPixmap(taskbackPixmap, r.x(), r.y(), r.width(), r.height(),
                      r.x(), r.y());
-    else
-    if (taskBarAtTop) {
+    }
+    else if (taskBarAtTop) {
         bool dh = (r.y() + r.height() == height());
         g.fillRect(r.x(), r.y(), r.width(), r.height() - dh);
         if (dh) {
             g.setColor(taskBarBg->darker());
             g.drawLine(r.x(), height() - 1, r.x() + r.width(), height() - 1);
         }
-    } else {
+    }
+    else {
         bool dy = (r.y() == 0);
         g.fillRect(r.x(), r.y() + dy, r.width(), r.height() - dy);
         if (dy) {
@@ -805,7 +845,7 @@ void TaskBar::handleButton(const XButtonEvent &button) {
 }
 
 void TaskBar::contextMenu(int x_root, int y_root) {
-    taskBarMenu->popup(this, 0, 0, x_root, y_root,
+    taskBarMenu->popup(this, nullptr, nullptr, x_root, y_root,
                        YPopupWindow::pfCanFlipVertical |
                        YPopupWindow::pfCanFlipHorizontal);
 }
@@ -838,12 +878,10 @@ void TaskBar::handleDrag(const XButtonEvent &/*down*/, const XMotionEvent &motio
 
     if (taskBarAtTop != newPosition) {
         taskBarAtTop = newPosition;
-        //setPosition(x(), taskBarAtTop ? -1 : int(manager->height() - height() + 1));
-        manager->setWorkAreaMoveWindows(true);
+        //setPosition(x(), taskBarAtTop ? -1 : int(desktop->height() - height() + 1));
         updateLocation();
-        repaint();
+        //repaint();
         //manager->updateWorkArea();
-        manager->setWorkAreaMoveWindows(false);
     }
 }
 
@@ -892,20 +930,17 @@ void TaskBar::popOut() {
         fIsHidden = taskBarAutoHide;
         if (fEdgeTrigger) {
             MSG(("start hide 4"));
-            fEdgeTrigger->startHide();
+            fEdgeTrigger->startTimer();
         }
     }
     relayoutNow();
 }
 
 void TaskBar::showBar() {
-    if (getFrame() == 0) {
-        manager->lockWorkArea();
-        manager->mapClient(handle());
-        if (getFrame() != 0) {
-            setWinLayerHint((taskBarAutoHide || fFullscreen) ? WinLayerAboveAll :
-                            fIsCollapsed ? WinLayerAboveDock :
-                            taskBarKeepBelow ? WinLayerBelow : WinLayerDock);
+    if (getFrame() == nullptr) {
+        manager->manageClient(handle());
+        updateWinLayer();
+        if (getFrame()) {
             getFrame()->setAllWorkspaces();
             if (enableAddressBar && ::showAddressBar && taskBarDoubleHeight)
                 getFrame()->activate(true);
@@ -913,7 +948,6 @@ void TaskBar::showBar() {
             parent()->setTitle("TaskBarFrame");
             getFrame()->updateLayer();
         }
-        manager->unlockWorkArea();
     }
 }
 
@@ -930,8 +964,13 @@ void TaskBar::handleCollapseButton() {
         fCollapseButton->repaint();
     }
 
+    if (fIsCollapsed)
+        updateWinLayer();
     relayout();
     updateLocation();
+    if (fIsCollapsed == false)
+        updateWinLayer();
+    xapp->sync();
 }
 
 void TaskBar::handlePopDown(YPopupWindow * /*popup*/) {
@@ -939,32 +978,38 @@ void TaskBar::handlePopDown(YPopupWindow * /*popup*/) {
 
 void TaskBar::configure(const YRect2& r) {
     if (r.resized() && 1 < r.width()) {
-        fGraphics.paint(r);
+        repaint();
     }
+    updateWMHints();
 }
 
 void TaskBar::repaint() {
-    fGraphics.paint();
+    GraphicsBuffer(this).paint();
 }
 
 void TaskBar::detachDesktopTray() {
     if (fDesktopTray) {
         MSG(("detach Tray"));
         fDesktopTray->detachTray();
-        delete fDesktopTray; fDesktopTray = 0;
+        delete fDesktopTray; fDesktopTray = nullptr;
     }
 }
 
-void TaskBar::removeTasksApp(YFrameWindow *w) {
-    if (taskPane())
-        taskPane()->removeApp(w);
+void TaskBar::updateFrame(YFrameWindow* frame) {
+    if (find(fUpdates, frame) < 0)
+        fUpdates += frame;
 }
 
-TaskBarApp *TaskBar::addTasksApp(YFrameWindow *w) {
-    if (taskPane())
-        return taskPane()->addApp(w);
-    else
-        return 0;
+void TaskBar::delistFrame(YFrameWindow* frame, TaskBarApp* task, TrayApp* tray) {
+    findRemove(fUpdates, frame);
+    if (taskPane() && task)
+        taskPane()->remove(task);
+    if (windowTrayPane() && tray)
+        windowTrayPane()->remove(tray);
+}
+
+TaskBarApp *TaskBar::addTasksApp(YFrameWindow* frame) {
+    return taskPane() ? taskPane()->addApp(frame) : nullptr;
 }
 
 void TaskBar::relayoutTasks() {
@@ -972,16 +1017,8 @@ void TaskBar::relayoutTasks() {
         taskPane()->relayout();
 }
 
-void TaskBar::removeTrayApp(YFrameWindow *w) {
-    if (windowTrayPane())
-        windowTrayPane()->removeApp(w);
-}
-
-TrayApp *TaskBar::addTrayApp(YFrameWindow *w) {
-    if (windowTrayPane())
-        return windowTrayPane()->addApp(w);
-    else
-        return 0;
+TrayApp *TaskBar::addTrayApp(YFrameWindow* frame) {
+    return windowTrayPane() ? windowTrayPane()->addApp(frame) : nullptr;
 }
 
 void TaskBar::relayoutTray() {
@@ -991,7 +1028,7 @@ void TaskBar::relayoutTray() {
 
 void TaskBar::showAddressBar() {
     popOut();
-    if (fAddressBar != 0)
+    if (fAddressBar != nullptr)
         fAddressBar->showNow();
 }
 
@@ -1007,7 +1044,7 @@ void TaskBar::setWorkspaceActive(long workspace, bool active) {
 
 void TaskBar::workspacesRepaint() {
     if (taskBarShowWorkspaces && fWorkspaces) {
-        fWorkspaces->repaint();
+        fWorkspacesUpdate = true;
     }
 }
 
@@ -1035,6 +1072,12 @@ void TaskBar::workspacesRelabelButtons() {
     }
 }
 
+void TaskBar::keyboardUpdate(mstring keyboard) {
+    if (fKeyboardStatus) {
+        fKeyboardStatus->updateKeyboard(keyboard);
+    }
+}
+
 bool TaskBar::windowTrayRequestDock(Window w) {
     if (fDesktopTray) {
         fDesktopTray->trayRequestDock(w, "SystemTray");
@@ -1043,27 +1086,35 @@ bool TaskBar::windowTrayRequestDock(Window w) {
     return false;
 }
 
-void TaskBar::switchToPrev()
-{
+void TaskBar::switchToPrev() {
     if (taskPane())
         taskPane()->switchToPrev();
 }
 
-void TaskBar::switchToNext()
-{
+void TaskBar::switchToNext() {
     if (taskPane())
         taskPane()->switchToNext();
 }
 
-void TaskBar::movePrev()
-{
+void TaskBar::movePrev() {
     if (taskPane())
         taskPane()->movePrev();
 }
 
-void TaskBar::moveNext()
-{
+void TaskBar::moveNext() {
     if (taskPane())
         taskPane()->moveNext();
 }
+
+void TaskBar::refresh() {
+    if (fApplications)
+        fApplications->repaint();
+    if (fWinList)
+        fWinList->repaint();
+    if (fShowDesktop)
+        fShowDesktop->repaint();
+    if (fObjectBar)
+        fObjectBar->refresh();
+}
+
 // vim: set sw=4 ts=4 et:
