@@ -1,5 +1,5 @@
-#ifndef __TASKBAR_H
-#define __TASKBAR_H
+#ifndef TASKBAR_H
+#define TASKBAR_H
 
 #include "yaction.h"
 #include "ytimer.h"
@@ -13,6 +13,7 @@ class MEMStatus;
 class CPUStatusControl;
 class NetStatusControl;
 class AddressBar;
+class KeyboardStatus;
 class MailBoxControl;
 class MailBoxStatus;
 class YButton;
@@ -26,16 +27,19 @@ class WorkspacesPane;
 class YXTray;
 class YSMListener;
 class TaskBar;
+class TaskBarApp;
+class TrayApp;
 
-class EdgeTrigger: public YWindow, public YTimerListener {
+class EdgeTrigger: public YDndWindow, public YTimerListener {
 public:
     EdgeTrigger(TaskBar *owner);
     virtual ~EdgeTrigger();
 
-    bool enabled() const;
-    void show();
-    void startHide();
-    void stopHide();
+    static bool enabled();
+    void show(bool enable);
+    enum HideOrShow { Hide, Show };
+    void startTimer(HideOrShow = Hide);
+    void stopTimer();
 
     virtual void handleDNDEnter();
     virtual void handleDNDLeave();
@@ -45,7 +49,7 @@ public:
 private:
     TaskBar *fTaskBar;
     lazy<YTimer> fAutoHideTimer;
-    bool fDoShow;
+    HideOrShow fHideOrShow;
 };
 
 class TaskBar:
@@ -66,7 +70,7 @@ private:
     virtual void handleClick(const XButtonEvent &up, int count);
     virtual void handleDrag(const XButtonEvent &down, const XMotionEvent &motion);
     virtual void handleEndDrag(const XButtonEvent &down, const XButtonEvent &up);
-
+    virtual void handleFocus(const XFocusChangeEvent& focus);
     virtual void handleCrossing(const XCrossingEvent &crossing);
     virtual void handleExpose(const XExposeEvent &expose) {}
 
@@ -75,6 +79,7 @@ private:
 
     void updateWMHints();
     void updateLocation();
+    void updateWinLayer();
     virtual void configure(const YRect2 &r);
     virtual void repaint();
 
@@ -86,10 +91,16 @@ public:
     void workspacesRepaint();
     void workspacesUpdateButtons();
     void workspacesRelabelButtons();
+    void keyboardUpdate(mstring keyboard);
 
-    void removeTasksApp(YFrameWindow *w);
-    class TaskBarApp *addTasksApp(YFrameWindow *w);
+    void updateFrame(YFrameWindow* frame);
+    void delistFrame(YFrameWindow* frame, TaskBarApp* task, TrayApp* tray);
+    void removeTasksApp(YFrameWindow* frame);
+    TaskBarApp* addTasksApp(YFrameWindow* frame);
     void relayoutTasks();
+    void relayoutTray();
+    TrayApp* addTrayApp(YFrameWindow* frame);
+    void removeTrayApp(YFrameWindow* frame);
 
     void popupStartMenu();
     void popupWindowListMenu();
@@ -103,11 +114,7 @@ public:
 
     void detachDesktopTray();
 
-    void relayoutTray();
-    class TrayApp *addTrayApp(YFrameWindow *w);
-    void removeTrayApp(YFrameWindow *w);
-
-    bool hidden() const { return fIsCollapsed | fIsHidden | !fIsMapped; }
+    bool hidden() const { return fIsCollapsed | fIsHidden | !getFrame(); }
     bool autoTimer(bool show);
     void updateFullscreen(bool fullscreen);
     Window edgeTriggerWindow() { return fEdgeTrigger->handle(); }
@@ -115,6 +122,7 @@ public:
     void switchToNext();
     void movePrev();
     void moveNext();
+    void refresh();
 
 private:
     void popOut();
@@ -123,7 +131,7 @@ private:
     TaskPane *taskPane() const { return fTasks; }
     TrayPane *windowTrayPane() const { return fWindowTray; }
 
-    virtual ref<YImage> getGradient() const { return fGradient; }
+    virtual ref<YImage> getGradient() { return fGradient; }
     const YSurface& getSurface() const { return fSurface; }
 
     void contextMenu(int x_root, int y_root);
@@ -131,15 +139,18 @@ private:
     void trayChanged();
     YXTray *netwmTray() { return fDesktopTray; }
 
+    void initApplets();
+    void updateLayout(unsigned& size_w, unsigned& size_h);
+
 private:
-    GraphicsBuffer fGraphics;
     YSurface fSurface;
     TaskPane *fTasks;
 
     ObjectButton *fCollapseButton;
     TrayPane *fWindowTray;
     YClock *fClock;
-    MailBoxControl *fMailBoxStatus;
+    KeyboardStatus *fKeyboardStatus;
+    MailBoxControl *fMailBoxControl;
     MEMStatus *fMEMStatus;
     CPUStatusControl *fCPUStatus;
     YApm *fApm;
@@ -152,30 +163,22 @@ private:
     AddressBar *fAddressBar;
     AWorkspaces *fWorkspaces;
     YXTray *fDesktopTray;
+    EdgeTrigger *fEdgeTrigger;
     YActionListener *wmActionListener;
     YSMListener *smActionListener;
     IApp *app;
 
+    lazy<TaskBarMenu> taskBarMenu;
+    ref<YImage> fGradient;
+    YArray<YFrameWindow*> fUpdates;
+
     bool fIsHidden;
     bool fFullscreen;
     bool fIsCollapsed;
-    bool fIsMapped;
     bool fMenuShown;
-
-    lazy<TaskBarMenu> taskBarMenu;
-
-    friend class WindowList;
-    friend class WindowListBox;
-
-    ref<YImage> fGradient;
-
     bool fNeedRelayout;
     bool fButtonUpdate;
-
-    void initApplets();
-    void updateLayout(unsigned &size_w, unsigned &size_h);
-
-    EdgeTrigger *fEdgeTrigger;
+    bool fWorkspacesUpdate;
 
     class YStrut {
     public:
