@@ -11,7 +11,7 @@
 #include "yrect.h"
 #include "yicon.h"
 #include "wpixmaps.h"
-#include "yxapp.h" // !!! remove (AltMask)
+#include "yxapp.h"
 #include "yprefs.h"
 #include "prefs.h"
 #include "ascii.h"
@@ -29,6 +29,7 @@ YColorName YButton::activeButtonFg(&clrActiveButtonText);
 
 ref<YFont> YButton::normalButtonFont;
 ref<YFont> YButton::activeButtonFont;
+int YButton::buttonObjectCount;
 
 YButton::YButton(YWindow *parent, YAction action, YMenu *popup) :
     YWindow(parent),
@@ -41,15 +42,12 @@ YButton::YButton(YWindow *parent, YAction action, YMenu *popup) :
     fPressed(false),
     fEnabled(true),
     fHotCharPos(-1), hotKey(-1),
-    fListener(NULL),
+    fListener(nullptr),
     fSelected(false), fArmed(false),
     wasPopupActive(false),
     fPopupActive(false)
 {
-    if (normalButtonFont == null)
-        normalButtonFont = YFont::getFont(XFA(normalButtonFontName));
-    if (activeButtonFont == null)
-        activeButtonFont = YFont::getFont(XFA(activeButtonFontName));
+    ++buttonObjectCount;
 }
 
 YButton::~YButton() {
@@ -60,7 +58,11 @@ YButton::~YButton() {
     }
     popdown();
     if (fPopup && fPopup->isShared() == false) {
-        delete fPopup;
+        delete fPopup; fPopup = nullptr;
+    }
+    if (--buttonObjectCount == 0) {
+        normalButtonFont = null;
+        activeButtonFont = null;
     }
 }
 
@@ -69,21 +71,22 @@ void YButton::paint(Graphics &g, int const d, const YRect &r) {
     YSurface surface(getSurface());
     g.drawSurface(surface, x, y, w, h);
 
-    if (fIcon != null)
+    if (fIcon != null) {
         fIcon->draw(g,
                     x + (w - fIconSize) / 2,
                     y + (h - fIconSize) / 2,
                     fIconSize);
-    else
-    if (fImage != null)
+    }
+    else if (fImage != null) {
         g.drawImage(fImage,
                     x + (w - fImage->width()) / 2,
                     y + (h - fImage->height()) / 2);
+    }
     else if (fText != null) {
-        ref<YFont> font = fPressed ? activeButtonFont : normalButtonFont;
+        ref<YFont> font(getFont());
 
         int const w(font->textWidth(fText));
-        int const p((width() - w) / 2);
+        int const p((width() + 1 - w) / 2);
         int yp((height() - font->height()) / 2
                + font->ascent() + d);
 
@@ -128,6 +131,9 @@ void YButton::paint(Graphics &g, const YRect &/*r*/) {
         if (wmLook != lookFlat) {
             paintFocus(g, YRect(x, y, w, h));
         }
+        if (surface.gradient != null) {
+            g.maxOpacity();
+        }
     }
 }
 
@@ -264,7 +270,7 @@ void YButton::updatePopup() {
 
 void YButton::handleButton(const XButtonEvent &button) {
     if (fEnabled) {
-        if (button.type == ButtonPress && button.button == 1) {
+        if (button.type == ButtonPress && button.button == Button1) {
             requestFocus(false);
             wasPopupActive = fArmed;
             setSelected(true);
@@ -278,16 +284,24 @@ void YButton::handleButton(const XButtonEvent &button) {
 
                 if ((!inWindow || wasPopupActive) && fArmed) {
                     setArmed(false, false);
-                    setSelected(false);
                 }
             }
+            setSelected(false);
         }
     }
+    bool unarm = fEnabled && fArmed
+              && button.type == ButtonRelease
+              && button.button == Button1
+              && getClickCount() == 1
+              && dragging();
     YWindow::handleButton(button);
+    if (unarm && !fPopupActive) {
+        setArmed(false, false);
+    }
 }
 
 void YButton::handleClick(const XButtonEvent &button, int count) {
-    if (fEnabled && fPopup == 0) {
+    if (fEnabled && fPopup == nullptr) {
         bool wasArmed = fArmed;
 
         setArmed(false, false);
@@ -320,20 +334,44 @@ void YButton::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
+ref<YFont> YButton::getActiveFont() {
+    if (activeButtonFont == null)
+        activeButtonFont = YFont::getFont(XFA(activeButtonFontName));
+    return activeButtonFont;
+}
+
+ref<YFont> YButton::getNormalFont() {
+    if (normalButtonFont == null)
+        normalButtonFont = YFont::getFont(XFA(normalButtonFontName));
+    return normalButtonFont;
+}
+
+YDimension YButton::getTextSize() {
+    if (fText != null) {
+        ref<YFont> font(getActiveFont());
+        return YDimension(font->textWidth(fText), font->height());
+    } else {
+        return YDimension(1, 1);
+    }
+}
+
 void YButton::updateSize() {
     int w = 72;
     int h = 18;
     if (fIcon != null) {
         w = h = fIconSize;
-    } else if (fImage != null) {
+    }
+    else if (fImage != null) {
         w = fImage->width();
         h = fImage->height();
-    } else if (fText != null) {
-        w = activeButtonFont->textWidth(fText);
-        h = activeButtonFont->ascent();
     }
-    setSize(w + 3 + 2 - (wmLook == lookMetal || wmLook == lookFlat),
-            h + 3 + 2 - (wmLook == lookMetal || wmLook == lookFlat));
+    else if (fText != null) {
+        YDimension d(getTextSize());
+        w = d.w;
+        h = d.h;
+    }
+    setSize(w + 3 + 2 - LOOK(lookMetal | lookFlat),
+            h + 3 + 2 - LOOK(lookMetal | lookFlat));
 }
 
 void YButton::setIcon(ref<YIcon> icon, int iconSize) {
@@ -352,7 +390,7 @@ void YButton::setImage(ref<YImage> image) {
     updateSize();
 }
 
-void YButton::setText(const ustring &str, int hotChar) {
+void YButton::setText(const mstring &str, int hotChar) {
     if (hotKey != -1) {
         removeAccelerator(hotKey, 0, this);
         if (xapp->AltMask != 0)
@@ -410,7 +448,7 @@ void YButton::popup(bool mouseDown) {
         fPopup->setActionListener(getActionListener());
 
         int xiscreen = desktop->getScreenForRect(x, y, width(), height());
-        if (fPopup->popup(this, this, 0,
+        if (fPopup->popup(this, this, nullptr,
                           x - 2,
                           y + height(),
                           0,
@@ -443,7 +481,7 @@ void YButton::actionPerformed(YAction action, unsigned modifiers) {
 }
 
 ref<YFont> YButton::getFont() {
-    return (fPressed ? activeButtonFont : normalButtonFont);
+    return fPressed ? getActiveFont() : getNormalFont();
 }
 
 YColor YButton::getColor() {
