@@ -21,12 +21,13 @@ protected:
 
 class YInputLine: public YWindow, public YTimerListener, public YActionListener {
 public:
-    YInputLine(YWindow *parent = 0, YInputListener *listener = 0);
+    YInputLine(YWindow *parent = nullptr, YInputListener *listener = nullptr);
     virtual ~YInputLine();
 
-    void setText(const ustring &text, bool asMarked);
-    ustring getText();
+    void setText(const mstring &text, bool asMarked);
+    mstring getText();
     ref<YFont> getFont() const { return inputFont; }
+    void setListener(YInputListener* listener) { fListener = listener; }
 
     virtual void paint(Graphics &g, const YRect &r);
     virtual bool handleKey(const XKeyEvent &key);
@@ -43,7 +44,7 @@ public:
 
     bool move(unsigned pos, bool extend);
     bool hasSelection() const { return (curPos != markPos) ? true : false; }
-    void replaceSelection(const ustring &str);
+    void replaceSelection(const mstring &str);
     bool deleteSelection();
     bool deleteNextChar();
     bool deletePreviousChar();
@@ -68,7 +69,7 @@ private:
     void autoScroll(int delta, const XMotionEvent *mouse);
     unsigned offsetToPos(int offset);
 
-    ustring fText;
+    mstring fText;
     unsigned markPos;
     unsigned curPos;
     int leftOfs;
