@@ -1,11 +1,11 @@
-#ifndef __YLABEL_H
-#define __YLABEL_H
+#ifndef YLABEL_H
+#define YLABEL_H
 
 #include "ywindow.h"
 
 class YLabel: public YWindow {
 public:
-    YLabel(const ustring &label, YWindow *parent = 0);
+    YLabel(const mstring& label, YWindow* parent);
     virtual ~YLabel();
 
     virtual void paint(Graphics &g, const YRect &r);
@@ -13,10 +13,10 @@ public:
     virtual void configure(const YRect2& r);
     virtual void repaint();
 
-    void setText(const ustring &label);
-    const ustring getText() const { return fLabel; }
+    void setText(const char* label);
+
 private:
-    ustring fLabel;
+    mstring fLabel;
     bool fPainted;
 
     void autoSize();
@@ -24,6 +24,7 @@ private:
     static YColorName labelFg;
     static YColorName labelBg;
     static ref<YFont> labelFont;
+    static int labelObjectCount;
 };
 
 #endif
