@@ -30,6 +30,12 @@ inline int getIconSize() {
     return int(YIcon::smallSize());
 }
 
+inline ref<YFont> getListBoxFont() {
+    if (listBoxFont == null)
+        listBoxFont = YFont::getFont(XFA(listBoxFontName));
+    return listBoxFont;
+}
+
 int YListBox::fAutoScrollDelta = 0;
 
 int YListItem::getWidth() {
@@ -37,8 +43,8 @@ int YListItem::getWidth() {
     if (getIcon() != null) {
         width += getIconSize();
     }
-    if (getText() != null && listBoxFont != null) {
-        width += listBoxFont->textWidth(getText()) + 3;
+    if (getText() != null) {
+        width += getListBoxFont()->textWidth(getText()) + 3;
     }
     return width;
 }
@@ -80,6 +86,7 @@ YListBox::YListBox(YScrollView *view, YWindow *aParent):
 
 YListBox::~YListBox() {
     fGradient = null;
+    listBoxFont = null;
 }
 
 bool YListBox::isFocusTraversable() {
@@ -266,7 +273,7 @@ bool YListBox::handleKey(const XKeyEvent &key) {
                     activateItem(i);
             }
             break;
-        case ' ':
+        case XK_space:
             if (fFocusedItem != -1) {
                 selectItem(fFocusedItem,
                            isSelected(fFocusedItem) ? false : true);
@@ -274,10 +281,14 @@ bool YListBox::handleKey(const XKeyEvent &key) {
             break;
         case XK_Home:
             setFocusedItem(0, clear, extend, false);
+            if (fHorizontalScroll)
+                fHorizontalScroll->move(0);
             break;
         case XK_End:
             if (getItemCount() > 0)
                 setFocusedItem(getItemCount() - 1, clear, extend, false);
+            if (fHorizontalScroll)
+                fHorizontalScroll->move(0);
             break;
         case XK_Up: {
             int const oldFocus(fFocusedItem);
@@ -323,9 +334,17 @@ bool YListBox::handleKey(const XKeyEvent &key) {
                                clear, extend, false);
             break;
         }
-        case 'a':
-        case '/':
-        case '\\':
+        case XK_Left:
+        case XK_KP_Left:
+        case XK_Right:
+        case XK_KP_Right:
+            if (fHorizontalScroll && fHorizontalScroll->handleScrollKeys(key)) {
+                return true;
+            }
+            break;
+        case XK_a:
+        case XK_slash:
+        case XK_backslash:
             if (m & ControlMask) {
                 for (int i = 0; i < getItemCount(); i++)
                     selectItem(i, (k != '\\'));
@@ -344,7 +363,7 @@ bool YListBox::handleKey(const XKeyEvent &key) {
 
                 for (int n = 0; n < 2 * getItemCount(); ++n) {
                     int i = (fFocusedItem + 1 + n) % getItemCount();
-                    cstring title(getItem(i)->getText());
+                    mstring title(getItem(i)->getText());
                     if (n < getItemCount()
                         ? strncasecmp(title, kstr, klen) == 0
                         : strstr(title, kstr) != nullptr)
@@ -412,7 +431,7 @@ void YListBox::handleButton(const XButtonEvent &button) {
                 setFocusedItem(no, false, true, true);
             fDragging = false;
             applySelection();
-            autoScroll(0, 0);
+            autoScroll(0, nullptr);
         }
     }
     if (fVerticalScroll->handleScrollMouse(button) == false)
@@ -493,12 +512,12 @@ void YListBox::paintItem(Graphics &g, int n) {
     YListItem *a = getItem(n);
     int x = 3;
     int lh = getLineHeight();
-    int fh = listBoxFont->height();
+    int fh = getListBoxFont()->height();
     int xpos = 0;
     int y = n * lh;
     int yPos = y + lh - (lh - fh) / 2 - listBoxFont->descent();
 
-    if (a == 0)
+    if (a == nullptr)
         return ;
 
     xpos += a->getOffset();
@@ -534,7 +553,7 @@ void YListBox::paintItem(Graphics &g, int n) {
         g.setPenStyle(true);
         int cw = 3 + 20 + a->getOffset();
         if (listBoxFont != null) {
-            ustring t = a->getText();
+            mstring t = a->getText();
             if (t != null)
                 cw += listBoxFont->textWidth(t) + 3;
         }
@@ -552,14 +571,12 @@ void YListBox::paintItem(Graphics &g, int n) {
         }
     }
 
-    ustring title = a->getText();
-
+    mstring title(a->getText());
     if (title != null) {
         g.setColor(s ? listBoxSelFg : listBoxFg);
         g.setFont(listBoxFont);
-
         g.drawChars(title,
-                    xpos + x + 20 - fOffsetX, yPos - fOffsetY);
+                    xpos + x + 4 + getIconSize() - fOffsetX, yPos - fOffsetY);
     }
 }
 
@@ -792,8 +809,4 @@ unsigned YListBox::contentHeight() {
     return getItemCount() * getLineHeight();
 }
 
-YWindow *YListBox::getWindow() {
-    return this;
-}
-
 // vim: set sw=4 ts=4 et:
