@@ -4,30 +4,20 @@
  * Copyright (C) 1997-2002 Marko Macek
  */
 #include "config.h"
-
-#include "yfull.h"
-#include "ykey.h"
 #include "ymenu.h"
-#include "yaction.h"
 #include "ymenuitem.h"
-#include "yrect.h"
-
 #include "yxapp.h"
 #include "prefs.h"
 #include "yprefs.h"
-
 #include "ascii.h"
-
-#include "yicon.h"
-
 #include <string.h>
 
-YColorName menuBg(&clrNormalMenu);
-YColorName menuItemFg(&clrNormalMenuItemText);
-YColorName activeMenuItemBg(&clrActiveMenuItem);
-YColorName activeMenuItemFg(&clrActiveMenuItemText);
-YColorName disabledMenuItemFg(&clrDisabledMenuItemText);
-YColorName disabledMenuItemSt(&clrDisabledMenuItemShadow);
+static YColorName menuBg(&clrNormalMenu);
+static YColorName menuItemFg(&clrNormalMenuItemText);
+static YColorName activeMenuItemBg(&clrActiveMenuItem);
+static YColorName activeMenuItemFg(&clrActiveMenuItemText);
+static YColorName disabledMenuItemFg(&clrDisabledMenuItemText);
+static YColorName disabledMenuItemSt(&clrDisabledMenuItemShadow);
 
 ref<YFont> menuFont;
 
@@ -43,7 +33,8 @@ int YMenu::fAutoScrollDeltaX = 0;
 int YMenu::fAutoScrollDeltaY = 0;
 int YMenu::fAutoScrollMouseX = -1;
 int YMenu::fAutoScrollMouseY = -1;
-YMenu *YMenu::fPointedMenu = 0;
+int YMenu::fMenuObjectCount;
+YMenu *YMenu::fPointedMenu = nullptr;
 
 void YMenu::setActionListener(YActionListener *actionListener) {
     fActionListener = actionListener;
@@ -68,14 +59,11 @@ YMenu::YMenu(YWindow *parent):
     fGradient(null),
     fMenusel(null)
 {
-    if (menuFont == null)
-        menuFont = YFont::getFont(XFA(menuFontName));
-
     paintedItem = selectedItem = -1;
     submenuItem = -1;
-    fPopup = 0;
-    fActionListener = 0;
-    fPopupActive = 0;
+    fPopup = nullptr;
+    fActionListener = nullptr;
+    fPopupActive = nullptr;
     fShared = false;
     fRaised = false;
     activatedX = -1;
@@ -84,6 +72,10 @@ YMenu::YMenu(YWindow *parent):
     fTimerY = 0;
     fTimerSubmenuItem = -1;
     addStyle(wsNoExpose);
+
+    if (menuFont == null)
+        menuFont = YFont::getFont(XFA(menuFontName));
+    ++fMenuObjectCount;
 }
 
 void YMenu::raise() {
@@ -105,6 +97,8 @@ YMenu::~YMenu() {
     fGradient = null;
     fMenusel = null;
     removeAll();
+    if (--fMenuObjectCount == 0)
+        menuFont = null;
 }
 
 void YMenu::activatePopup(int flags) {
@@ -122,7 +116,7 @@ void YMenu::activatePopup(int flags) {
 void YMenu::deactivatePopup() {
     hideSubmenu();
     if (fPointedMenu == this)
-        fPointedMenu = 0;
+        fPointedMenu = nullptr;
     if (fMenuTimer)
         fMenuTimer->disableTimerListener(this);
 }
@@ -131,7 +125,7 @@ void YMenu::donePopup(YPopupWindow *popup) {
     PRECONDITION(popup != 0);
     PRECONDITION(fPopup != 0);
     if (fPointedMenu == this)
-        fPointedMenu = 0;
+        fPointedMenu = nullptr;
     if (fPopup) {
         hideSubmenu();
         if (selectedItem != -1)
@@ -204,7 +198,7 @@ void YMenu::focusItem(int itemNo) {
 }
 
 void YMenu::activateSubMenu(int item, bool byMouse) {
-    YMenu *sub = 0;
+    YMenu *sub = nullptr;
     if (item != -1 && getItem(item)->isEnabled())
         sub = getItem(item)->getSubmenu();
 
@@ -219,9 +213,9 @@ void YMenu::activateSubMenu(int item, bool byMouse) {
             getOffsets(l, t, r, b);
             findItemPos(item, xp, yp, ih);
             YRect rect(x(), y(), width(), height());
-            if (sub->getActionListener() == 0)
+            if (sub->getActionListener() == nullptr)
                 sub->setActionListener(getActionListener());
-            sub->popup(0, this, 0,
+            sub->popup(nullptr, this, nullptr,
                        x() + int(width()) - r, y() + yp - t,
                        int(width()) - r - l, -1,
                        getXiScreen(),
@@ -268,7 +262,7 @@ int YMenu::activateItem(int modifiers, bool byMouse) {
         if (item->getAction() != actionNull) {
             finishPopup(item, item->getAction(), modifiers);
         }
-        else if (item->getSubmenu() != 0) {
+        else if (item->getSubmenu() != nullptr) {
             focusItem(selectedItem);
             activateSubMenu(selectedItem, byMouse);
         }
@@ -370,7 +364,7 @@ bool YMenu::handleKey(const XKeyEvent &key) {
                 } else if (k == XK_Return || k == XK_KP_Enter) {
                     if (selectedItem != -1 &&
                         (getItem(selectedItem)->getAction() != actionNull ||
-                         getItem(selectedItem)->getSubmenu() != 0))
+                         getItem(selectedItem)->getSubmenu() != nullptr))
                     {
                         activateItem(key.state, false);
                         return true;
@@ -434,7 +428,7 @@ void YMenu::handleButton(const XButtonEvent &button) {
         if (button.type == ButtonPress) {
             fPopupActive = fPopup;
         } else if (button.type == ButtonRelease) {
-            if (fPopupActive == fPopup && fPopup != 0 && nocascade) {
+            if (fPopupActive == fPopup && fPopup != nullptr && nocascade) {
                 hideSubmenu();
                 focusItem(selItem);
                 paintItems();
@@ -452,7 +446,7 @@ void YMenu::handleButton(const XButtonEvent &button) {
             if (selectedItem != -1) {
                 noAction =
                 getItem(selectedItem)->getAction() == actionNull &&
-                getItem(selectedItem)->getSubmenu() == 0;
+                getItem(selectedItem)->getSubmenu() == nullptr;
             }
 
             if (selectedItem == -1 || noAction) {
@@ -462,10 +456,10 @@ void YMenu::handleButton(const XButtonEvent &button) {
                     focusItem(findActiveItem(itemCount() - 1, 1));
             } else {
                 if ((getItem(selectedItem)->getAction() != actionNull ||
-                     getItem(selectedItem)->getSubmenu() != 0)
+                     getItem(selectedItem)->getSubmenu() != nullptr)
                     &&
                     (getItem(selectedItem)->getAction() == actionNull ||
-                     getItem(selectedItem)->getSubmenu() == 0 || !nocascade)
+                     getItem(selectedItem)->getSubmenu() == nullptr || !nocascade)
                    )
                 {
 
@@ -638,46 +632,58 @@ void YMenu::autoScroll(int deltaX, int deltaY, int mx, int my, const XMotionEven
     beginAutoScroll(deltaX || deltaY, motion);
 }
 
-YMenuItem *YMenu::addItem(const ustring &name, int hotCharPos, const ustring &param, YAction action, const char *icons) {
-    return add(new YMenuItem(name, hotCharPos, param, action, 0), icons);
+YMenuItem *YMenu::addItem(const mstring &name, int hotCharPos, const mstring &param, YAction action, const char *icons) {
+    return add(new YMenuItem(name, hotCharPos, param, action, nullptr), icons);
 }
 
-YMenuItem *YMenu::addItem(const ustring &name, int hotCharPos, YAction action, YMenu *submenu, const char *icons) {
+YMenuItem *YMenu::addItem(const mstring &name, int hotCharPos, YAction action, YMenu *submenu, const char *icons) {
     return add(new YMenuItem(name, hotCharPos, null, action, submenu), icons);
 }
 
-YMenuItem *YMenu::addSubmenu(const ustring &name, int hotCharPos, YMenu *submenu, const char *icons) {
+YMenuItem *YMenu::addSubmenu(const mstring &name, int hotCharPos, YMenu *submenu, const char *icons) {
     return add(new YMenuItem(name, hotCharPos, null, actionNull, submenu), icons);
 }
 
-YMenuItem *YMenu::addItem(const ustring &name, int hotCharPos, const ustring &param, YAction action) {
-    return add(new YMenuItem(name, hotCharPos, param, action, 0));
+YMenuItem *YMenu::addItem(const mstring &name, int hotCharPos, const mstring &param, YAction action) {
+    return add(new YMenuItem(name, hotCharPos, param, action, nullptr));
 }
 
-YMenuItem *YMenu::addItem(const ustring &name, int hotCharPos, YAction action, YMenu *submenu) {
+YMenuItem *YMenu::addItem(const mstring &name, int hotCharPos, YAction action, YMenu *submenu) {
     return add(new YMenuItem(name, hotCharPos, null, action, submenu));
 }
 
-YMenuItem *YMenu::addSubmenu(const ustring &name, int hotCharPos, YMenu *submenu) {
+YMenuItem *YMenu::addSubmenu(const mstring &name, int hotCharPos, YMenu *submenu) {
     return add(new YMenuItem(name, hotCharPos, null, actionNull, submenu));
 }
 
-YMenuItem * YMenu::addSeparator() {
-    return add(new YMenuItem());
+void YMenu::addSeparator() {
+    int n = itemCount();
+    if (n > 0 && !fItems[n - 1]->isSeparator()) {
+        void(add(new YMenuItem()));
+    }
 }
 
-YMenuItem *YMenu::addLabel(const ustring &name) {
+YMenuItem *YMenu::addLabel(const mstring &name) {
     return add(new YMenuItem(name));
 }
 
+bool YMenu::lastIsSeparator() const {
+    int n = itemCount();
+    return n > 0 && fItems[n - 1]->isSeparator();
+}
+
+YMenuItem* YMenu::lastItem() const {
+    int n = itemCount();
+    return n > 0 ? fItems[n - 1] : nullptr;
+}
+
 void YMenu::removeAll() {
     hideSubmenu();
     const int n = itemCount();
     for (int i = 0; i < n; ++i) {
         YMenu* sub = fItems[i]->getSubmenu();
         if (sub && sub->isShared()) {
-            YMenuItem* nil(nullptr);
-            swap(nil, fItems[i]);
+            fItems[i]->setSubmenu(nullptr);
         }
     }
     fItems.clear();
@@ -690,10 +696,13 @@ YMenuItem * YMenu::add(YMenuItem *item) {
 }
 
 YMenuItem * YMenu::add(YMenuItem *item, const char *icons) {
-    //YIcon *icon = 0;
-    ref<YIcon> icon = YIcon::getIcon(icons);
-    if (icon->isCached() && item) item->setIcon(icon);
-    if (item) fItems.append(item);
+    if (item) {
+        ref<YIcon> icon(YIcon::getIcon(icons));
+        if (icon != null && icon->isCached()) {
+            item->setIcon(icon);
+        }
+        fItems.append(item);
+    }
     return item;
 }
 
@@ -709,7 +718,7 @@ YMenuItem * YMenu::addSorted(YMenuItem *item, bool duplicates, bool ignoreCase)
             fItems.insert(i, item);
             return item;
         } else {
-            return 0;
+            return nullptr;
         }
     }
     if (item) fItems.append(item);
@@ -719,24 +728,24 @@ YMenuItem * YMenu::addSorted(YMenuItem *item, bool duplicates, bool ignoreCase)
 YMenuItem *YMenu::findAction(YAction action) {
     for (int i = 0; i < itemCount(); i++)
         if (action == getItem(i)->getAction()) return getItem(i);
-    return 0;
+    return nullptr;
 }
 
 YMenuItem *YMenu::findSubmenu(const YMenu *sub) {
     for (int i = 0; i < itemCount(); i++)
         if (sub == getItem(i)->getSubmenu()) return getItem(i);
-    return 0;
+    return nullptr;
 }
 
-YMenuItem *YMenu::findName(const ustring &name, const int first) {
+YMenuItem *YMenu::findName(const mstring &name, const int first) {
     if (name != null)
         for (int i = first; i < itemCount(); i++) {
-            ustring iname = getItem(i)->getName();
+            mstring iname = getItem(i)->getName();
             if (iname != null && iname.equals(name))
                 return getItem(i);
     }
 
-    return 0;
+    return nullptr;
 }
 
 int YMenu::findFirstLetRef(char firstLet, const int first, const int ignCase) {
@@ -744,7 +753,7 @@ int YMenu::findFirstLetRef(char firstLet, const int first, const int ignCase) {
         firstLet = ASCII::toUpper(firstLet);
     for (int i = first; i < itemCount(); i++) {
         YMenuItem *temp = getItem(i);
-        ustring iLetterRef = temp->getName();
+        mstring iLetterRef = temp->getName();
         if (iLetterRef != null) {
             int iLetter = iLetterRef.charAt(0);
             if (ignCase)
@@ -998,10 +1007,10 @@ void YMenu::paintItem(Graphics &g, const int i, const int l, const int t, const
     int const fontBaseLine(menuFont->ascent());
 
     YMenuItem *mitem = getItem(i);
-    ustring name = mitem->getName();
-    ustring param = mitem->getParam();
+    mstring name = mitem->getName();
+    mstring param = mitem->getParam();
 
-    if (mitem->getName() == null && mitem->getSubmenu() == 0) {
+    if (mitem->getName() == null && mitem->getSubmenu() == nullptr) {
         if (draw && t + 4 >= minY && t <= maxY)
             drawSeparator(g, 1, t, width() - 2);
     } else {
@@ -1146,7 +1155,7 @@ void YMenu::paintItem(Graphics &g, const int i, const int l, const int t, const
                     g.setColor(fg);
                     g.drawChars(param,
                                 paramPos + delta, baseLine);
-                } else if (mitem->getSubmenu() != 0) {
+                } else if (mitem->getSubmenu() != nullptr) {
                     if (mitem->getAction() != actionNull) {
                         g.setColor(menuBg);
                         if (false) {
@@ -1234,8 +1243,8 @@ void YMenu::paint(Graphics &g, const YRect &r1) {
 void YMenu::hideSubmenu() {
     if (fPopup)
         fPopup->popdown();
-    fPopup = 0;
-    fPopupActive = 0;
+    fPopup = nullptr;
+    fPopupActive = nullptr;
     submenuItem = -1;
 }
 
