@@ -15,6 +15,13 @@
 #include <X11/Xft/Xft.h>
 #endif
 
+#ifdef CONFIG_I18N
+#include <wctype.h>
+#endif
+
+#define utf8ellipsis "\xe2\x80\xa6"
+const unsigned utf32ellipsis = 0x2026;
+
 static inline Display* display()  { return xapp->display(); }
 
 /******************************************************************************/
@@ -31,7 +38,7 @@ Graphics::Graphics(YWindow & window,
     rDepth = (window.depth() ? window.depth() : xapp->depth());
     gc = XCreateGC(display(), drawable(), vmask, gcv);
 #ifdef CONFIG_XFREETYPE
-    fXftDraw = 0;
+    fXftDraw = nullptr;
 #endif
 }
 
@@ -47,7 +54,7 @@ Graphics::Graphics(YWindow & window):
     XGCValues gcv; gcv.graphics_exposures = False;
     gc = XCreateGC(display(), drawable(), GCGraphicsExposures, &gcv);
 #ifdef CONFIG_XFREETYPE
-    fXftDraw = 0;
+    fXftDraw = nullptr;
 #endif
 }
 
@@ -63,7 +70,7 @@ Graphics::Graphics(ref<YPixmap> pixmap, int x_org, int y_org):
     XGCValues gcv; gcv.graphics_exposures = False;
     gc = XCreateGC(display(), drawable(), GCGraphicsExposures, &gcv);
 #ifdef CONFIG_XFREETYPE
-    fXftDraw = 0;
+    fXftDraw = nullptr;
 #endif
 }
 
@@ -77,7 +84,7 @@ Graphics::Graphics(Drawable drawable, unsigned w, unsigned h, unsigned depth,
 {
     gc = XCreateGC(display(), drawable, vmask, gcv);
 #ifdef CONFIG_XFREETYPE
-    fXftDraw = 0;
+    fXftDraw = nullptr;
 #endif
 }
 
@@ -91,7 +98,7 @@ Graphics::Graphics(Drawable drawable, unsigned w, unsigned h, unsigned depth):
     XGCValues gcv; gcv.graphics_exposures = False;
     gc = XCreateGC(display(), drawable, GCGraphicsExposures, &gcv);
 #ifdef CONFIG_XFREETYPE
-    fXftDraw = 0;
+    fXftDraw = nullptr;
 #endif
 }
 
@@ -107,7 +114,7 @@ Graphics::~Graphics() {
 #ifdef CONFIG_XFREETYPE
     if (fXftDraw) {
         XftDrawDestroy(fXftDraw);
-        fXftDraw = 0;
+        fXftDraw = nullptr;
     }
 #endif
 }
@@ -152,19 +159,19 @@ void Graphics::clearArea(int x, int y, unsigned w, unsigned h)
     setFunction(GXcopy);
 }
 
-void Graphics::copyArea(const int x, const int y,
-                        const unsigned width, const unsigned height,
-                        const int dx, const int dy)
+void Graphics::copyArea(int x, int y,
+                        unsigned width, unsigned height,
+                        int dx, int dy)
 {
     XCopyArea(display(), drawable(), drawable(), gc,
               x - xOrigin, y - yOrigin, width, height,
               dx - xOrigin, dy - yOrigin);
 }
 
-void Graphics::copyDrawable(Drawable const d,
-                            const int x, const int y,
-                            const unsigned w, const unsigned h,
-                            const int dx, const int dy)
+void Graphics::copyDrawable(Drawable d,
+                            int x, int y,
+                            unsigned w, unsigned h,
+                            int dx, int dy)
 {
     if (d == None)
         return;
@@ -185,9 +192,9 @@ void Graphics::copyPixmap(ref<YPixmap> p, int dx, int dy) {
 }
 
 void Graphics::copyPixmap(ref<YPixmap> p,
-                          const int x, const int y,
-                          const unsigned w, const unsigned h,
-                          const int dx, const int dy)
+                          int x, int y,
+                          unsigned w, unsigned h,
+                          int dx, int dy)
 {
     if (p == null)
         return;
@@ -218,19 +225,6 @@ void Graphics::drawLine(int x1, int y1, int x2, int y2) {
               x2 - xOrigin, y2 - yOrigin);
 }
 
-void Graphics::drawLines(XPoint *points, int n, int mode) {
-    int n1 = (mode == CoordModeOrigin) ? n : 1;
-    for (int i = 0; i < n1; i++) {
-        points[i].x -= xOrigin;
-        points[i].y -= yOrigin;
-    }
-    XDrawLines(display(), drawable(), gc, points, n, mode);
-    for (int i = 0; i < n1; i++) {
-        points[i].x += xOrigin;
-        points[i].y += yOrigin;
-    }
-}
-
 void Graphics::drawSegments(XSegment *segments, int n) {
     for (int i = 0; i < n; i++) {
         segments[i].x1 -= xOrigin;
@@ -271,10 +265,9 @@ void Graphics::drawArc(int x, int y, unsigned width, unsigned height, int a1, in
 
 /******************************************************************************/
 
-void Graphics::drawChars(const ustring &s, int x, int y) {
+void Graphics::drawChars(mstring s, int x, int y) {
     if (fFont != null && s != null) {
-        cstring cs(s);
-        fFont->drawGlyphs(*this, x, y, cs.c_str(), cs.c_str_len());
+        fFont->drawGlyphs(*this, x, y, s.c_str(), s.length());
     }
 }
 
@@ -287,77 +280,78 @@ void Graphics::drawString(int x, int y, char const * str) {
     drawChars(str, 0, int(strlen(str)), x, y);
 }
 
+/**
+ * Draw a string but limit its width. If overlong, truncate and show optional
+ * ... character.
+ */
 void Graphics::drawStringEllipsis(int x, int y, const char *str, int maxWidth) {
-    int const len(strlen(str));
-    int const w = (fFont != null) ? fFont->textWidth(str, len) : 0;
+    const int len(strlen(str));
 
-    if (fFont == null || w <= maxWidth) {
+    if (fFont == null || fFont->textWidth(str, len) <= maxWidth) {
         drawChars(str, 0, len, x, y);
-    } else {
-        int maxW = 0;
-        if (!showEllipsis)
-            maxW = (maxWidth);
-        else
-            maxW = (maxWidth - fFont->textWidth("...", 3));
+        return;
+    }
 
-        int l(0), w(0);
-        int sl(0), sw(0);
+    auto ellipsis = showEllipsis && fFont->supports(utf32ellipsis) ?
+            utf8ellipsis : "...";
 
+    if (showEllipsis)
+        maxWidth -= fFont->textWidth(ellipsis, 3);
+
+    int rawPos(0), drawPos(0), trimLen(0), trimWid(0);
+
+    if (maxWidth > 0) {
+        while (rawPos < len) {
+            int glyphLen, glyphWidth;
 #ifdef CONFIG_I18N
-        if (multiByte) mblen(NULL, 0);
-#endif
-
-        if (maxW > 0) {
-            while (l < len) {
-                int nc, wc;
-#ifdef CONFIG_I18N
-                if (multiByte) {
-                    nc = mblen(str + l, size_t(len - l));
-                    if (nc < 1) { // bad things
-                        l++;
-                        continue;
-                    }
-                    wc = fFont->textWidth(str + l, nc);
-                } else
-#endif
-                {
-                    nc = 1;
-                    wc = fFont->textWidth(str + l, 1);
+            wchar_t wc;
+            if (multiByte) {
+                glyphLen = mbtowc(&wc, str + rawPos, size_t(len - rawPos));
+                if (glyphLen < 1) { // bad things
+                    rawPos++;
+                    continue;
                 }
-
-                if (w + wc < maxW) {
-                    if (1 == nc && ASCII::isWhiteSpace(str[l]))
-                    {
-                        sl+= nc;
-                        sw+= wc;
-                    } else {
-                        sl =
-                            sw = 0;
-                    }
-
-                    l+= nc;
-                    w+= wc;
-                } else
-                    break;
+                glyphWidth = fFont->textWidth(str + rawPos, glyphLen);
+            } else
+#endif
+            {
+                glyphLen = 1;
+                glyphWidth = fFont->textWidth(str + rawPos, 1);
             }
-        }
 
-        l -= sl;
-        w -= sw;
+            if (drawPos + glyphWidth >= maxWidth)
+                break;
+            bool isSpace =
+#ifdef CONFIG_I18N
+                multiByte ?
+                    !iswprint(wc) :
+#endif
+                    (1 == glyphLen && ASCII::isWhiteSpace(str[rawPos]));
 
-        if (l > 0)
-            drawChars(str, 0, l, x, y);
-        if (showEllipsis) {
-            if (l < len)
-                drawChars("...", 0, 3, x + w, y);
+            if (isSpace)
+            {
+                trimLen += glyphLen;
+                trimWid += glyphWidth;
+            } else {
+                trimLen = trimWid = 0;
+            }
+
+            rawPos += glyphLen;
+            drawPos += glyphWidth;
         }
+        // chop off trailing whitespace
+        rawPos -= trimLen;
+        drawPos -= trimWid;
+    }
+
+    if (rawPos > 0)
+        drawChars(str, 0, rawPos, x, y);
+
+    if (showEllipsis && rawPos < len) {
+        drawChars(ellipsis, 0, 3, x + drawPos, y);
     }
 }
 
-void Graphics::drawStringEllipsis(int x, int y, const ustring &str, int maxWidth) {
-    cstring cs(str);
-    return drawStringEllipsis(x, y, cs.c_str(), maxWidth);
-}
 
 void Graphics::drawCharUnderline(int x, int y, const char *str, int charPos) {
 /// TODO #warning "FIXME: don't mess with multibyte here, use a wide char"
@@ -367,7 +361,7 @@ void Graphics::drawCharUnderline(int x, int y, const char *str, int charPos) {
     int c = 0, cp = 0;
 
 #ifdef CONFIG_I18N
-    if (multiByte) mblen(NULL, 0);
+    if (multiByte) mblen(nullptr, 0);
 #endif
     while (c <= len && cp <= charPos + 1) {
         if (charPos == cp) {
@@ -403,11 +397,6 @@ void Graphics::drawCharUnderline(int x, int y, const char *str, int charPos) {
         drawLine(x + left, y + 2, x + right, y + 2);
 }
 
-void Graphics::drawCharUnderline(int x, int y, const ustring &str, int charPos) {
-    cstring cs(str);
-    return drawCharUnderline(x, y, cs.c_str(), charPos);
-}
-
 void Graphics::drawStringMultiline(int x, int y, const char *str) {
     unsigned const tx(x + fFont->multilineTabPos(str));
 
@@ -436,11 +425,6 @@ void Graphics::drawStringMultiline(int x, int y, const char *str) {
         drawChars(str, 0, int(strlen(str)), x, y);
 }
 
-void Graphics::drawStringMultiline(int x, int y, const ustring &str) {
-    cstring cs(str);
-    return drawStringMultiline(x, y, cs.c_str());
-}
-
 /******************************************************************************/
 
 void Graphics::fillRect(int x, int y, unsigned width, unsigned height) {
@@ -460,8 +444,8 @@ void Graphics::fillRects(XRectangle *rects, int n) {
     }
 }
 
-void Graphics::fillPolygon(XPoint *points, int const n, int const shape,
-                           int const mode)
+void Graphics::fillPolygon(XPoint *points, int n, int shape,
+                           int mode)
 {
     int n1 = (mode == CoordModeOrigin) ? n : 1;
 
@@ -532,7 +516,7 @@ void Graphics::setFunction(int function) {
 
 /******************************************************************************/
 
-void Graphics::drawImage(ref<YImage> img, int const x, int const y) {
+void Graphics::drawImage(ref<YImage> img, int x, int y) {
     drawImage(img, 0, 0, img->width(), img->height(), x, y);
 }
 
@@ -560,12 +544,12 @@ void Graphics::drawImage(ref<YImage> img, int x, int y, unsigned w, unsigned h,
     }
 }
 
-void Graphics::drawPixmap(ref<YPixmap> pix, int const x, int const y) {
+void Graphics::drawPixmap(ref<YPixmap> pix, int x, int y) {
     drawPixmap(pix, 0, 0, pix->width(), pix->height(), x, y);
 }
 
-void Graphics::drawPixmap(ref<YPixmap> pix, int const sx, int const sy,
-        const unsigned w, const unsigned h, const int dx, const int dy) {
+void Graphics::drawPixmap(ref<YPixmap> pix, int sx, int sy,
+                          unsigned w, unsigned h, int dx, int dy) {
     Pixmap pixmap(pix->pixmap(rdepth()));
     if (pixmap == None) {
         tlog("Graphics::%s: attempt to draw pixmap 0x%lx of depth %d with gc of depth %d\n",
@@ -581,7 +565,7 @@ void Graphics::drawPixmap(ref<YPixmap> pix, int const sx, int const sy,
                   sx, sy, w, h, dx - xOrigin, dy - yOrigin);
 }
 
-void Graphics::drawMask(ref<YPixmap> pix, int const x, int const y) {
+void Graphics::drawMask(ref<YPixmap> pix, int x, int y) {
     if (pix->mask())
         XCopyArea(display(), pix->mask(), drawable(), gc,
                   0, 0, pix->width(), pix->height(), x - xOrigin, y - yOrigin);
@@ -603,7 +587,7 @@ void Graphics::drawClippedPixmap(Pixmap pix, Pixmap clip,
     XFreeGC(display(), clipPixmapGC);
 }
 
-void Graphics::compositeImage(ref<YImage> img, int const sx, int const sy, unsigned w, unsigned h, int dx, int dy) {
+void Graphics::compositeImage(ref<YImage> img, int sx, int sy, unsigned w, unsigned h, int dx, int dy) {
 
     if (picture()) {
         int rx = dx;
@@ -788,6 +772,19 @@ void Graphics::drawBorderG(int x, int y, unsigned wid, unsigned hei, bool raised
     setColor(back);
 }
 
+void Graphics::drawLines(XPoint *points, int n, int mode) {
+    int n1 = (mode == CoordModeOrigin) ? n : 1;
+    for (int i = 0; i < n1; i++) {
+        points[i].x -= xOrigin;
+        points[i].y -= yOrigin;
+    }
+    XDrawLines(display(), drawable(), gc, points, n, mode);
+    for (int i = 0; i < n1; i++) {
+        points[i].x += xOrigin;
+        points[i].y += yOrigin;
+    }
+}
+
 void Graphics::drawCenteredPixmap(int x, int y, unsigned w, unsigned h, ref<YPixmap> pixmap) {
     int r = x + w;
     int b = y + h;
@@ -907,8 +904,8 @@ void Graphics::fillPixmap(ref<YPixmap> pixmap, int x, int y,
 }
 
 void Graphics::drawSurface(YSurface const & surface, int x, int y, unsigned w, unsigned h,
-                           int const sx, int const sy,
-                           const unsigned sw, const unsigned sh
+                           int sx, int sy,
+                           unsigned sw, unsigned sh
 ) {
     if (surface.gradient != null)
         drawGradient(surface.gradient, x, y, w, h, sx, sy, sw, sh);
@@ -922,8 +919,8 @@ void Graphics::drawSurface(YSurface const & surface, int x, int y, unsigned w, u
 }
 
 void Graphics::drawGradient(ref<YImage> gradient,
-                            int const x, int const y, const unsigned w, const unsigned h,
-                            int const gx, int const gy, const unsigned gw, const unsigned gh)
+                            int x, int y, unsigned w, unsigned h,
+                            int gx, int gy, unsigned gw, unsigned gh)
 {
     ref<YImage> scaled = gradient->scale(gw, gh);
     if (scaled != null)
@@ -1071,17 +1068,26 @@ void Graphics::setClipMask(Pixmap mask) {
 void Graphics::resetClip() {
     XSetClipMask(display(), gc, None);
 #ifdef CONFIG_XFREETYPE
-    XftDrawSetClip(handleXft(), 0);
+    XftDrawSetClip(handleXft(), nullptr);
 #endif
 }
 
+void Graphics::maxOpacity() {
+    if (rdepth() == 32 && xapp->alpha()) {
+        setFunction(GXor);
+        setColorPixel(0xFF000000);
+        fillRect(0, 0, rWidth, rHeight);
+        setFunction(GXcopy);
+    }
+}
+
 /******************************************************************************/
 
 void GraphicsBuffer::paint(Pixmap pixmap, const YRect& rect) {
     if (window()->handle() && window()->destroyed())
         return;
 
-    const bool clipping = false;
+    bool clipping = false;
     const int x(rect.x());
     const int y(rect.y());
     const unsigned w(rect.width());
@@ -1096,25 +1102,29 @@ void GraphicsBuffer::paint(Pixmap pixmap, const YRect& rect) {
 
     Graphics gfx(pixmap, w, h, depth);
 
-    if (clipping && fNesting == 1) {
-        if (x || y || w < window()->width() || h < window()->height()) {
+    if (fNesting == 1) {
+        if (fClipping || x || y ||
+            w < window()->width() || h < window()->height())
+        {
             XRectangle clip = { short(x), short(y),
                                (unsigned short)w, (unsigned short)h };
             gfx.setClipRectangles(&clip, 1);
+            clipping = true;
         }
-    }
 
-    if (fNesting == 1) {
         gfx.clearArea(x, y, w, h);
     }
 
     window()->paint(gfx, rect);
 
-    if (pixmap == fPixmap && fNesting == 1) {
-        if ( !window()->destroyed()) {
+    if (fNesting == 1) {
+        if (pixmap == fPixmap && !window()->destroyed()) {
             window()->setBackgroundPixmap(pixmap);
             window()->clearArea(x, y, w, h);
         }
+        if (clipping) {
+            gfx.resetClip();
+        }
     }
 
     fNesting -= 1;
