@@ -54,7 +54,7 @@ struct YDimension {
 
 class YFont: public virtual refcounted {
 public:
-    static ref<YFont> getFont(ustring name, ustring xftFont, bool antialias = true);
+    static ref<YFont> getFont(mstring name, mstring xftFont, bool antialias = true);
 
     virtual ~YFont() {}
 
@@ -62,16 +62,16 @@ public:
     virtual int height() const { return ascent() + descent(); }
     virtual int descent() const = 0;
     virtual int ascent() const = 0;
-    virtual int textWidth(const ustring &s) const = 0;
+    virtual int textWidth(mstring s) const = 0;
     virtual int textWidth(char const * str, int len) const = 0;
 
     virtual void drawGlyphs(class Graphics & graphics, int x, int y,
                             char const * str, int len) = 0;
+    virtual bool supports(unsigned ucs4char) { return ucs4char <= 255; }
 
     int textWidth(char const * str) const;
     int multilineTabPos(char const * str) const;
     YDimension multilineAlloc(char const * str) const;
-    YDimension multilineAlloc(const ustring &str) const;
 };
 
 /******************************************************************************/
@@ -98,14 +98,14 @@ public:
 
     void clear();
     void clearArea(int x, int y, unsigned w, unsigned h);
-    void copyArea(const int x, const int y, const unsigned width, const unsigned height,
-                  const int dx, const int dy);
-    void copyDrawable(const Drawable d, const int x, const int y,
-                      const unsigned w, const unsigned h, const int dx, const int dy);
+    void copyArea(int x, int y, unsigned width, unsigned height,
+                  int dx, int dy);
+    void copyDrawable(Drawable d, int x, int y,
+                      unsigned w, unsigned h, int dx, int dy);
     void copyImage(ref<YImage> image, int x, int y);
     void copyPixmap(ref<YPixmap> p, int dx, int dy);
-    void copyPixmap(ref<YPixmap> p, const int x, const int y,
-                     const unsigned w, const unsigned h, const int dx, const int dy);
+    void copyPixmap(ref<YPixmap> p, int x, int y,
+                     unsigned w, unsigned h, int dx, int dy);
 
     void drawPoint(int x, int y);
     void drawLine(int x1, int y1, int x2, int y2);
@@ -116,30 +116,26 @@ public:
     void drawArc(int x, int y, unsigned width, unsigned height, int a1, int a2);
     void drawArrow(YDirection direction, int x, int y, unsigned size, bool pressed = false);
 
-    void drawChars(const ustring &s, int x, int y);
+    void drawChars(mstring s, int x, int y);
     void drawChars(char const * data, int offset, int len, int x, int y);
     void drawCharUnderline(int x, int y, char const * str, int charPos);
 
-    void drawCharUnderline(int x, int y, const ustring &str, int charPos);
-
     void drawString(int x, int y, char const * str);
     void drawStringEllipsis(int x, int y, char const * str, int maxWidth);
-    void drawStringEllipsis(int x, int y, const ustring &str, int maxWidth);
     void drawStringMultiline(int x, int y, char const * str);
-    void drawStringMultiline(int x, int y, const ustring &str);
 
-    void drawPixmap(ref<YPixmap> pix, int const x, int const y);
-    void drawPixmap(ref<YPixmap> pix, int const x, int const y, unsigned w, unsigned h, int dx, int dy);
-    void drawImage(ref<YImage> pix, int const x, int const y);
-    void drawImage(ref<YImage> pix, int const x, int const y, unsigned w, unsigned h, int dx, int dy);
-    void compositeImage(ref<YImage> pix, int const x, int const y, unsigned w, unsigned h, int dx, int dy);
-    void drawMask(ref<YPixmap> pix, int const x, int const y);
+    void drawPixmap(ref<YPixmap> pix, int x, int y);
+    void drawPixmap(ref<YPixmap> pix, int x, int y, unsigned w, unsigned h, int dx, int dy);
+    void drawImage(ref<YImage> pix, int x, int y);
+    void drawImage(ref<YImage> pix, int x, int y, unsigned w, unsigned h, int dx, int dy);
+    void compositeImage(ref<YImage> pix, int x, int y, unsigned w, unsigned h, int dx, int dy);
+    void drawMask(ref<YPixmap> pix, int x, int y);
     void drawClippedPixmap(Pixmap pix, Pixmap clip,
                            int x, int y, unsigned w, unsigned h, int toX, int toY);
     void fillRect(int x, int y, unsigned width, unsigned height);
     void fillRects(XRectangle * rects, int n);
-    void fillPolygon(XPoint * points, int const n, int const shape,
-                     int const mode);
+    void fillPolygon(XPoint * points, int n, int shape,
+                     int mode);
     void fillArc(int x, int y, unsigned width, unsigned height, int a1, int a2);
     void setColor(YColor aColor);
     void setColorPixel(unsigned long pixel);
@@ -163,16 +159,16 @@ public:
                     int sx = 0, int sy = 0);
 
     void drawSurface(YSurface const & surface, int x, int y, unsigned w, unsigned h,
-                     int const sx, int const sy, const unsigned sw, const unsigned sh);
+                     int sx, int sy, unsigned sw, unsigned sh);
     void drawSurface(YSurface const & surface, int x, int y, unsigned w, unsigned h) {
         drawSurface(surface, x, y, w, h, 0, 0, w, h);
     }
 
     void drawGradient(ref<YImage> gradient,
-                      int const x, int const y, const unsigned w, const unsigned h,
-                      int const gx, int const gy, const unsigned gw, const unsigned gh);
+                      int x, int y, const unsigned w, const unsigned h,
+                      int gx, int gy, unsigned gw, unsigned gh);
     void drawGradient(ref<YImage> gradient,
-                      int const x, int const y, const unsigned w, const unsigned h) {
+                      int x, int y, unsigned w, unsigned h) {
         drawGradient(gradient, x, y, w, h, 0, 0, w, h);
     }
 
@@ -198,6 +194,8 @@ public:
     void setClipRectangles(XRectangle *rect, int count);
     void setClipMask(Pixmap mask = None);
     void resetClip();
+    void maxOpacity();
+
 private:
     Drawable fDrawable;
     GC gc;
@@ -210,6 +208,9 @@ private:
     Picture fPicture;
     int xOrigin, yOrigin;
     unsigned rWidth, rHeight, rDepth;
+
+    Graphics(Graphics const&) = delete;
+    Graphics& operator=(Graphics const&) = delete;
 };
 
 /******************************************************************************/
@@ -217,8 +218,9 @@ private:
 
 class GraphicsBuffer {
 public:
-    GraphicsBuffer(YWindow* ywindow) :
+    GraphicsBuffer(YWindow* ywindow, bool clipping = false) :
         fWindow(ywindow),
+        fClipping(clipping),
         fNesting(0),
         fPixmap(None),
         fDim(0, 0)
@@ -235,6 +237,7 @@ public:
 
 private:
     YWindow* fWindow;
+    bool fClipping;
     int fNesting;
     Pixmap fPixmap;
     YDimension fDim;
