@@ -1,7 +1,6 @@
-#ifndef __YPREFS_H
-#define __YPREFS_H
+#ifndef YPREFS_H
+#define YPREFS_H
 
-#include "ylib.h"
 #include "yconfig.h"
 
 XIV(bool, dontRotateMenuPointer,                true)
@@ -31,25 +30,13 @@ XIV(int, ToolTipTime,                           0)
 ///#warning "move this one back to WM"
 XIV(bool, grabRootWindow,                       true)
 
-#if defined(__linux__)
 XSV(const char *, iconPath,
-                                                "/usr/share/icons/oxygen/base:"
-                                                "/usr/share/icons/Adwaita:"
-                                                "/usr/share/icons/hicolor:"
-                                                "/usr/share/icons:"
-                                                "/usr/share/pixmaps:"
-                                                "/usr/local/share/icons/hicolor:"
-                                                )
-#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
-XSV(const char *, iconPath,
-                                                "/usr/local/share/icons/hicolor:"
                                                 "/usr/local/share/icons:"
                                                 "/usr/local/share/pixmaps:"
+                                                "/usr/share/icons:"
+                                                "/usr/share/pixmaps:"
                                                 )
-#else
-XSV(const char *, iconPath,                     0)
-#endif
-
+XSV(const char *, iconThemes,                   "*:-HighContrast")
 XSV(const char *, themeName,                    CONFIG_DEFAULT_THEME)
 XSV(const char *, xineramaPrimaryScreenName,    0)
 
