@@ -14,6 +14,7 @@ public:
         PRECONDITION(ww < INT_MAX);
         PRECONDITION(hh < INT_MAX);
     }
+    YRect(const XRectangle& r) : xx(r.x), yy(r.y), ww(r.width), hh(r.height) { }
 
     int x() const { return xx; }
     int y() const { return yy; }
@@ -58,6 +59,12 @@ public:
         return !(*this == r);
     }
 
+    void operator+=(const YRect& r) {
+        int mx = min(xx, r.xx), mw = int(max(xx + int(ww), r.xx + int(r.ww)));
+        int my = min(yy, r.yy), mh = int(max(yy + int(hh), r.yy + int(r.hh)));
+        setRect(mx, my, unsigned(mw - mx), unsigned(mh - my));
+    }
+
     int xx, yy;
     unsigned ww, hh;
 };
