@@ -59,11 +59,14 @@ public:
     bool typed(Atom type) const { return fData && type == fType; }
 
     template<class T> T* data() const { return reinterpret_cast<T*>(fData); }
-    template<class T> T* retrieve() { T* t(data<T>()); fData = 0; return t; }
+    template<class T> T* retrieve() { T* t(data<T>()); fData = nullptr; return t; }
     long operator[](int i) const { return data<long>()[i]; }
     long operator*() const { return *data<long>(); }
     template<class T> T* operator->() const { return data<T>(); }
 
+    Atom* begin() const { return data<Atom>(); }
+    Atom* end() const { return begin() + fSize; }
+
 private:
     Window fWind;
     unsigned char* fData;
@@ -86,7 +89,7 @@ public:
 
 class YXApplication: public YApplication {
 public:
-    YXApplication(int *argc, char ***argv, const char *displayName = 0);
+    YXApplication(int *argc, char ***argv, const char *displayName = nullptr);
     virtual ~YXApplication();
 
     Display * display()   const { return fDisplay; }
@@ -101,7 +104,7 @@ public:
     int displayHeight() { return DisplayHeight(display(), screen()); }
     Atom atom(const char* name) { return XInternAtom(display(), name, False); }
     void sync() { XSync(display(), False); }
-    void send(XClientMessageEvent& ev, Window win, long mask) {
+    void send(XClientMessageEvent& ev, Window win, long mask = NoEventMask) {
         XSendEvent(display(), win, False, mask,
                    reinterpret_cast<XEvent*>(&ev));
     }
@@ -118,10 +121,12 @@ public:
     void saveEventTime(const XEvent &xev);
     Time getEventTime(const char *debug) const;
 
-    int grabEvents(YWindow *win, Cursor ptr, unsigned int eventMask, int grabMouse = 1, int grabKeyboard = 1, int grabTree = 0);
-    int releaseEvents();
+    bool grabEvents(YWindow *win, Cursor ptr, unsigned long eventMask,
+                    bool grabMouse = true, bool grabKeyboard = true,
+                    bool grabTree = false);
+    bool releaseEvents();
     void handleGrabEvent(YWindow *win, XEvent &xev);
-    bool handleIdle();
+    virtual bool handleIdle();
     void handleWindowEvent(Window xwindow, XEvent &xev);
 
     void replayEvent();
@@ -142,7 +147,7 @@ public:
 
     void alert();
 
-    void setClipboardText(cstring data);
+    void setClipboardText(mstring data);
     void dropClipboard();
 
     static YCursor leftPointer;
@@ -201,12 +206,12 @@ private:
     friend class YXPoll;
     YXPoll xfd;
 
-    int fGrabTree;
+    lazy<class YClipboard> fClip;
     YWindow *fXGrabWindow;
-    int fGrabMouse;
     YWindow *fGrabWindow;
 
-    lazy<class YClipboard> fClip;
+    bool fGrabTree;
+    bool fGrabMouse;
     bool fReplayEvent;
 
     virtual bool handleXEvents();
