@@ -4,6 +4,8 @@
 #include "wmoption.h"
 #include "ytimer.h"
 #include <X11/Xutil.h>
+#include <X11/Xproto.h>
+#include <X11/extensions/Xcomposite.h>
 
 extern YColorName taskBarBg;
 
@@ -38,7 +40,7 @@ public:
 bool TrayMessage::append(const char* data, long size) {
     MSG(("TrayMessage::append"));
     if (offset < length) {
-        if (bytes == NULL) {
+        if (bytes == nullptr) {
             bytes = new char[1 + length];
         }
         long extra = min(length - offset, size);
@@ -55,20 +57,22 @@ static bool windowDestroyed(Window win) {
     return None == XGetWindowAttributes(xapp->display(), win, &attr);
 }
 
-static int getOrder(cstring title) {
+static int getOrder(mstring title) {
     WindowOption opt(title);
-    if (defOptions)
-        defOptions->mergeWindowOption(opt, title, false);
     if (hintOptions)
         hintOptions->mergeWindowOption(opt, title, true);
+    if (defOptions)
+        defOptions->mergeWindowOption(opt, title, false);
     return opt.order;
 }
 
 struct DockRequest {
     Window window;
     lazy<YTimer> timer;
-    cstring title;
-    DockRequest(Window w, YTimer* t, cstring s): window(w), timer(t), title(s) {}
+    mstring title;
+    DockRequest(Window w, YTimer* t, mstring s):
+        window(w), timer(t), title(s)
+    { }
 };
 
 class YXTrayProxy: public YWindow, private YTimerListener {
@@ -84,10 +88,14 @@ private:
     YObjectArray<DockRequest> fDockRequests;
     typedef YObjectArray<DockRequest>::IterType DockIter;
     mstring toolTip;
+    unsigned char error_code;
+    unsigned char request_code;
+    static YXTrayProxy* singleton;
 
     bool requestDock(Window win);
-    cstring fetchTitle(Window win);
     bool enableBackingStore(Window win);
+    void handleError(XErrorEvent *xerr);
+    mstring fetchTitle(Window win);
     static int dockError(Display *disp, XErrorEvent *xerr);
 
     typedef YObjectArray<TrayMessage> MessageListType;
@@ -103,15 +111,22 @@ private:
         static const char s0[] = "_NET_SYSTEM_TRAY_S";
         return 0 == strncmp(_NET_SYSTEM_TRAY_S0.str(), s0, sizeof s0 - 1);
     }
+    bool trace() const {
+        return fTray->trace();
+    }
 
     virtual bool handleTimer(YTimer *timer);
     virtual void updateToolTip();
 };
+YXTrayProxy* YXTrayProxy::singleton;
 
 YXTrayProxy::YXTrayProxy(const YAtom& atom, YXTray *tray):
     _NET_SYSTEM_TRAY_S0(atom),
-    fTray(tray)
+    fTray(tray),
+    error_code(0),
+    request_code(0)
 {
+    singleton = this;
     addStyle(wsNoExpose);
     setTitle("YXTrayProxy");
     if (isExternal()) {
@@ -122,10 +137,8 @@ YXTrayProxy::YXTrayProxy(const YAtom& atom, YXTray *tray):
         setProperty(_XA_NET_SYSTEM_TRAY_VISUAL, XA_VISUALID, visualid);
     }
 
-    XSetSelectionOwner(xapp->display(),
-                       _NET_SYSTEM_TRAY_S0,
-                       handle(),
-                       CurrentTime);
+    XSetSelectionOwner(xapp->display(), _NET_SYSTEM_TRAY_S0,
+                       handle(), CurrentTime);
 
     XClientMessageEvent xev = {};
     xev.type = ClientMessage;
@@ -142,22 +155,23 @@ YXTrayProxy::YXTrayProxy(const YAtom& atom, YXTray *tray):
 YXTrayProxy::~YXTrayProxy() {
     Window w = XGetSelectionOwner(xapp->display(), _NET_SYSTEM_TRAY_S0);
     if (w == handle()) {
-        XSetSelectionOwner(xapp->display(),
-                           _NET_SYSTEM_TRAY_S0,
-                           None,
-                           CurrentTime);
+        XSetSelectionOwner(xapp->display(), _NET_SYSTEM_TRAY_S0,
+                           None, CurrentTime);
     }
 }
 
 void YXTrayProxy::beginMessage(Window win, long msec, long size, long ident) {
-    MSG(("YXTrayProxy::beginMessage"));
+    if (trace())
+        tlog("systray begin message 0x%08lx: ms=%ld, sz=%ld, id=%ld",
+                win, msec, size, ident);
     if (messages.getCount() < MaxTrayNumMessages) {
         messages.append(new TrayMessage(win, msec, size, ident));
     }
 }
 
 void YXTrayProxy::cancelMessage(Window win, long ident) {
-    MSG(("YXTrayProxy::cancelMessage"));
+    if (trace())
+        tlog("systray cancel message 0x%08lx: id=%ld", win, ident);
     bool change = false;
     for (IterType iter = messages.reverseIterator(); ++iter; ) {
         if (iter->window == win && iter->ident == ident) {
@@ -170,7 +184,8 @@ void YXTrayProxy::cancelMessage(Window win, long ident) {
 }
 
 void YXTrayProxy::messageData(Window win, const char* bytes, long len) {
-    MSG(("YXTrayProxy::messageData"));
+    if (trace())
+        tlog("systray message data 0x%08lx: len=%ld", win, len);
     bool change = false;
     for (IterType iter = messages.reverseIterator(); ++iter; ) {
         if (iter->window == win) {
@@ -273,10 +288,10 @@ bool YXTrayProxy::enableBackingStore(Window win) {
 }
 
 bool YXTrayProxy::requestDock(Window win) {
-    cstring title(fetchTitle(win));
+    mstring title(fetchTitle(win));
     MSG(("systemTrayRequestDock 0x%lX, title \"%s\"", win, title.c_str()));
 
-    if (title == null && windowDestroyed(win)) {
+    if (title == null && (error_code == BadWindow || windowDestroyed(win))) {
         MSG(("Ignoring tray request for unknown window 0x%08lX", win));
         return false;
     }
@@ -300,18 +315,66 @@ bool YXTrayProxy::requestDock(Window win) {
     return fTray->trayRequestDock(win, title);
 }
 
-cstring YXTrayProxy::fetchTitle(Window win) {
+mstring YXTrayProxy::fetchTitle(Window win) {
+    mstring title;
     xsmart<char> name;
-    if (XFetchName(xapp->display(), win, &name) && name && name[0]) {
-    } else {
-        XTextProperty text = {};
-        if (XGetTextProperty(xapp->display(), win, &text, _XA_NET_WM_NAME))
-            name = (char *) text.value;
+    error_code = request_code = 0;
+    if (XFetchName(xapp->display(), win, &name)) {
+        title = (char *) name;
     }
-    return name && name[0] ? cstring(name) : null;
+    else if (error_code == BadWindow) {
+        return null;
+    }
+    if (title.isEmpty()) {
+        XTextProperty text = {};
+        if (XGetTextProperty(xapp->display(), win, &text, _XA_NET_WM_NAME)) {
+            title = reinterpret_cast<char *>(text.value);
+            XFree(text.value);
+        }
+        else if (error_code == BadWindow) {
+            return null;
+        }
+    }
+    if (title.isEmpty()) {
+        XClassHint hint;
+        if (XGetClassHint(xapp->display(), win, &hint)) {
+            title = hint.res_name;
+            XFree(hint.res_name);
+            XFree(hint.res_class);
+        }
+        else if (error_code == BadWindow) {
+            return null;
+        }
+    }
+    return title;
 }
 
-int YXTrayProxy::dockError(Display *disp, XErrorEvent *xerr) {
+void YXTrayProxy::handleError(XErrorEvent* e) {
+    request_code = e->request_code;
+    error_code = e->error_code;
+
+    if (trace()) {
+        Display* display = xapp->display();
+        char message[80], req[80], number[80];
+
+        snprintf(number, 80, "%d", e->request_code);
+        XGetErrorDatabaseText(display, "XRequest",
+                              number, "",
+                              req, sizeof(req));
+        if (req[0] == 0)
+            snprintf(req, 80, "[request_code=%d]", e->request_code);
+
+        if (XGetErrorText(display, e->error_code, message, 80) != Success)
+            *message = '\0';
+
+        tlog("systray %s(0x%08lx): %s", req, e->resourceid, message);
+    }
+}
+
+int YXTrayProxy::dockError(Display *disp, XErrorEvent* e) {
+    if (singleton) {
+        singleton->handleError(e);
+    }
     // Ignore bad dock attempts.
     return Success;
 }
@@ -319,12 +382,13 @@ int YXTrayProxy::dockError(Display *disp, XErrorEvent *xerr) {
 void YXTrayProxy::handleClientMessage(const XClientMessageEvent &message) {
     const Window window = message.window;
     unsigned long type = message.message_type;
-    const long opcode = message.data.l[1];
 
     if (type == _XA_NET_SYSTEM_TRAY_OPCODE) {
+        const long opcode = message.data.l[1];
         if (opcode == SYSTEM_TRAY_REQUEST_DOCK) {
             const Window dock = message.data.l[2];
             XErrorHandler old = XSetErrorHandler(dockError);
+            error_code = request_code = 0;
             requestDock(dock);
             XSetErrorHandler(old);
         }
@@ -332,30 +396,28 @@ void YXTrayProxy::handleClientMessage(const XClientMessageEvent &message) {
             const long milli = message.data.l[2];
             const long bytes = message.data.l[3];
             const long ident = message.data.l[4];
-            MSG(("systemTrayBeginMessage win=%lX, mil=%ld, len=%ld, id=%ld",
-                  window, milli, bytes, ident));
             beginMessage(window, milli, bytes, ident);
         }
         else if (opcode == SYSTEM_TRAY_CANCEL_MESSAGE) {
             const long ident = message.data.l[2];
-            MSG(("systemTrayCancelMessage win=%lX, id=%ld", window, ident));
             cancelMessage(window, ident);
         }
-        else {
-            MSG(("systemTray???Message %ld ignored", opcode));
+        else if (trace()) {
+            tlog("systray ignoring unknown opcode %ld from 0x%08lx",
+                 opcode, window);
         }
     }
     else if (type == _XA_NET_SYSTEM_TRAY_MESSAGE_DATA) {
-        MSG(("systemTrayMessageData"));
         const long length = long(sizeof message.data.b);
         messageData(window, message.data.b, length);
     }
-    else {
-        MSG(("YXTrayProxy::handleClientMessage type %lu ignored", type));
+    else if (trace()) {
+        tlog("systray unknown client message type %ld from 0x%08lx",
+             type, window);
     }
 }
 
-YXTrayEmbedder::YXTrayEmbedder(YXTray *tray, Window win, Window ldr, cstring title):
+YXTrayEmbedder::YXTrayEmbedder(YXTray *tray, Window win, Window ldr, mstring title):
     YWindow(tray),
     fVisible(false),
     fTray(tray),
@@ -383,14 +445,15 @@ YXTrayEmbedder::YXTrayEmbedder(YXTray *tray, Window win, Window ldr, cstring tit
         XCompositeRedirectWindow(xapp->display(), win, CompositeRedirectManual);
     }
 
-    fClient->infoMapped();
-    fClient->sendNotify();
-    fClient->sendActivate();
-
     if (xapp->alpha() == false)
         fClient->setParentRelative();
-    fVisible = true;
-    fClient->show();
+}
+
+void YXTrayEmbedder::realise() {
+    client()->show();
+    client()->infoMapped();
+    client()->sendNotify();
+    client()->sendActivate();
 }
 
 YXTrayEmbedder::~YXTrayEmbedder() {
@@ -405,7 +468,13 @@ YXTrayEmbedder::~YXTrayEmbedder() {
     delete fClient;
 }
 
+bool YXTrayEmbedder::trace() const {
+    return fTray->trace();
+}
+
 void YXTrayEmbedder::damagedClient() {
+    if (trace())
+        tlog("systray damage repaint 0x%08lx", fClient->handle());
     repaint();
 }
 
@@ -419,6 +488,8 @@ void YXTrayEmbedder::detach() {
         fClient->reparent(desktop, 0, 0);
         fClient->hide();
         XRemoveFromSaveSet(xapp->display(), fClient->handle());
+        if (trace())
+            tlog("systray detached dock window 0x%08lx", fClient->handle());
     }
 }
 
@@ -428,10 +499,18 @@ bool YXTrayEmbedder::destroyedClient(Window win) {
 }
 
 void YXTrayEmbedder::handleClientUnmap(Window win) {
-    fTray->showClient(win, false);
+    if (trace())
+        tlog("systray client unmaps  0x%08lx when %s",
+                win, fVisible ? "visible" : "unmapped");
+    if (fVisible)
+        fTray->showClient(win, false);
+    else if (client()->testDestroyed())
+        fTray->destroyedClient(client_handle());
 }
 
 void YXTrayEmbedder::handleClientMap(Window win) {
+    if (trace())
+        tlog("systray client map window 0x%08lx", win);
     fClient->show();
     fTray->showClient(win, true);
 }
@@ -479,6 +558,12 @@ void YXTrayEmbedder::handleConfigureRequest(const XConfigureRequestEvent &config
     fTray->handleConfigureRequest(configureRequest);
 }
 
+void YXTrayEmbedder::handleClientMessage(const XClientMessageEvent& message)
+{
+    if (trace())
+        logClientMessage(message);
+}
+
 void YXTrayEmbedder::handleMapRequest(const XMapRequestEvent &mapRequest) {
     fClient->show();
     fTray->showClient(mapRequest.window, true);
@@ -490,12 +575,14 @@ YXTray::YXTray(YXTrayNotifier *notifier,
                YWindow *aParent,
                bool drawBevel):
     YWindow(aParent),
-    fTrayProxy(0),
+    fTrayProxy(nullptr),
     fNotifier(notifier),
     fLocked(false),
     fRunProxy(internal == false),
+    fTrace(YTrace::traces("systray")),
     fDrawBevel(drawBevel)
 {
+    DBG { fTrace = true; }
     addStyle(wsNoExpose);
     setTitle("YXTray");
     setParentRelative();
@@ -504,7 +591,7 @@ YXTray::YXTray(YXTrayNotifier *notifier,
 }
 
 YXTray::~YXTray() {
-    delete fTrayProxy; fTrayProxy = 0;
+    delete fTrayProxy; fTrayProxy = nullptr;
 }
 
 void YXTray::getScaleSize(unsigned& w, unsigned& h)
@@ -530,17 +617,20 @@ Window YXTray::getLeader(Window win) {
     return prop ? *prop : None;
 }
 
-bool YXTray::trayRequestDock(Window win, cstring title) {
-    MSG(("trayRequestDock win 0x%lX, title \"%s\"", win, title.c_str()));
+bool YXTray::trayRequestDock(Window win, mstring title) {
+    if (trace())
+        tlog("systray dock requested 0x%08lx \"%s\"", win, title.c_str());
 
     if (destroyedClient(win)) {
-        MSG(("Ignoring tray request for destroyed window 0x%08lX", win));
+        if (trace())
+            tlog("systray ignoring destroyed window 0x%08lx", win);
         return false;
     }
 
     Window leader = getLeader(win);
     if (leader == None && windowDestroyed(win)) {
-        MSG(("Ignoring tray request for failing window 0x%08lX", win));
+        if (trace())
+            tlog("systray ignoring destroyed window 0x%08lx", win);
         return false;
     }
 
@@ -549,7 +639,8 @@ bool YXTray::trayRequestDock(Window win, cstring title) {
     unsigned ww = embed->client()->width();
     unsigned hh = embed->client()->height();
 
-    MSG(("docking window size %d %d", ww, hh));
+    if (trace())
+        tlog("systray docking window 0x%08lx size %d %d", win, ww, hh);
 
     /* Workaround for GTK-Apps */
     if (ww == 0 && hh == 0)
@@ -571,8 +662,12 @@ bool YXTray::trayRequestDock(Window win, cstring title) {
              inrange(find(fRegained, iter->leader()), 0, found - 1))));
 
     iter.insert(embed);
+    embed->realise();
     updateTrayWindows();
 
+    if (trace())
+        tlog("systray docked window  0x%08lx at %d", win, iter.where());
+
     unsigned w = max(width(), 2U * fDrawBevel) + ww;
     unsigned h = max(height(), hh + fDrawBevel);
     trayUpdateGeometry(w, h, true);
@@ -581,31 +676,33 @@ bool YXTray::trayRequestDock(Window win, cstring title) {
 }
 
 bool YXTray::destroyedClient(Window win) {
-    MSG(("undock N=%d, win=%lX", fDocked.getCount(), win));
     bool change = false;
     for (IterType ec = fDocked.reverseIterator(); ++ec; ) {
-        MSG(("win %lX, %d handle %lX", win, ec.where(), ec->handle()));
         if (ec->client_handle() == win) {
-            MSG(("removing i=%d, win=%lX", ec.where(), win));
             ec.remove();
-            updateTrayWindows();
+            if (trace())
+                tlog("systray removed window 0x%08lx from %d",
+                     win, ec.where());
             change = true;
         }
     }
-    if (change)
+    if (change) {
+        updateTrayWindows();
         relayout(true);
+    }
     return change;
 }
 
-void YXTray::handleConfigureRequest(const XConfigureRequestEvent &configureRequest)
+void YXTray::handleConfigureRequest(const XConfigureRequestEvent &request)
 {
-    MSG(("tray configureRequest w=%d h=%d\n",
-        configureRequest.width, configureRequest.height));
+    if (trace())
+        tlog("systrace configure req 0x%08lx to w=%d h=%d",
+            request.window, request.width, request.height);
     bool changed = false;
     for (IterType ec = fDocked.iterator(); ++ec; ) {
-        if (ec->client_handle() == configureRequest.window) {
-            unsigned ww = configureRequest.width;
-            unsigned hh = configureRequest.height;
+        if (ec->client_handle() == request.window) {
+            unsigned ww = request.width;
+            unsigned hh = request.height;
             if (fRunProxy) {
                 /* scale icons */
                 getScaleSize(ww, hh);
@@ -685,6 +782,8 @@ void YXTray::relayout(bool enforced) {
 
     for (IterType ec = fDocked.reverseIterator(); ++ec; ) {
         if (ec->client()->destroyed()) {
+            if (trace())
+                tlog("systray destroyed %08lx", ec->client_handle());
             ec.remove();
             updateTrayWindows();
             enforced = true;
@@ -709,7 +808,8 @@ void YXTray::relayout(bool enforced) {
             countVisible++;
         }
         else {
-            MSG(("relayout sanity remove %lX", ec->client_handle()));
+            if (trace())
+                tlog("systray sanity remove %08lx", ec->client_handle());
             ec.remove();
             updateTrayWindows();
             --ec;
@@ -753,28 +853,6 @@ void YXTray::trayUpdateGeometry(unsigned w, unsigned h, bool visible) {
     }
 }
 
-bool YXTray::kdeRequestDock(Window win) {
-    if (fDocked.isEmpty())
-        return false;
-    puts("trying to dock");
-    YAtom _NET_SYSTEM_TRAY_S0("_NET_SYSTEM_TRAY_S", true);
-    Window w = XGetSelectionOwner(xapp->display(), _NET_SYSTEM_TRAY_S0);
-
-    if (w && w != handle()) {
-        XClientMessageEvent xev = {};
-        xev.type = ClientMessage;
-        xev.window = w;
-        xev.message_type = _XA_NET_SYSTEM_TRAY_OPCODE;
-        xev.format = 32;
-        xev.data.l[0] = CurrentTime;
-        xev.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
-        xev.data.l[2] = win; //fTray2->handle();
-        xapp->send(xev, w, StructureNotifyMask);
-        return true;
-    }
-    return false;
-}
-
 void YXTray::updateTrayWindows() {
     const int count = fDocked.getCount();
     Window windows[count];
@@ -790,8 +868,8 @@ void YXTray::regainTrayWindows() {
     YProperty prop(desktop, _XA_KDE_NET_SYSTEM_TRAY_WINDOWS,
                    F32, 123L, XA_WINDOW, True);
     fRegained.clear();
-    for (int i = 0; i < int(prop.size()); ++i) {
-        fRegained.append(prop[i]);
+    for (Atom window : prop) {
+        fRegained += window;
     }
 }
 
